
## prereq ####
library(data.table)
library(magrittr)
library(lubridate)
library(forcats)
library(MBC)
library(foreach)

'%+%' <- function(x,y) paste0(x,y)
source("R/0-load-check-paths.R")

source("R/functions/QDM_snc.R")

## get modis data ####

load("data/modis-auxilary.rda")
load("data/modis-55x133/modis-snc-cloudremoval-2day-interpol.rda")
dt_modis[, month := month(date)]

## get info ####

"data/rcm-histrcp-2000-2017" %>%
  list.files %>%
  grep("ECMWF", ., value = T, invert = T) %>%
  grep("ICHEC.*RACMO.*r1i", ., value = T, invert = T) %>%
  grep("DMI-HIRHAM5", ., value = T, invert = T) %>%
  
  data.table(fn = .) %>%
  .[, to_split := gsub(".rds", "", fn)] %>%
  .[, c("gcm","rcm","experiment","ensemble") := tstrsplit(to_split, "_")] %>%
  .[, to_split := NULL] -> dt_rcm_info

dt_gcmrcm <- unique(dt_rcm_info[, .(gcm, rcm)])


## run function ####

# set options
qdm_opt_ratio = T
qdm_opt_trace = 0.001

# get parallel
library(doParallel)
# registerDoParallel(parallel::detectCores() - 1)
registerDoParallel(5)


res <- foreach(
  i_gcmrcm = 1:nrow(dt_gcmrcm),
  .inorder = F) %dopar% {
  
  # set info
  dt_info_i <- merge(dt_gcmrcm[i_gcmrcm], dt_rcm_info)
  
  gcm <- dt_gcmrcm[i_gcmrcm, gcm]
  rcm <- dt_gcmrcm[i_gcmrcm, rcm]
  rcp <- "rcp85"
  
  fn_hist <- file.path("data/rcm-histrcp-2000-2017", 
                       dt_info_i[experiment == "historical", fn])
  fn_rcp_past <- file.path("data/rcm-histrcp-2000-2017",
                           dt_info_i[experiment == "rcp85", fn]) 
  fn_rcp_future <- file.path("data/rcm-rcp-2071-2100",
                             dt_info_i[experiment == "rcp85", fn]) 
  
  fn_hist_out <- file.path(path_clirsnow_rcm_bc, "2071-2100",
                           paste0(gcm, "_", rcm, "_", rcp, "_past.rds"))
  fn_rcp_out <- file.path(path_clirsnow_rcm_bc, "2071-2100",
                          paste0(gcm, "_", rcm, "_", rcp, "_future.rds"))
  
  
  print(Sys.time())
  print("Started: ")
  print(dt_gcmrcm[i_gcmrcm])
  
  # read data
  dt_hist <- readRDS(fn_hist)
  dt_rcp_past <- readRDS(fn_rcp_past)
  
  dt_past <- rbindlist(list(dt_hist[date >= "2002-10-01"],
                            dt_rcp_past[date <= "2017-12-31"]))
  dt_future <- readRDS(fn_rcp_future)
  
  
  # icells (common to rcm and modis)
  icell_rcm <- unique(dt_past$icell)
  icell_modis <- unique(dt_modis$icell)
  icell_common <- icell_modis[icell_modis %in% icell_rcm] 
  # subset to alpconv
  icell_common <- icell_common[icell_common %in% dt_alt_zone_alpconv[alpconv == "in", icell]]
  
  dt_past <- dt_past[icell %in% icell_common]
  dt_future <- dt_future[icell %in% icell_common]
  
  # prep for loop
  dt_past[, month := month(date)]
  setkey(dt_past, icell, month)
  dt_past[, snc.qdm := NA_real_]
  setnames(dt_past, "rcm.snc", "snc")
  
  dt_future[, month := month(date)]
  setkey(dt_future, icell, month)
  dt_future[, snc.qdm := NA_real_]
  setnames(dt_future, "rcm.snc", "snc")
  
  # qdm routine
  for(i_icell in icell_common){
    # print(i_icell)
    for(i_month in 1:12){
      
      l_qdm <- QDM_snc(o.c = dt_modis[icell == i_icell & month == i_month & !is.na(snc), 
                                      snc], 
                       m.c = dt_past[icell == i_icell & month == i_month, 
                                     snc], 
                       m.p = dt_future[icell == i_icell & month == i_month, 
                                       snc],
                       ratio = qdm_opt_ratio, 
                       trace = qdm_opt_trace)
      
      dt_past[icell == i_icell & month == i_month, 
              snc.qdm := l_qdm$mhat.c]
      
      dt_future[icell == i_icell & month == i_month, 
                snc.qdm := l_qdm$mhat.p]
      
    }
    
  }
  
  # save data
  saveRDS(dt_past, file = fn_hist_out)
  saveRDS(dt_future, file = fn_rcp_out)
  
  print(Sys.time())
  print("Finished: ")
  print(dt_gcmrcm[i_gcmrcm])
  
  return(NULL)
}


## EOF ####

