# make climatologies
#
# common-year averages for eval
# what for gcm driven?

library(foreach)
library(data.table)
library(magrittr)
library(fs)

## prep station data ####

# obs
dat_stn_info_all <- readRDS("data-aux/stn/00-stn-info.rds")
dat_stn_info_all[, station_file := gsub("[^[:alnum:]]+", "_", stn_name)]
dat_stn_info_all[, .N, station_file] %>% .[N > 1] # note that names are not unique!

load("data-aux/stn/07-subset2-4x-monthly.rda")


# rcm
dat_rcm <- foreach(
  fn = dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/rds/point/02-merged/"),
  .final = rbindlist
) %do% {
  dat <- readRDS(fn)
  dat[, station_file := fn %>% path_file() %>% path_ext_remove()]
}


# rcm orog
dat_rcm_orog <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/rds/point/03-orog.rds")

# rcm plausible snow
dat_rcm_th <- readRDS("data/auxilary-05-snow-max-threshold-rotpole.rds")


## remove cells with not plausible snow ####

dat_rcm_orog[, .(rcm, station_file, 
                 grid_lon = round(grid_lon, 3), grid_lat = round(grid_lat, 3))] %>% 
  merge(dat_rcm_th[snow_plausible == "yes",
                   .(rcm,
                     grid_lon = round(lon, 3), grid_lat = round(lat, 3))], 
        by = c("rcm", "grid_lon", "grid_lat")) -> dat_rcm_snow_ok

dat_rcm_p <- merge(dat_rcm, dat_rcm_snow_ok, by = c("rcm", "station_file"))


## check which stations are in the same grid cell ####

dat_rcm_orog[,
             .N, 
             .(rcm, orog, grid_lon, grid_lat)] %>% 
  .[N > 1] %>% 
  merge(dat_rcm_orog) %>% 
  merge(dat_stn_info_all[, .(station_file, elev)], by = "station_file") -> dat_mult_cell 
setkey(dat_mult_cell, rcm, grid_lon, grid_lat)

dat_mult_cell %>% summary
dat_mult_cell[, .N, station_file]


# need to be kept in mind!


# lapse rates -------------------------------------------------------------

# monthly lapse rates as in liston and elder 2006
# dt_month_lapse <- data.table(month = 1:12,
#                              lapse_rate = c(4.4, 5.9, 7.1, 7.8, 8.1, 8.2, 8.1, 8.1, 7.7, 6.8, 5.5, 4.7))

# monthly lapse rates as in rolland 2003
dt_month_lapse <- data.table(month = 1:12,
                             lapse_rate = c(4.5, 5.0, 5.8, 6.2, 6.5, 6.5, 6.5, 6.5, 6.0, 5.5, 5.0, 4.5))


## EVAL full ####

dat_stn_eval_full[, .N, .(stn_id, month)] %>% summary

# subset rcm eval to month-years in stn
dat_stn_eval_full %>% 
  merge(dat_stn_info_all[, .(station_file, stn_id, elev)]) %>% 
  merge(dat_rcm_p[experiment == "evaluation"], 
        by = c("station_file", "year", "month")) %>% 
  merge(dat_rcm_orog[, .(station_file, rcm, orog)],
        by = c("station_file", "rcm")) -> dat_merge

dat_merge_avg <- dat_merge[!is.na(snd) & !is.na(tas), 
                           .(hs = mean(hs),
                             snd = mean(snd),
                             tmean = mean(tmean),
                             tas = mean(tas),
                             n_years = .N),
                           .(station_file, stn_id, elev,
                             rcm, orog,
                             month)]  

# add delta
dat_merge_avg[, delta_alt_rcm_stn := orog - elev]
dat_merge_avg[, delta_tmean_rcm_stn := tas - tmean]


dat_merge_avg <- merge(dat_merge_avg, dt_month_lapse, by = "month")

dat_merge_avg[, delta_tmean_rcm_stn_lapse := delta_tmean_rcm_stn + delta_alt_rcm_stn * lapse_rate / 1000]
# dat_merge_avg[, delta_tmean_lapse := delta_tmean + delta_alt * 6.4 / 1000]


saveRDS(dat_merge_avg, file = "data/climatology-03-stn_eval-full.rds")



# GCM full ----------------------------------------------------------------


dat_stn_gcm_full[, .N, .(stn_id, month)] %>% summary

dat_stn_clim <- dat_stn_gcm_full[year >= 1970 & year <= 2005,
                                 .(hs = mean(hs),
                                   tmean = mean(tmean),
                                   n_stn_years = .N),
                                 .(stn_id, month)]

dat_stn_clim[, .N, keyby = n_stn_years] # -> ok min 30 years for clim
dat_stn_clim[, .N, keyby = month]

dat_rcm_clim <- dat_rcm_p[experiment == "historical" & year >= 1970 & !is.na(snd) & !is.na(tas),
                          .(snd = mean(snd),
                            tas = mean(tas)),
                          .(station_file, month, rcm, gcm)]

dat_stn_clim %>% 
  merge(dat_stn_info_all[, .(station_file, stn_id, elev)]) %>% 
  merge(dat_rcm_clim, 
        by = c("station_file", "month")) %>% 
  merge(dat_rcm_orog[, .(station_file, rcm, orog)],
        by = c("station_file", "rcm")) -> dat_merge

# add delta
dat_merge[, delta_alt_rcm_stn := orog - elev]
dat_merge[, delta_tmean_rcm_stn := tas - tmean]


dat_merge <- merge(dat_merge, dt_month_lapse, by = "month")

dat_merge[, delta_tmean_rcm_stn_lapse := delta_tmean_rcm_stn + delta_alt_rcm_stn * lapse_rate / 1000]
# dat_merge_avg[, delta_tmean_lapse := delta_tmean + delta_alt * 6.4 / 1000]


saveRDS(dat_merge, file = "data/climatology-03-stn_gcm-full.rds")



# EVAL sub ----------------------------------------------------------------


dat_stn_eval_sub[, .N, .(stn_id, month)] %>% summary

# subset rcm eval to month-years in stn
dat_stn_eval_sub %>% 
  merge(dat_stn_info_all[, .(station_file, stn_id, elev)]) %>% 
  merge(dat_rcm_p[experiment == "evaluation"], 
        by = c("station_file", "year", "month")) %>% 
  merge(dat_rcm_orog[, .(station_file, rcm, orog)],
        by = c("station_file", "rcm")) -> dat_merge

dat_merge_avg <- dat_merge[!is.na(snd) & !is.na(tas), 
                           .(hs = mean(hs),
                             snd = mean(snd),
                             tmean = mean(tmean),
                             tas = mean(tas),
                             n_years = .N),
                           .(station_file, stn_id, elev,
                             rcm, orog,
                             month)]  

# add delta
dat_merge_avg[, delta_alt_rcm_stn := orog - elev]
dat_merge_avg[, delta_tmean_rcm_stn := tas - tmean]


dat_merge_avg <- merge(dat_merge_avg, dt_month_lapse, by = "month")

dat_merge_avg[, delta_tmean_rcm_stn_lapse := delta_tmean_rcm_stn + delta_alt_rcm_stn * lapse_rate / 1000]
# dat_merge_avg[, delta_tmean_lapse := delta_tmean + delta_alt * 6.4 / 1000]


saveRDS(dat_merge_avg, file = "data/climatology-03-stn_eval-sub.rds")





# GCM sub -----------------------------------------------------------------



dat_stn_gcm_sub[, .N, .(stn_id, month)] %>% summary

dat_stn_clim <- dat_stn_gcm_sub[year >= 1970 & year <= 2005,
                                 .(hs = mean(hs),
                                   tmean = mean(tmean),
                                   n_stn_years = .N),
                                 .(stn_id, month)]

dat_stn_clim[, .N, keyby = n_stn_years] # -> hmmm, maybe subset to stn_years >=26
dat_stn_clim[, .N, keyby = month]

dat_stn_clim <- dat_stn_clim[n_stn_years >= 26]

dat_rcm_clim <- dat_rcm_p[experiment == "historical" & year >= 1970 & !is.na(snd) & !is.na(tas),
                          .(snd = mean(snd),
                            tas = mean(tas)),
                          .(station_file, month, rcm, gcm)]

dat_stn_clim %>% 
  merge(dat_stn_info_all[, .(station_file, stn_id, elev)]) %>% 
  merge(dat_rcm_clim, 
        by = c("station_file", "month")) %>% 
  merge(dat_rcm_orog[, .(station_file, rcm, orog)],
        by = c("station_file", "rcm")) -> dat_merge

# add delta
dat_merge[, delta_alt_rcm_stn := orog - elev]
dat_merge[, delta_tmean_rcm_stn := tas - tmean]


dat_merge <- merge(dat_merge, dt_month_lapse, by = "month")

dat_merge[, delta_tmean_rcm_stn_lapse := delta_tmean_rcm_stn + delta_alt_rcm_stn * lapse_rate / 1000]
# dat_merge_avg[, delta_tmean_lapse := delta_tmean + delta_alt * 6.4 / 1000]


saveRDS(dat_merge, file = "data/climatology-03-stn_gcm-sub.rds")


## EVAL sub, with SNW approx ####


dat_stn_eval_sub[, .N, .(stn_id, month)] %>% summary

# subset rcm eval to month-years in stn
dat_stn_eval_sub %>% 
  merge(dat_stn_info_all[, .(station_file, stn_id, elev)]) %>% 
  merge(dat_rcm_p[experiment == "evaluation"], 
        by = c("station_file", "year", "month")) %>% 
  merge(dat_rcm_orog[, .(station_file, rcm, orog)],
        by = c("station_file", "rcm")) -> dat_merge

dat_merge_avg <- dat_merge[!is.na(snw) & !is.na(tas), 
                           .(hs = mean(hs),
                             snw = mean(snw),
                             tmean = mean(tmean),
                             tas = mean(tas),
                             n_years = .N),
                           .(station_file, stn_id, elev,
                             rcm, orog,
                             month)]  


# add delta
dat_merge_avg[, delta_alt_rcm_stn := orog - elev]
dat_merge_avg[, delta_tmean_rcm_stn := tas - tmean]


dat_merge_avg <- merge(dat_merge_avg, dt_month_lapse, by = "month")

dat_merge_avg[, delta_tmean_rcm_stn_lapse := delta_tmean_rcm_stn + delta_alt_rcm_stn * lapse_rate / 1000]
# dat_merge_avg[, delta_tmean_lapse := delta_tmean + delta_alt * 6.4 / 1000]



saveRDS(dat_merge_avg, file = "data/climatology-04-stn_eval-sub_snw-approx.rds")





# GCM sub, with SNW approx -----------------------------------------------------------------



dat_stn_gcm_sub[, .N, .(stn_id, month)] %>% summary

dat_stn_clim <- dat_stn_gcm_sub[year >= 1970 & year <= 2005,
                                .(hs = mean(hs),
                                  tmean = mean(tmean),
                                  n_stn_years = .N),
                                .(stn_id, month)]

dat_stn_clim[, .N, keyby = n_stn_years] # -> hmmm, maybe subset to stn_years >=26
dat_stn_clim[, .N, keyby = month]

dat_stn_clim <- dat_stn_clim[n_stn_years >= 26]

dat_rcm_clim <- dat_rcm_p[experiment == "historical" & year >= 1970 & !is.na(snw) & !is.na(tas),
                          .(snw = mean(snw),
                            tas = mean(tas)),
                          .(station_file, month, rcm, gcm)]

dat_stn_clim %>% 
  merge(dat_stn_info_all[, .(station_file, stn_id, elev)]) %>% 
  merge(dat_rcm_clim, 
        by = c("station_file", "month")) %>% 
  merge(dat_rcm_orog[, .(station_file, rcm, orog)],
        by = c("station_file", "rcm")) -> dat_merge


# add delta
dat_merge[, delta_alt_rcm_stn := orog - elev]
dat_merge[, delta_tmean_rcm_stn := tas - tmean]


dat_merge <- merge(dat_merge, dt_month_lapse, by = "month")

dat_merge[, delta_tmean_rcm_stn_lapse := delta_tmean_rcm_stn + delta_alt_rcm_stn * lapse_rate / 1000]
# dat_merge_avg[, delta_tmean_lapse := delta_tmean + delta_alt * 6.4 / 1000]


saveRDS(dat_merge, file = "data/climatology-04-stn_gcm-sub_snw-approx.rds")




## EOF ####
