## create a regions raster to use in python for regional snow land line filtering ####


# source("R/0-load-check-paths.R")
library(rgdal)
library(raster)


## sample modis ####
rr_sample <- brick("/mnt/SAT2/ProjectData/SNOW_3rd_gen/aux_snow_all_alps_LAEA_dem_h18-h19.tif")

## alpine convention boundaries ####

shp_alps <- readOGR("/mnt/CEPH_BASEDATA/GIS/ALPS/BOUNDARIES/", "AC_border_utm")
shp_alps

shp_alps_laea <- spTransform(shp_alps, crs(rr_sample))

# create mask (yes, if any line touches pixel)
rr_alpconv <- rasterize(shp_alps_laea, rr_sample, getCover = T)
rr_alpconv
rr_alpconv[rr_alpconv > 0] <- 1
rr_alpconv <- rr_alpconv + 1
# plot(rr_alpconv)
# table(rr_alpconv[])

## HISTALP zones ####

shp_histalp_latlon <- readOGR("data-aux/shp-histalp-zone",
                              "Shape_CRSM")

shp_histalp_laea <- spTransform(shp_histalp_latlon,
                                crs(rr_sample))

rr_histalp_zone <- rasterize(shp_histalp_laea, rr_sample)

rr_histalp_zone_NS <- rr_histalp_zone
rr_histalp_zone_NS[rr_histalp_zone == 4] <- 1
rr_histalp_zone_NS[rr_histalp_zone == 3] <- 2


## final ####
rr_final <- rr_histalp_zone
rr_final[rr_alpconv == 1] <- rr_histalp_zone[rr_alpconv == 1] + 4
rr_final[is.na(rr_final)] <- 0
# plot(rr_final)


## write raster ####

# remove attributes
rr_histalp_zone <- setValues(raster(rr_histalp_zone), rr_histalp_zone[])
rr_histalp_zone_NS <- setValues(raster(rr_histalp_zone_NS), rr_histalp_zone_NS[])
rr_final <- setValues(raster(rr_final), rr_final[])

# 2 regions (alpconv)
writeRaster(rr_alpconv, 
            filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/MODIS_SNOW_CLOUDFREE/eurac_modis_regions_2_alpconv_laea.tif",
            overwrite = T)

# 4 regions (histalp)
writeRaster(rr_histalp_zone, 
            filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/MODIS_SNOW_CLOUDFREE/eurac_modis_regions_4_histalp_laea.tif",
            overwrite = T)

# 2 regions (histalp N-S)
writeRaster(rr_histalp_zone_NS, 
            filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/MODIS_SNOW_CLOUDFREE/eurac_modis_regions_2_histalp_northsouth_laea.tif",
            overwrite = T)


# 8 regions (HISTALP x alpconv)
writeRaster(rr_final, 
          filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/MODIS_SNOW_CLOUDFREE/eurac_modis_regions_8_histalp_alpconv_laea.tif",
          overwrite = T)



## EOF ####