# linearly interpolate snow cover area for 1 and 2 days missing
# 2002(-07-04) - 2017 (as of Oct '18)
# ----------------------------------- #

## prereq ####

library(magrittr)
library(data.table)
library(lubridate)
library(magrittr)
library(stringr)

dt_modis <- readRDS("data/modis/01-modis-snow-nosnow-clouds.rds")

## fill dates ####

dt_modis[clouds < 0.3, 
         snc := snow / (snow + no.snow)]

# remove pixels with 0 obs
# dt_modis[, all(is.na(snc)), icell] %>% 
#   .[V1 == TRUE, icell] -> icell_to_remove
# dt_modis <- dt_modis[! icell %in% icell_to_remove ]



dt_modis[, 
         .(date = seq(min(date), max(date), by = "day")),
         .(icell)] %>% 
  merge(dt_modis[, .(icell, date, snc)], all.x = T, by = c("icell", "date")) -> dt_modis_full



## interpolate linearly for gaps of 1 and 2 days ####


dt_modis_full[, 
              na.rle.id := rleid(is.na(snc)), 
              .(icell)]
dt_modis_full[, 
              na.rle.id.length := .N, 
              .(icell, na.rle.id)]
dt_modis_full[, 
              snc.fill := approx(date, snc, xout = date)$y,
              .(icell)]
dt_modis_full[is.na(snc) & na.rle.id.length > 2, 
              snc.fill := NA]


## save interpolated version ####

dt_modis <- dt_modis_full[, .(icell, date, snc = snc.fill)]

saveRDS(dt_modis, file = "data/modis/02-modis-snc.rds")


## make monthly data ####

dt_modis[, year := year(date)]
dt_modis[, month := month(date)]
dt_modis[, 
         ":="(n_not_na = sum(!is.na(snc)),
              n_days_month = days_in_month(as.Date(date))),
         .(icell, year, month)]
# take only > 0.9 months
dt_modis_month <- dt_modis[n_not_na >= 0.9 * n_days_month, 
                           .(snc_mean = mean(snc, na.rm = T),
                             snc_median = median(snc, na.rm = T)),
                           .(icell, year, month)]


saveRDS(dt_modis_month, "data/modis/03-modis-snc-monthly.rds")

## EOF ####

