## extract the 2000 daily RCM grid to rds ####
# by rcm-gcm (all vars)

# path_in <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/daily/rcm/intercomp-2000/"
# path_out <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/rds/grid/intercomp-2000/"

path_in <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/daily/rcm/intercomp-1971-2000-sub/"
path_out <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/rds/grid/intercomp-1971-2000-sub/"

# -> not working for the 30 years, too much data!

library(eurocordexr)
library(foreach)
library(mitmatmisc)

init_parallel_ubuntu()

dat_th <- readRDS("data/auxilary-03-snow-max-threshold.rds")

dat_inv <- get_inventory(path_in, add_files = T)
all_rcms <- unique(dat_inv$institute_rcm)

n_bins <- 100
l_breaks <- list(snw = seq(-0.01, 5000, length.out = n_bins),
                 snd = seq(-0.01, 8, length.out = n_bins),
                 snc = seq(-0.01, 1.00, length.out = n_bins))

l_breaks_zoom <- list(snw = seq(-0.01, 200, length.out = n_bins),
                      snd = seq(-0.01, 0.5, length.out = n_bins),
                      snc = seq(-0.01, 1.00, length.out = n_bins))


dat_all <- foreach(
  i_rcm = all_rcms,
  .final = function(x) rbindlist(x, fill = T, use.names = T)
) %do% {
  
  dat_inv_i_rcm <- dat_inv[institute_rcm == i_rcm]
  all_vars <- unique(dat_inv_i_rcm$variable)
  n_files <- length(dat_inv_i_rcm$list_files[[1]])
  
  # take one file (time) at a time
  foreach(
    i_file = 1:n_files,
    .final = function(x) rbindlist(x, fill = T, use.names = T)
  ) %do% {
    
    # loop over all variables
    dat_1file_allvars <- foreach(
      i_var = all_vars,
      .final = function(x) Reduce(f = function(x,y) merge(x, y, all = T), x = x)
    ) %dopar% {
      
      file_to_read <- dat_inv_i_rcm[variable == i_var, list_files][[1]][i_file]
      
      dat_nc <- nc_grid_to_dt(filename = file_to_read, 
                              i_var,
                              interpolate_to_standard_calendar = T, 
                              verbose = T)
      dat_nc[, rcm := i_rcm]
      dat_nc[, date := ymd(date)]
      dat_nc
      
    }
    
    
    dat_1file_allvars <- na.omit(dat_1file_allvars)
    # adjust units
    dat_1file_allvars[, snc := snc/100]
    
    dat_rcm_p <- merge(dat_1file_allvars, 
                       dat_th[snow_plausible == "yes"], 
                       by = c("rcm", "icell"))
    
    # remove all zeros
    zz <- dat_rcm_p[, lapply(.SD, function(x) x == 0), .SDcols = all_vars]
    zz_mat <- as.matrix(zz)
    zz_id <- apply(zz_mat, 1, all)
    dat_rcm_0 <- dat_rcm_p[zz_id == F]
    
    # summarize all
    for(i_var in all_vars){
      
      new_col <- paste0(i_var, "_f")
      setnames(dat_rcm_0, i_var, "current_col")
      dat_rcm_0[, c(new_col) := cut(current_col, 
                                    breaks = l_breaks[[i_var]], 
                                    labels = head(l_breaks[[i_var]], -1))]
      setnames(dat_rcm_0, "current_col", i_var)
    }
    dat_rcm_0_nn <- dat_rcm_0[,
                              .(nn = .N),
                              by = c("rcm", paste0(all_vars, "_f"))]
    dat_rcm_0_nn
    
    # summarize zoom
    for(i_var in all_vars){
      
      new_col <- paste0(i_var, "_f")
      setnames(dat_rcm_0, i_var, "current_col")
      dat_rcm_0[, c(new_col) := cut(current_col, 
                                    breaks = l_breaks_zoom[[i_var]], 
                                    labels = head(l_breaks_zoom[[i_var]], -1))]
      setnames(dat_rcm_0, "current_col", i_var)
    }
    
    dat_rcm_0[,  c("rcm", paste0(all_vars, "_f")), with = F] %>% 
      na.omit() %>% 
      .[,
        .(nn = .N),
        by = c("rcm", paste0(all_vars, "_f"))] -> dat_rcm_0_nn_zoom 
    
    
    # return both
    dat_rcm_0_nn[, zoom := "orig"]
    dat_rcm_0_nn_zoom[, zoom := "zoom"]
    rbindlist(list(dat_rcm_0_nn, dat_rcm_0_nn_zoom),
              use.names = T, fill = T)

    
    
  }
  

}

saveRDS(dat_all, file = "data/intercomp-review.rds")





