## extract the maximum daily RCM grid to rds ####
# by rcm-gcm (all vars)

library(eurocordexr)
library(foreach)

#library(mitmatmisc)
#init_parallel_ubuntu()

## maximum ####

dat_inv_max <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/daily/rcm/max-snow/", add_files = T)
all_vars <- unique(dat_inv_max$variable)

dat_max <- foreach(
  i_var = all_vars,
  .final = function(x) Reduce(f = function(x,y) merge(x, y, all = T), x = x)
) %do% {
  
  dat_i <- dat_inv_max[variable == i_var]
  
  foreach(
    i_row = 1:nrow(dat_i),
    .final = rbindlist
  ) %do% {
    
    foreach(
      i_file = dat_i[i_row, list_files][[1]],
      .final = rbindlist
    ) %do% {
      
      dat_nc <- nc_grid_to_dt(i_file, i_var, interpolate_to_standard_calendar = T, verbose = T)
      dat_nc[, gcm := dat_i[i_row, gcm]]
      dat_nc[, rcm := dat_i[i_row, institute_rcm]]
      dat_nc[, experiment := dat_i[i_row, experiment]]
      
      dat_nc
      
    }
    
  }
  
}

# max over all dates
dat_max2 <- dat_max[!{is.na(snc) & is.na(snd) & is.na(snw)}, 
                    .(max_snc = max(snc)/100,
                      max_snd = max(snd),
                      max_snw = max(snw)),
                    .(icell, gcm, rcm, experiment)]





## minimum ####

dat_inv_min <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/daily/rcm/min-snow/", add_files = T)
all_vars <- unique(dat_inv_min$variable)


dat_min <- foreach(
  i_var = all_vars,
  .final = function(x) Reduce(f = function(x,y) merge(x, y, all = T), x = x)
) %do% {
  
  dat_i <- dat_inv_min[variable == i_var]
  
  foreach(
    i_row = 1:nrow(dat_i),
    .final = rbindlist
  ) %do% {
    
    foreach(
      i_file = dat_i[i_row, list_files][[1]],
      .final = rbindlist
    ) %do% {
      
      dat_nc <- nc_grid_to_dt(i_file, i_var, interpolate_to_standard_calendar = T, verbose = T)
      dat_nc[, gcm := dat_i[i_row, gcm]]
      dat_nc[, rcm := dat_i[i_row, institute_rcm]]
      dat_nc[, experiment := dat_i[i_row, experiment]]
      
      dat_nc
      
    }
    
  }
  
}

# min over all dates
dat_min2 <- dat_min[!{is.na(snc) & is.na(snd) & is.na(snw)}, 
                    .(min_snc = min(snc)/100,
                      min_snd = min(snd),
                      min_snw = min(snw)),
                    .(icell, gcm, rcm, experiment)]




## merge min and max and save ####

dat_minmax <- merge(dat_min2, dat_max2)

saveRDS(dat_minmax, "data/snow-minmax.rds")





# maximum rotpole ---------------------------------------------------------


dat_inv_max <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/daily/rcm/max-snow-rotpole/", add_files = T)
all_vars <- unique(dat_inv_max$variable)

dat_max <- foreach(
  i_var = all_vars,
  .final = function(x) Reduce(f = function(x,y) merge(x, y, all = T), x = x)
) %do% {
  
  dat_i <- dat_inv_max[variable == i_var]
  
  foreach(
    i_row = 1:nrow(dat_i),
    .final = rbindlist
  ) %do% {
    
    foreach(
      i_file = dat_i[i_row, list_files][[1]],
      .final = rbindlist
    ) %do% {
      
      dat_nc <- nc_grid_to_dt(i_file, i_var, 
                              interpolate_to_standard_calendar = T, verbose = T)
      
      # add real alt and lon
      dat_lat <- nc_grid_to_dt(i_file, "lat")
      dat_lon <- nc_grid_to_dt(i_file, "lon")
      
      dat_nc %>% 
        merge(dat_lat[, .(icell, lat)]) %>% 
        merge(dat_lon[, .(icell, lon)]) -> dat_nc
      
      # subset to alps
      dat_nc <- dat_nc[lon > 0 & lon < 20 & lat > 40 & lat < 50]
      
      dat_nc[, gcm := dat_i[i_row, gcm]]
      dat_nc[, rcm := dat_i[i_row, institute_rcm]]
      dat_nc[, experiment := dat_i[i_row, experiment]]
      
      dat_nc
      
    }
    
  }
  
}

# max over all dates
dat_max2 <- dat_max[!{is.na(snc) & is.na(snd) & is.na(snw)}, 
                    .(max_snc = max(snc)/100,
                      max_snd = max(snd),
                      max_snw = max(snw)),
                    .(icell, lon, lat, gcm, rcm, experiment)]

saveRDS(dat_max2, file = "data/snow-max-rotpole.rds")


