# min and max snow variables
# identify unplausible grid cells

library(data.table)
library(ggplot2)
library(magrittr)

dat_minmax <- readRDS("data/snow-minmax.rds")
dat_aux <- readRDS("data/auxilary-01-modis-altzonealpconv.rds")

dat_minmax <- merge(dat_minmax, dat_aux, by = "icell")


## check which RCMs affected ####

dat_max_summary <- dat_minmax[, 
                              .(snd = max(max_snd, na.rm = T),
                                snw = max(max_snw, na.rm = T)), 
                              keyby = .(rcm)]
dat_max_summary
# ok: CCLM4-8-17, REMO2009, REMO2015, RegCM4-6
# affected: 
#   snw: ALADIN53, ALADIN63, WRF381P, KNMI-RACMO22E, RCA4
#   snd: ALADIN63, WRF331F, KNMI-RACMO22E

rcm_short_names <- readRDS("data/auxilary-04-rcm-names.rds")
dat_max_summary[, rcm_short := rcm_short_names[rcm]]
setkey(dat_max_summary, rcm_short)

library(flextable)
library(officer)
flextable(dat_max_summary,
          col_keys = c("rcm_short", "snd", "snw")) %>% 
  set_header_labels(rcm_short = "RCM",
                    snd = "SND [m]",
                    snw = "SNW") %>% 
  compose(part = "header", j = "snw", 
          value = as_paragraph("SNW [kg m", as_sup("-2"), "]")) %>% 
  set_formatter(snd = function(x) ifelse(is.infinite(x), "", sprintf("%.01f", x)),
                snw = function(x) ifelse(is.infinite(x), "", sprintf("%.0f", x))) %>% 
  autofit() -> ft
  


read_docx() %>% 
  body_add_flextable(ft) %>% 
  print(target = "table/rcm-max.docx")

## identify grid cells above threshold ####

# snd
dat_snd <- dat_minmax[!is.na(min_snd), 
           .(max_snd = max(max_snd)),
           .(icell, rcm)]
dat_snd[, max_snd_10m := ifelse(max_snd > 10, "above", "below")]

# dat_snd <- merge(dat_snd, dat_aux, by = "icell")
# ggplot(dat_snd, aes(x,y,fill=max_snd_10m))+
#   geom_raster()+
#   facet_wrap(~rcm)+
#   theme_bw()


# snw
dat_snw <- dat_minmax[!is.na(min_snw), 
                      .(max_snw = max(max_snw)),
                      .(icell, rcm)]
dat_snw[, max_snw_7000 := ifelse(max_snw > 7000, "above", "below")]

# dat_snw <- merge(dat_snw, dat_aux, by = "icell")
# ggplot(dat_snw, aes(x,y,fill=max_snw_7000))+
#   geom_raster()+
#   facet_wrap(~rcm)+
#   theme_bw()


# merge
dat_th <- merge(dat_snd, dat_snw, by = c("icell", "rcm"), all = T)
dat_th[, snow_plausible := "yes"]
dat_th[max_snd_10m == "above", snow_plausible := "no"]
dat_th[max_snw_7000 == "above", snow_plausible := "no"]

# dat_th %>% with(table(snow_plausible, max_snd_10m, useNA = "a"))
# dat_th %>% with(table(snow_plausible, max_snw_7000, max_snd_10m, useNA = "a"))

saveRDS(dat_th, "data/auxilary-03-snow-max-threshold.rds")

