# extract AMUNDSEN data from flo

library(raster)
library(data.table)
library(eurocordexr)
library(fs)
library(foreach)

# aux --------------------------------------------------------------

# align modis with flo's aggregate
# (necessary, because different grid in different version of cloudremoval)

rr_alt <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/amundsen-flo/altitude.tif")
rr_cov <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/amundsen-flo/coverage.tif")

icell_sub <- which(!is.na(rr_alt[]))

# rr_modis <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/modis_10d/modis_snc_55x133_2000-02-24.tif")
# rr_modis[icell_sub] <- 100
# mapview::mapview(list(rr_alt, rr_modis))

rr_modis <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/modis_10d/modis_snc_55x133_2000-02-24.tif")
rr_modis[] <- 1:ncell(rr_modis)
rr_modis2 <- resample(rr_modis, rr_alt, method = "ngb")
rr_modis2
# rr_modis2[icell_sub] <- 100
# mapview::mapview(list(rr_alt, rr_modis2))

dat_icell_match <- data.table(icell_amundsen = icell_sub, icell_modis = rr_modis2[icell_sub])

rr_modis <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/modis_10d/modis_snc_55x133_2000-02-24.tif")
rr_modis[dat_icell_match$icell_modis] <- 100
# mapview::mapview(list(rr_alt, rr_modis))


dat_aux_amundsen <- copy(dat_icell_match)
dat_aux_amundsen[, elev := rr_alt[icell_amundsen]]
dat_aux_amundsen[, coverage := rr_cov[icell_amundsen]]

saveRDS(dat_aux_amundsen, file = "data/auxiliary-04-amundsen.rds")

# snow th5-15 ----------------------------------------------------------------

all_files <- dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/amundsen-flo/02-comparison-th5-15/")

dat_amundsen_th <- foreach(
  i_fn = all_files,
  .final = rbindlist
) %do% {
  
  # nc_open(i_fn)
  i_fn %>% 
    path_file() %>% 
    path_ext_remove() %>% 
    strsplit("_") %>% 
    .[[1]] -> i_fn_split
  
  setNames(i_fn_split,
           c("gcm", "rcm", "experiment", "th")) %>% 
    as.list() %>% 
    as.data.table() -> dat_info
  
  dat_swe <- nc_grid_to_dt(i_fn, "swe")
  dat_swe2 <- dat_swe[!is.na(swe)]
  
  dat_scf <- nc_grid_to_dt(i_fn, "scf")
  dat_scf2 <- dat_scf[!is.na(scf)]
  
  dat_out <- cbind(dat_info,
                   merge(dat_swe2, dat_scf2, by = c("icell", "date")))
  
  dat_out
  
}

saveRDS(dat_amundsen_th, 
        file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/amundsen-01-th-comp.rds")

# snow th5 all ------------------------------------------------------------


all_files <- dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/amundsen-flo/03-full-th5/")


dat_amundsen_full <- foreach(
  i_fn = all_files,
  .final = rbindlist
) %do% {
  
  # nc_open(i_fn)
  i_fn %>% 
    path_file() %>% 
    path_ext_remove() %>% 
    strsplit("_") %>% 
    .[[1]] -> i_fn_split
  
  setNames(i_fn_split,
           c("gcm", "rcm", "experiment", "th")) %>% 
    as.list() %>% 
    as.data.table() -> dat_info
  
  dat_swe <- nc_grid_to_dt(i_fn, "swe")
  dat_swe2 <- dat_swe[!is.na(swe)]
  
  dat_scf <- nc_grid_to_dt(i_fn, "scf")
  dat_scf2 <- dat_scf[!is.na(scf)]
  
  dat_out <- cbind(dat_info,
                   merge(dat_swe2, dat_scf2, by = c("icell", "date")))
  
  dat_out
  
}


saveRDS(dat_amundsen_full, 
        file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/amundsen-02-full.rds")
