# extract bc data for AMUNDSEN data coverage

# library(raster)
library(data.table)
# library(eurocordexr)
library(fs)
library(foreach)


dat_aux_amundsen <- readRDS("data/auxiliary-04-amundsen.rds")

dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/amundsen-flo/03-full-th5/") %>% 
  path_file() %>% 
  path_ext_remove() %>% 
  tstrsplit("_") %>% 
  as.data.table -> dat_info_amundsen
setnames(dat_info_amundsen, c("gcm", "rcm", "rcp", "th"))
dat_info_amundsen[, fn_match := paste0(rcm, "_", gcm, "_", rcp)]

# loop over bc ------------------------------------------------------------

# subset to actually available models from AMUNDSEN

# QM
all_files <- dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/02_qm/")
foreach(x = all_files) %do% {
  stringr::str_subset(x, dat_info_amundsen$fn_match)
} %>% unlist -> sub_files

clim_qm <- foreach(
  i_fn = sub_files,
  .final = rbindlist
) %do% {
  
  i_fn %>% 
    path_file() %>% 
    path_ext_remove() %>% 
    strsplit("_") %>% 
    .[[1]] -> fn_info
  
  dat <- readRDS(i_fn)
  dat_sub <- dat[icell %in% dat_aux_amundsen$icell_modis]

  dat_sub[, ":="(rcm = fn_info[1],
                 gcm = fn_info[2],
                 experiment = fn_info[3],
                 fp = fn_info[6])]
  
  dat_sub
  
}


# QDM
all_files <- dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/03_qdm/")
foreach(x = all_files) %do% {
  stringr::str_subset(x, dat_info_amundsen$fn_match)
} %>% unlist -> sub_files

clim_qdm <- foreach(
  i_fn = sub_files,
  .final = rbindlist
) %do% {
  
  i_fn %>% 
    path_file() %>% 
    path_ext_remove() %>% 
    strsplit("_") %>% 
    .[[1]] -> fn_info
  
  dat <- readRDS(i_fn)
  dat_sub <- dat[icell %in% dat_aux_amundsen$icell_modis]
  
  dat_sub[, ":="(rcm = fn_info[1],
                 gcm = fn_info[2],
                 experiment = fn_info[3],
                 fp = fn_info[6])]
  
  dat_sub
  
}


# MBC
all_files <- dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/04_mbc_qdm/")
foreach(x = all_files) %do% {
  stringr::str_subset(x, dat_info_amundsen$fn_match)
} %>% unlist -> sub_files

clim_mbc <- foreach(
  i_fn = sub_files,
  .final = rbindlist
) %do% {
  
  i_fn %>% 
    path_file() %>% 
    path_ext_remove() %>% 
    strsplit("_") %>% 
    .[[1]] -> fn_info
  
  dat <- readRDS(i_fn)
  dat_sub <- dat[icell %in% dat_aux_amundsen$icell_modis]
  
  dat_sub[, ":="(rcm = fn_info[1],
                 gcm = fn_info[2],
                 experiment = fn_info[3],
                 fp = fn_info[6])]
  
  dat_sub
  
}


# combine and save --------------------------------------------------------

setnames(clim_mbc, "snc_qdm", "snc_mbc")

clim_qm %>% 
  merge(clim_qdm) %>% 
  merge(clim_mbc) -> dat_merge

saveRDS(dat_merge,
        file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/amundsen-03-bc-data.rds")
  


