## Auxilary data MODIS snow cover ####

# Layer 1: Slope
# Layer 2: Aspect
# Layer 3: Land Cover Corine
# Layer 4: Cloud Free Reference Image
# Layer 5: Altitude

library(raster)
library(data.table)
library(magrittr)
library(rgdal)
library(forcats)

source("R/functions/raster2datatable.R")

rr_aux <- brick("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif")

# # reproject to LAEA
# rr_aux <- projectRaster(rr_aux_laea,
#                         crs = sp::CRS("+proj=laea +lat_0=52 +lon_0=10 +x_0=4321000 +y_0=3210000 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs"),
#                         method = "ngb")


## cloud free reference ####

rr_cloudfree <- rr_aux[[4]]


## corine ####
# auxilary for rest, in order to set remapped 0 value to NA instead
rr_corine <- rr_aux[[3]]

## elevation ####

# get band
rr_alt_250m <- rr_aux[[5]]

# set water to NA
rr_alt_250m[rr_alt_250m == -32768] <- NA


# set remap-zeros to NA
rr_alt_250m[rr_corine == 0] <- NA

# (mask to AC)
# rr_alt_250m <- mask(rr_alt_250m, rr_mask_alps_250m)

# aggregate to CORDEX resolution
# na.rm: remove coastal pixels (sparse)
rr_alt <- aggregate(rr_alt_250m, 
                    fact = c(37, 53), 
                    fun = mean,
                    na.rm = F)

# transform into data.table
rr_alt %>% 
  raster2datatable(xy = T, icell = T, remove.na = T) -> dt_alt

setnames(dt_alt, 3, "alt")



## HISTALP zones ####

shp_histalp_latlon <- readOGR("data-aux/shp-histalp-zone",
                              "Shape_CRSM")

shp_histalp_laea <- spTransform(shp_histalp_latlon,
                                crs(rr_alt))

rr_histalp_zone <- rasterize(shp_histalp_laea, rr_alt)

dt_histalp_zone <- raster2datatable(rr_histalp_zone, 
                                    xy = F,
                                    icell = T,
                                    remove.na = T)

dt_histalp_zone[, zone := fct_recode(as.character(Id), 
                                     SW = "0",
                                     NW = "1",
                                     NE = "2",
                                     SE = "3")]

dt_histalp_zone[, Id := NULL]


# merge to altitude data
dt_alt_zone <- merge(dt_alt, dt_histalp_zone)


## add alpine convention ####
# read
shp_alps <- readOGR("/mnt/CEPH_BASEDATA/GIS/ALPS/BOUNDARIES/", "AC_border_utm")
shp_alps

## aggregated res ####

sample_modis <- rr_alt
sample_modis

shp_alps_laea <- spTransform(shp_alps, crs(sample_modis))

# create data.table
# shp_alps_laea %>% 
#   fortify %>% 
#   data.table -> dt_alps_laea

# create mask (yes, if any line touches pixel)
rr_mask_alps <- rasterize(shp_alps_laea, sample_modis, getCover = T)
rr_mask_alps
rr_mask_alps[rr_mask_alps == 0] <- NA
rr_mask_alps[!is.na(rr_mask_alps)] <- 1

rr_mask_alps %>% 
  raster2datatable(icell = T) %>% 
  .[, .(icell, alpconv = ifelse(is.na(layer), "out", "in"))] -> dt_alpconv


# merge to alt+zone
dt_alt_zone_alpconv <- merge(dt_alt_zone, dt_alpconv)

## save ####

rr_alpconv <- rr_mask_alps

# save(rr_alt, dt_alt,
#      rr_histalp_zone, dt_histalp_zone,
#      rr_alpconv, dt_alpconv,
#      dt_alt_zone, dt_alt_zone_alpconv,
#      file = "data/modis-auxilary.rda")

saveRDS(dt_alt_zone_alpconv, file = "data/auxiliary-03-modis.rds")


## EOF ####
