# get the cordex inventory to be used


# prereq

library(eurocordexr)

dat_inv <- get_inventory("/mnt/CEPH_BASEDATA/METEO/SCENARIO", add_files = T)


# # REMOVE AFTER CLEANED -------
dat_inv_temp <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/wget-snow/", add_files = T)
dat_inv <- rbind(dat_inv, dat_inv_temp)
dat_inv <- dat_inv[!duplicated(dat_inv, by = 1:9)]
# # END REMOVE ------



f_remove <- function(dat_to_remove){
  
  # remove IPSL WRF331 since not any more on ESGF (replaced by WRF381?)
  # but still available for 0.44 resolution, so let's keep it here
  # dat_to_remove <- dat_to_remove[institute_rcm != "IPSL-INERIS-WRF331F"]
  
  # remove REMO2009 SND, since not any more on ESGF
  dat_to_remove <- dat_to_remove[!{institute_rcm == "MPI-CSC-REMO2009" & variable == "snd"}]
  
  # DMI use only v2 (or v3 for NCC)
  # dat_to_remove <- dat_to_remove[! { institute_rcm == "DMI-HIRHAM5" & 
  #     {{downscale_realisation != "v2" & gcm %in% c("ICHEC-EC-EARTH", "MOHC-HadGEM2-ES")} | 
  #         {downscale_realisation != "v3" & gcm %in% c("NCC-NorESM1-M")}}
  # }]
  # also remove v1 ERA
  # dat_to_remove <- dat_to_remove[!{institute_rcm == "DMI-HIRHAM5" & 
  #     gcm == "ECMWF-ERAINT" & downscale_realisation == "v1"}]
  
  # DMI do not use v1 (only >= v2, see errata table euro-cordex)
  dat_to_remove <- dat_to_remove[! { institute_rcm == "DMI-HIRHAM5" & downscale_realisation == "v1"}]
  
  # remove HIRHAM5 NCC v2
  dat_to_remove <- dat_to_remove[! { institute_rcm == "DMI-HIRHAM5" & downscale_realisation == "v2" &
      gcm == "NCC-NorESM1-M"}]

  # multiple ensembles in ICHEC (take only r3 for SMHI, KNMI, DMI, and COSMOcr)
  dat_to_remove <- dat_to_remove[!{gcm == "ICHEC-EC-EARTH" & ensemble != "r3i1p1" & 
      institute_rcm %in% c("SMHI-RCA4", "KNMI-RACMO22E", "DMI-HIRHAM5",
                           "CLMcom-ETH-COSMO-crCLIM-v1-1")}]
  
  # multiple ensembles in MPI-M for REMO2009, only take r2
  dat_to_remove <- dat_to_remove[!{gcm == "MPI-M-MPI-ESM-LR" & ensemble != "r2i1p1" &
      institute_rcm %in% c("MPI-CSC-REMO2009", "CLMcom-ETH-COSMO-crCLIM-v1-1")}]
  
  # multiple ensembles in MPI-M for RCA4, only take r3
  dat_to_remove <- dat_to_remove[!{gcm == "MPI-M-MPI-ESM-LR" & 
      institute_rcm == "SMHI-RCA4" & ensemble != "r3i1p1"}]
  
  dat_to_remove
  
}


## orography ####

dat_inv_orog <- dat_inv[variable == "orog"]
dat_inv_orog2 <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/wget-orog/", add_files = T)


# keep only one file per RCM, since all the same anyway
dat_inv_orog_unique <- unique(rbind(dat_inv_orog,
                                    dat_inv_orog2), 
                              by = "institute_rcm")

saveRDS(dat_inv_orog_unique, file = "data/eurocordex-inventory-v02/orog.rds")



## grid ####


# ** snc only ---------------------------------------------------------------------



dat_inv_snow <- dat_inv[timefreq == "day" & variable == "snc" & experiment != "rcp45"]
dat_inv_snow <- f_remove(dat_inv_snow)

check_inventory(dat_inv_snow)

saveRDS(dat_inv_snow, "data/eurocordex-inventory-v02/snc.rds")


# ** snc, snd, snw for max snow -------------------------------------------

dat_inv_snow_max <- dat_inv[timefreq == "day" & variable %in% c("snc", "snd", "snw") &
                              experiment %in% c("historical", "evaluation")]
dat_inv_snow_max <- f_remove(dat_inv_snow_max)

check_inventory(dat_inv_snow_max)


dat_inv_snow[, .(gcm, institute_rcm, ensemble, downscale_realisation)] %>% 
  unique  %>% 
  merge(dat_inv_snow_max) -> dat_inv_snow_max_sub


saveRDS(dat_inv_snow_max_sub, "data/eurocordex-inventory-v02/snow-for-max.rds")


## EOF ####

