# get MODIS snow cover from full cloud-removal 
# aggregate to ~0.11 degrees
# save as geotiff
# ----------------------------------- #

library(raster)
library(lubridate)
library(magrittr)
library(stringr)


## prereq ####

path_in <- "/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/05_temporal_complete_max10d/"
path_out <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/modis_10d/"



# resolution aggregate factors
# hres <- res(rr)  # 231.6564
hres <- 231.6564
12245 / hres # vertical
8823 / hres # horiz @ lat=44
8209 / hres # horiz @ lat=48
(8823 + 8209)/2 / hres # mean
# -> y: 53, x : 37


# get files
all_files <- list.files(path_in)

mitmatmisc::init_parallel_ubuntu(6)

## loop over all dates (in parallel) ####
foreach(fn = all_files, 
        .inorder = F
) %dopar% {
  
  # apply working horse
  # extract date from filename
  fn %>% 
    str_extract("[0-9]{8}") %>% 
    ymd -> fn_date
  
  # check if file exist, and skip if yes
  filepath <- file.path(path_out, 
                        paste0("modis_snc_55x133_", fn_date, ".tif"))
  if(file.exists(filepath)) return(paste0("skipped (file exists already):", filepath))
  
  
  # get raster from Products
  rr <- raster(file.path(path_in, fn))
  
  # remove water
  rr_na <- rr
  rr_na[rr_na > 3 | rr == 0] <- NA
  
  # get snow fraction
  rr_temp <- rr_na == 1
  rr_snow <- aggregate(rr_temp, fact = c(37, 53), fun = mean)
  
  # get no-snow fraction
  rr_temp <- rr_na == 2
  rr_nosnow <- aggregate(rr_temp, fact = c(37, 53), fun = mean)
  
  # get clouds fraction
  rr_temp <- rr_na == 3
  rr_clouds <- aggregate(rr_temp, fact = c(37, 53), fun = mean)
  
  # all three add to 1
  # rr.sum <- rr_snow + rr_nosnow + rr_clouds
  # plot(rr.sum)
  # summary(rr.sum)
  
  # create raster brick
  rr_brick <- brick(rr_snow, rr_nosnow, rr_clouds)
  names(rr_brick) <- c("snow", "no.snow", "clouds")
  
  # save brick to file
  writeRaster(rr_brick,
              filename = filepath)
  
  return(paste0("done: ", filepath))
  
}

