# use CDO to:
#   remap selected grid inventory
#   MODIS time frame: 2000-2019
#                     and also future?
#   variables: snc, orog
#
# ------------------------- #

library(data.table)
library(magrittr)
library(fs)
library(foreach)
# get parallel
library(doParallel)
registerDoParallel(cores = 6)

# set remapping algorithm:
# NEEDS TO BE CONSTANT FOR ALL VARIABLES!!
# otherwise if bil for temp and nn for snc, then interpolation artifacts?!
# hard coded to nn, because bilinear introduces inplausibilities with constant 0 and 100 snc
# remap_type <- "remapnn" # nearest-neighbour
# remap_type <- "remapbil" # bilinear


## OROG ####


# remap function for fx variables (OROG, ...)
f_remap_fx <- function(dat_i, path_out){
  
  foreach(i = 1:nrow(dat_i),
          .inorder = F,
          .packages = c("data.table", "magrittr")
  ) %dopar% {
    
    in_file <- dat_i$list_files[i][[1]]
    out_file <- path(path_out, 
                     dat_i[i, variable], 
                     path_file(in_file))
    
    if(!dir_exists(path_dir(out_file))) dir_create(path_dir(out_file))
    if(file_exists(out_file)) return(paste0("Skipped: ", out_file))
    
    cdo_call <- paste0("cdo ", "remapnn", ",", "data/gridspec_55x133_laea",
                       " ", in_file, 
                       " ", out_file)
    system(cdo_call)
    return(paste0("Done: ", out_file))
  }
}




# orog
# f_remap_fx(dat_i = readRDS("data/eurocordex-inventory-v03/orog.rds"),
#            path_out = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_fixed/")
# 
# 



## SNC 2000-2020 & 2071-2100; daily ####

# remap function with date selection (for SNC, TAS, ...)
f_remap <- function(dat_i, path_out_daily, seldate = "20000101,20201231"){
  
  foreach(i = 1:nrow(dat_i),
          .inorder = F,
          .packages = c("data.table", "magrittr")
  ) %dopar% {
    
    # get and set filename
    files_original <- dat_i$list_files[i][[1]]
    
    # remap & monmean proc
    foreach(j = seq_along(files_original)) %do% {
      
      # daily 2000-2020
      in_file <- files_original[j]
      out_file <- path(path_out_daily, 
                       # dat_i[i, variable], 
                       path_file(in_file))
      
      if(!dir_exists(path_dir(out_file))) dir_create(path_dir(out_file))
      if(file_exists(out_file)) return(paste0("Skipped: ", out_file))
      
      cdo_call <- paste0("cdo ", "remapnn", ",", "data/gridspec_55x133_laea",
                         " ", "-seldate,", seldate,
                         " ", in_file, 
                         " ", out_file)
      system(cdo_call)
      
      return(paste0("Done: ", out_file, "\n"))
    }
  }
}



# snc
# f_remap(dat_i = readRDS("data/eurocordex-inventory-v03/snc.rds"),
#         path_out_daily = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/rcm/2000-2020/",
#         seldate = "20000101,20201231")
# 
# f_remap(dat_i = readRDS("data/eurocordex-inventory-v03/snc.rds"),
#         path_out_daily = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/rcm/2071-2100/",
#         seldate = "20710101,21001231")

f_remap(dat_i = readRDS("data/eurocordex-inventory-v03/snc.rds"),
        path_out_daily = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/rcm/2041-2070/",
        seldate = "20410101,20701231")

# max snow (for implausible snow cells) -----------------------------------

# remap function with tim* aggregation
f_remap_tim <- function(dat_i, path_out_daily, agg){
  
  foreach(i = 1:nrow(dat_i),
          .inorder = F,
          .packages = c("data.table", "magrittr")
  ) %dopar% {
    
    # get and set filename
    files_original <- dat_i$list_files[i][[1]]
    
    # remap & monmean proc
    foreach(j = seq_along(files_original)) %do% {
      
      # daily
      in_file <- files_original[j]
      out_file <- path(path_out_daily, 
                       dat_i[i, variable], 
                       path_file(in_file))
      
      if(!dir_exists(path_dir(out_file))) dir_create(path_dir(out_file))
      if(file_exists(out_file)) return(paste0("Skipped: ", out_file))
      
      cdo_call <- paste0("cdo ", "remapnn", ",", "data/gridspec_55x133_laea",
                         " ", "-tim", agg,
                         " ", in_file, 
                         " ", out_file)
      system(cdo_call)
      return(paste0("Done: ", out_file))
      
    }
  }
}

# snc, snd, snw
# f_remap_tim(dat_i = readRDS("data/eurocordex-inventory-v02/snow-for-max.rds"),
#             path_out_daily = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/rcm/max-snow/",
#             agg = "max")
# f_remap_tim(dat_i = readRDS("data/eurocordex-inventory/grid/intercomp-snc-snd-snw.rds"),
#             path_out_daily = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_evaluation/nc/daily/rcm/min-snow/",
#             agg = "min")


## EOF ####