# extract the maximum daily RCM grid to rds 
# by rcm-gcm (all vars)

library(eurocordexr)
library(foreach)

#library(mitmatmisc)
#init_parallel_ubuntu()

## get data: maximum ####

dat_inv_max <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/rcm/max-snow/", 
                             add_files = T)
all_vars <- unique(dat_inv_max$variable)

dat_max <- foreach(
  i_var = all_vars,
  .final = function(x) Reduce(f = function(x,y) merge(x, y, all = T), x = x)
) %do% {
  
  dat_i <- dat_inv_max[variable == i_var]
  
  foreach(
    i_row = 1:nrow(dat_i),
    .final = rbindlist
  ) %do% {
    
    foreach(
      i_file = dat_i[i_row, list_files][[1]],
      .final = rbindlist
    ) %do% {
      
      dat_nc <- nc_grid_to_dt(i_file, i_var, interpolate_to_standard_calendar = T, verbose = T)
      dat_nc[, gcm := dat_i[i_row, gcm]]
      dat_nc[, rcm := dat_i[i_row, institute_rcm]]
      dat_nc[, experiment := dat_i[i_row, experiment]]
      
      dat_nc
      
    }
    
  }
  
}

# max over all dates
dat_max2 <- dat_max[!{is.na(snc) & is.na(snd) & is.na(snw)}, 
                    .(max_snc = max(snc)/100,
                      max_snd = max(snd),
                      max_snw = max(snw)),
                    .(icell, gcm, rcm, experiment)]


## check which RCMs affected ####

rcm_short_names <- readRDS("data/auxiliary-01-rcm-names.rds")
dat_max2[, rcm_short := rcm_short_names[rcm]]

dat_max_summary <- dat_max2[, 
                            .(snd = max(max_snd, na.rm = T),
                              snw = max(max_snw, na.rm = T)), 
                            keyby = .(rcm_short)]
dat_max_summary
# ok: CCLM4-8-17, HIRHAM5, REMO2009, REMO2015, RegCM4-6
# affected: 
#   snw: ALADIN53, ALADIN63, RACMO22E, RCA4, WRF381P
#   snd: ALADIN63, RACMO22E, WRF331F



library(flextable)
library(officer)
flextable(dat_max_summary,
          col_keys = c("rcm_short", "snd", "snw")) %>% 
  set_header_labels(rcm_short = "RCM",
                    snd = "SND [m]",
                    snw = "SNW") %>% 
  compose(part = "header", j = "snw", 
          value = as_paragraph("SNW [kg m", as_sup("-2"), "]")) %>% 
  set_formatter(snd = function(x) ifelse(is.infinite(x), "", sprintf("%.01f", x)),
                snw = function(x) ifelse(is.infinite(x), "", sprintf("%.0f", x))) %>% 
  autofit() -> ft



read_docx() %>% 
  body_add_flextable(ft) %>% 
  print(target = "table/rcm-max.docx")

## identify grid cells above threshold ####

# snd
dat_snd <- dat_max2[!is.na(max_snd), 
                    .(max_snd = max(max_snd)),
                    .(icell, rcm, rcm_short)]
dat_snd[, max_snd_10m := ifelse(max_snd > 10, "above", "below")]

# dat_snd <- merge(dat_snd, dat_aux, by = "icell")
# ggplot(dat_snd, aes(x,y,fill=max_snd_10m))+
#   geom_raster()+
#   facet_wrap(~rcm)+
#   theme_bw()


# snw
dat_snw <- dat_max2[!is.na(max_snw), 
                    .(max_snw = max(max_snw)),
                    .(icell, rcm, rcm_short)]
dat_snw[, max_snw_7000 := ifelse(max_snw > 7000, "above", "below")]

# dat_snw <- merge(dat_snw, dat_aux, by = "icell")
# ggplot(dat_snw, aes(x,y,fill=max_snw_7000))+
#   geom_raster()+
#   facet_wrap(~rcm)+
#   theme_bw()


# merge
dat_th <- merge(dat_snd, dat_snw, by = c("icell", "rcm", "rcm_short"), all = T)
dat_th[, snow_plausible := "yes"]
dat_th[max_snd_10m == "above", snow_plausible := "no"]
dat_th[max_snw_7000 == "above", snow_plausible := "no"]

# dat_th %>% with(table(snow_plausible, max_snd_10m, useNA = "a"))
# dat_th %>% with(table(snow_plausible, max_snw_7000, max_snd_10m, useNA = "a"))

saveRDS(dat_th, "data/auxiliary-02-snow-max-threshold.rds")




