# plot subset of CP

# library(raster)
library(stars)
library(sf)
library(data.table)
library(ggplot2)
library(magrittr)



# cp scaled ---------------------------------------------------------------

rs_cp <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp_10bins.tif")
rs_cp <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp_20bins.tif")

# extent ST
sf_ita <- read_sf("/mnt/CEPH_BASEDATA/GIS/NATIONAL/ITALY/BOUNDARIES/IT_bnd_prov_pol_ISTAT_2011.shp")
sf_bz <- sf_ita %>% dplyr::filter(NOME_PRO == "BOLZANO/BOZEN")
sf_bz_rs <- st_transform(sf_bz, st_crs(rs_cp))

rs_cp_crop <- st_crop(rs_cp, sf_bz_rs)
rs_plot <- st_as_stars(rs_cp_crop)

# 10bin
cut(0:100/100, breaks = c(0, 0.001, 1:9/10, 0.999, 1), include.lowest = T) %>% 
  levels -> band_lbl
# 20bin
cut(0:100/100, breaks = c(0, 0.001, 1:19/20, 0.999, 1), include.lowest = T) %>% 
  levels -> band_lbl
rs_plot <- rs_plot %>% 
  st_set_dimensions("band", values = band_lbl)

ggplot()+
  geom_stars(data = rs_plot, downsample = c(3,3,3), na.rm = T)+
  scale_fill_viridis_c("CP", limits = c(0,1), na.value = "white")+
  facet_wrap(~band)+
  coord_equal()+
  theme_void()+
  ggtitle("CP MODIS by SCF")


gg_out <- ggplot()+
  geom_stars(data = rs_plot, downsample = 0, na.rm = T)+
  scale_fill_viridis_c("CP", limits = c(0,1), na.value = "white")+
  facet_wrap(~band)+
  coord_equal()+
  theme_void()+
  ggtitle("CP MODIS by SCF")

ggsave(gg_out,
       filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/fig/ds/cp-20bin-south-tyrol.png",
       width = 14, height = 7)


# cp nn ---------------------------------------------------------------

rs_nn <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp_totn_10bins.tif")
rs_nn <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp_totn_20bins.tif")

# extent ST
sf_ita <- read_sf("/mnt/CEPH_BASEDATA/GIS/NATIONAL/ITALY/BOUNDARIES/IT_bnd_prov_pol_ISTAT_2011.shp")
sf_bz <- sf_ita %>% dplyr::filter(NOME_PRO == "BOLZANO/BOZEN")
sf_bz_rs <- st_transform(sf_bz, st_crs(rs_nn))

rs_nn_crop <- st_crop(rs_nn, sf_bz_rs)
rs_plot_nn <- st_as_stars(rs_nn_crop)

# 10bins
cut(0:100/100, breaks = c(0, 0.001, 1:9/10, 0.999, 1), include.lowest = T) %>% 
  levels -> band_lbl
# 20bins
cut(0:100/100, breaks = c(0, 0.001, 1:19/20, 0.999, 1), include.lowest = T) %>% 
  levels -> band_lbl

rs_plot_nn <- rs_plot_nn %>% 
  st_set_dimensions("band", values = band_lbl)

ggplot()+
  geom_stars(data = rs_plot_nn, downsample = c(2,2,2), na.rm = T)+
  # scale_fill_viridis_c("#", na.value = "white", trans = "log10")+
  scale_fill_viridis_b("#", na.value = "white", 
                       breaks = c(10,100,1000), 
                       trans = "log10",
                       show.limits = T)+
  facet_wrap(~band)+
  coord_equal()+
  theme_void()+
  ggtitle("CP MODIS by SCF (total N in SCF bin)")


gg_out <- ggplot()+
  geom_stars(data = rs_plot_nn, downsample = 0, na.rm = T)+
  # scale_fill_viridis_c("#", na.value = "white", trans = "log10")+
  scale_fill_viridis_b("#", na.value = "white", 
                       breaks = c(10,100,1000), 
                       trans = "log10",
                       show.limits = T)+
  facet_wrap(~band)+
  coord_equal()+
  theme_void()+
  ggtitle("CP MODIS by SCF (total N in SCF bin)")

ggsave(gg_out,
       filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/fig/ds/cp-20bin-south-tyrol-nn.png",
       width = 14, height = 7)
