# plot wasserstein distances example wise


library(raster)
# library(stars)
# library(sf)
library(data.table)
library(ggplot2)
library(magrittr)
library(scico)
library(forcats)

mat_dist <- as.matrix(readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/dist-01-wasserstein.rds"))

icell_sample <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/sample-pixels-01-stratified-elev.rds")

icell_sample



# prep data ---------------------------------------------------------------

rr_alt <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_alt[rr_alt < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_alt[rr_corine == 0] <- NA



rr_scf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/scf_cp50_v01.tif")
rr_scf
# plot(rr_scf)
# plot(is.na(rr_scf))

dat_scf <- data.table(icell = 1:ncell(rr_scf),
                      scf_th = rr_scf[],
                      alt = rr_alt[])
dat_scf[, alt_f := cut(alt, c(-150, 0, 1:47*100, 4800), dig.lab = 5)]




# add alt_diff
rr_alt_lr <- aggregate(rr_alt, 
                       fact = c(37, 53), 
                       fun = mean,
                       na.rm = F)

mat_alt_lr <- as.matrix(rr_alt_lr)
mat_alt_lr2hr <- mat_alt_lr[rep(1:55, each = 53),
                            rep(1:133, each = 37)]
mat_alt_lr2hr <- mat_alt_lr2hr[1:2867, 1:4901]
rr_alt_lr2hr <- raster(rr_alt)
rr_alt_lr2hr[] <- mat_alt_lr2hr

dat_scf[, alt_lr := rr_alt_lr2hr[]]
dat_scf[, alt_diff := alt_lr - alt]

# add lr2hr icell
rr_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")
rr_lr[] <- 1:ncell(rr_lr)
mat_icell_lr <- as.matrix(rr_lr)
mat_icell_lr2hr <- mat_icell_lr[rep(1:55, each = 53),
                                rep(1:133, each = 37)]
mat_icell_lr2hr <- mat_icell_lr2hr[1:2867, 1:4901]
rr_icell_lr2hr <- raster(rr_alt)
rr_icell_lr2hr[] <- mat_icell_lr2hr

dat_scf[, icell_lr := rr_icell_lr2hr[]]


# loop --------------------------------------------------------------------

dat_scf_sample <- dat_scf[icell_lr %in% icell_sample]
dat_scf_sample[, icell_lr_fct := fct_reorder(factor(icell_lr), alt_lr)]
# dat_scf_sample[, current := F]

pdf(file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/fig/ds/wasserstein-dist-sample.pdf",
    width = 16, height = 8)

for(i_icell in levels(dat_scf_sample$icell_lr_fct)){
  
  
  wasser_dist <- mat_dist[icell_sample, i_icell]
  icell_min_dist <- which.min(wasser_dist[wasser_dist > 0]) %>% names %>% as.numeric
  dat_dist <- data.table(wasser_dist = wasser_dist,
                         icell_lr = icell_sample)
  
  dat_plot <- merge(dat_scf_sample,
                    dat_dist)
  dat_plot[, current := ifelse(icell_lr == i_icell, "current", "other")]
  dat_plot[, wasser_dist := round(wasser_dist, 1)]
  dat_plot[icell_lr == icell_min_dist, current := "min_dist"]
  
  gg <- dat_plot %>% 
    ggplot(aes(alt))+
    geom_histogram(aes(fill = current))+
    facet_wrap(~icell_lr_fct + wasser_dist, scales = "free_y")+
    theme_bw()+
    ggtitle("icell sample plus wasserstein dist (facet labels)")
  
  print(gg)
}
dev.off()