# compare flo data to ds

library(stars)
# library(raster)
library(data.table)
library(magrittr)
library(ggplot2)
library(cowplot)
library(fs)

path_out <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/fig/ds/compare-flo-maps/"
rs_template_ds <- read_stars("data/rofental_modis_laea_flo.tif")

# overview ds ------------------------------------------------------------

c("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/03_qdm/",
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/02_qm/") %>% 
  lapply(function(x){
    
    data.table(bc_var = path_file(x),
               fn = dir_ls(x))
    
  }) %>% rbindlist -> dat_ds

dat_ds[, c("institute_rcm", "gcm", "experiment", "ensemble", "dsr", "fp") := 
         tstrsplit(path_ext_remove(path_file(fn)), "_")]

# overview amundsen -------------------------------------------------------


x <- "data-aux/amundsen-flo-hr/maps/"
dat_am <- data.table(fn = dir_ls(x))
dat_am[, c("zz", "gcm", "institute_rcm", "experiment", "period") := 
         tstrsplit(path_ext_remove(path_file(fn)), "_")]
dat_am[, fp := ifelse(period == "2000-2020", "past", "future")]



# in common? --------------------------------------------------------------

dat_loop <- dat_am[, .(gcm, institute_rcm, experiment)] %>% 
  unique %>% 
  merge(unique(dat_ds[, .(institute_rcm, gcm, experiment)]))

# plot maps ---------------------------------------------------------------

f_read_ds <- function(fn, rs0 = rs_template_ds){
  
  rs <- read_stars(fn)
  rs <- st_crop(rs, rs0)
  rs <- rs/365.25
  st_as_stars(rs)
  
}

foreach(
  i = seq_len(nrow(dat_loop))
) %do% {
  
  tit <- dat_loop[i, paste0(institute_rcm, ", ",
                     gcm, ", ",
                     experiment)]
  
  fn_out <- path(path_out,
                 dat_loop[i, paste0(institute_rcm, "_",
                                    gcm, "_",
                                    experiment, ".png")])
  
  dat_i_am <- dat_loop[i, ] %>% merge(dat_am)
  dat_i_ds <- dat_loop[i, ] %>% merge(dat_ds)
  
  rs_rof_past <- read_stars(dat_i_am[fp == "past", fn])
  rs_rof_future <- read_stars(dat_i_am[fp == "future", fn])
  
  rs_ds_past_qm <- f_read_ds(dat_i_ds[fp == "past" & bc_var == "02_qm", fn])
  rs_ds_future_qm <- f_read_ds(dat_i_ds[fp == "future" & bc_var == "02_qm", fn])
  rs_ds_past_qdm <- f_read_ds(dat_i_ds[fp == "past" & bc_var == "03_qdm", fn])
  rs_ds_future_qdm <- f_read_ds(dat_i_ds[fp == "future" & bc_var == "03_qdm", fn])
  

  # diff limits
  lim_diff <- range(rs_rof_future[[1]] - rs_rof_past[[1]],
                    rs_ds_future_qm[[1]] - rs_ds_past_qm[[1]],
                    rs_ds_future_qdm[[1]] - rs_ds_past_qdm[[1]],
                    na.rm  = T)
  
  
  gg_rof_past <- ggplot()+
    geom_stars(data = rs_rof_past)+
    scale_fill_viridis_c("", limits = c(0,1))+
    theme_void()+
    ggtitle(tit,
            "AMUNDSEN 2000-2020")
  
  gg_rof_future <- ggplot()+
    geom_stars(data = rs_rof_future)+
    scale_fill_viridis_c("", limits = c(0,1))+
    theme_void()+
    ggtitle("",
            "AMUNDSEN 2071-2100")
  
  gg_rof_diff <- ggplot()+
    geom_stars(data = rs_rof_future - rs_rof_past)+
    scale_fill_gradient2("", limits = lim_diff)+
    theme_void()+
    ggtitle("",
            "AMUNDSEN difference")
  
  
  
  gg_ds_past_qm <- ggplot()+
    geom_stars(data = rs_ds_past_qm)+
    scale_fill_viridis_c("", limits = c(0,1))+
    theme_void()+
    ggtitle("",
            "Downscale QM 2000-2020")
  
  gg_ds_future_qm <- ggplot()+
    geom_stars(data = rs_ds_future_qm)+
    scale_fill_viridis_c("", limits = c(0,1))+
    theme_void()+
    ggtitle("",
            "Downscale QM 2071-2100")
  
  gg_ds_diff_qm <- ggplot()+
    geom_stars(data = rs_ds_future_qm - rs_ds_past_qm)+
    scale_fill_gradient2("", limits = lim_diff)+
    theme_void()+
    ggtitle("",
            "Downscale QM difference")
  
  
  
  
  gg_ds_past_qdm <- ggplot()+
    geom_stars(data = rs_ds_past_qdm)+
    scale_fill_viridis_c("", limits = c(0,1))+
    theme_void()+
    ggtitle("",
            "Downscale QDM 2000-2020")
  
  gg_ds_future_qdm <- ggplot()+
    geom_stars(data = rs_ds_future_qdm)+
    scale_fill_viridis_c("", limits = c(0,1))+
    theme_void()+
    ggtitle("",
            "Downscale QDM 2071-2100")
  
  gg_ds_diff_qdm <- ggplot()+
    geom_stars(data = rs_ds_future_qdm - rs_ds_past_qdm)+
    scale_fill_gradient2("", limits = lim_diff)+
    theme_void()+
    ggtitle("",
            "Downscale QDM difference")
  
  
  gg_out <- plot_grid(gg_rof_past, gg_ds_past_qm, gg_ds_past_qdm,
                      gg_rof_future, gg_ds_future_qm, gg_ds_future_qdm, 
                      gg_rof_diff, gg_ds_diff_qm, gg_ds_diff_qdm,
                      nrow = 3)

  ggsave(fn_out,
         gg_out,
         width = 12,
         height = 9)
    
}






