# compare flo data to ds

# library(stars)
library(raster)
library(data.table)
library(magrittr)
library(ggplot2)
library(cowplot)
library(fs)




# glacier fraction by elev ------------------------------------------------

# from summary overview

# dat_ds_pixremoved <- data.table(elev = seq(0, 4000, by = 50),
#                                glac = "<5%")
# dat_ds_pixremoved[elev >= 2200 & elev <= 3000, glac := "5-50%"]
# dat_ds_pixremoved[elev > 3000, glac := ">50%"]

# dat_ds_pixremoved <- data.table(elev = seq(0, 3500, by = 50),
#                                glac = "<10%")
# dat_ds_pixremoved[elev >= 2600 & elev <= 3000, glac := "10-50%"]
# dat_ds_pixremoved[elev > 3000, glac := ">50%"]

dat_ds_pixremoved <- data.table(elev = seq(0, 3500, by = 50),
                               glac = "<20%")
dat_ds_pixremoved[elev >= 2700, glac := ">=20%"]

# flo data -------------------------------------------------------------

dir_ls("data-aux/amundsen-flo-hr/", regexp = "elev") %>% 
  lapply(function(x){
    
    path_file(x) %>% 
      path_ext_remove() %>% 
      stringr::str_split("_", simplify = T) -> x_split
    
    dat <- fread(x) %>% 
      melt(id.vars = "elev", value.name = "scf")
    
    dat[, c("gcm", "institute_rcm") := tstrsplit(variable, "_")]
    
    dat[, .(gcm, institute_rcm, 
            experiment = x_split[1,4], period = x_split[1,5], 
            elev, scf)]
    
    
  }) %>% rbindlist() -> dat_am_elev



dat_am_elev[, fp := ifelse(period == "2000-2020", "past", "future")]



# rcm data rofental ----------------------------------------------------------------



dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds_comp_amundsen/elev_noglacier_norca4/03_qdm/") %>% 
  lapply(readRDS) %>% 
  rbindlist -> dat_ds_elev



dat_ds_elev[, elev := tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans]

setorder(dat_ds_elev, bc_var, institute_rcm, gcm, experiment, fp, elev)



# subset to rcms with full region coverage --------------------------------


dat_ds_elev <- dat_ds_elev[institute_rcm %in% c(
  "CLMcom-CCLM4-8-17", 
  "CLMcom-ETH-COSMO-crCLIM-v1-1",
  # "CNRM-ALADIN63", # 7 lr pixels missing
  "IPSL-WRF381P",
  "SMHI-RCA4" # 2 lr pixels missing
  )]


dat_ds_elev[, .(institute_rcm, gcm, experiment)] %>% 
  unique %>% 
  .[, .N, experiment]



# alpine wide ens mean by elev --------------------------------------------


# common icell
dat_icell <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/ds/common-icell.rds")
names(dat_icell)
rcms_common <- c("MODIS", "CLMcom-CCLM4-8-17", "CLMcom-ETH-COSMO-crCLIM-v1-1",
                 "CNRM-ALADIN63", "IPSL-WRF381P", "SMHI-RCA4")
lgl_icell_common <- dat_icell[, 
                              apply(.SD, 1, all), 
                              .SDcols = rcms_common]


# glacier 

rr_rgi <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/rgi/modis-hr.tif")
lgl_icell_noglacier <- rr_rgi[] <= 0.1


# scd

rr_modis <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")
rr_rcp26 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp26_noglacier.tif")
rr_rcp85 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp85_noglacier.tif")


rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA

dat_modis <- data.table(icell = 1:ncell(rr_modis),
                        scd_modis = rr_modis[],
                        scd_rcp26 = rr_rcp26[],
                        scd_rcp85 = rr_rcp85[],
                        elev = rr_elev[])
# dat_modis <- dat_modis[!is.na(scd)]
# dat_modis$elev %>% qplot
# dat_modis[lgl_icell_common & elev > 3000, elev] %>% qplot
# dat_modis[lgl_icell_common & lgl_icell_noglacier & elev > 3000, elev] %>% qplot
# dat_modis[lgl_icell_common & lgl_icell_noglacier, elev] %>% summary

dat_modis[, elev_f := cut(elev, breaks = c(seq(-200, 3600, by = 100), 4200), dig.lab = 5)]

dat_modis2 <- dat_modis[lgl_icell_common & lgl_icell_noglacier, 
                        .(scd_modis = mean(scd_modis),
                          scd_rcp26 = mean(scd_rcp26),
                          scd_rcp85 = mean(scd_rcp85),
                          nn = .N),
                        .(elev_f)]

dat_modis2 %>% ggplot(aes(scd_modis, elev_f))+geom_point()





# elev: ds and am together ------------------------------------------------------

dat_plot <- rbind(
  dat_ds_elev[bc_var == "03_qdm" & experiment == "rcp85" & elev > 850 & elev < 3500,
              .(gcm, institute_rcm, fp, elev, scf, ff = "Downscaling")],
  dat_am_elev[experiment == "rcp85" & elev > 850 & elev < 3500,
              .(gcm, institute_rcm, fp, elev, scf, ff = "AMUNDSEN")]
)

setorder(dat_plot, ff, fp, gcm, institute_rcm, elev)


dat_plot %>% 
  ggplot(aes(scf, elev, colour = ff))+
  geom_path(aes(group = paste0(ff, fp, institute_rcm, gcm)))+
  facet_wrap(~fp)+
  theme_bw()

dat_plot_mmm <- dat_plot[, 
                         .(mod_mean = mean(scf),
                           mod_min = min(scf),
                           mod_max = max(scf)),
                         .(ff, fp, elev)]

dat_plot_mmm %>% 
  ggplot(aes(mod_mean, elev, 
             xmin = mod_min, xmax = mod_max,
             colour = ff, fill = ff))+
  # geom_tile(inherit.aes = F, data = dat_glacier_freq[glac != "<10%"],
  geom_tile(inherit.aes = F, data = dat_glacier_freq[glac == ">50%"],
            aes(x = 0.5, width = Inf, y = elev, height = 50),
            fill = "grey90")+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_path()+
  facet_wrap(~fp)+
  theme_bw()


dat_plot_mmm[, fp_ff := paste0(ff, " ", fp)]

dat_plot_mmm %>% 
  ggplot(aes(mod_mean, elev, 
             xmin = mod_min, xmax = mod_max,
             colour = fp_ff, fill = fp_ff))+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_path()+
  scale_color_brewer(palette = "Paired", aesthetics = c("color", "fill"))+
  # scale_linetype_manual(values = c("solid", "solid", "dashed", "dashed"))+
  theme_bw()


dat_plot_diff <- dat_plot %>% 
  dcast(gcm + institute_rcm + elev + ff ~ fp, value.var = "scf")

dat_plot_diff[, scf_diff := future - past]
setorder(dat_plot_diff, ff, gcm, institute_rcm, elev)

dat_plot_diff %>% 
  ggplot(aes(scf_diff, elev, colour = ff))+
  geom_vline(xintercept = 0)+
  geom_path(aes(group = paste0(ff, institute_rcm, gcm)))+
  theme_bw()



# add study region average ------------------------------------------------

dat_modis2[, elev := tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans]

dat_modis2[elev < 3500, 
           .(elev, past = scd_modis/365.25, future = scd_rcp85/365.25)] %>% 
  melt(id.vars = "elev",
       variable.name = "fp",
       variable.factor = F) -> dat_plot_alps
setorder(dat_plot_alps, fp, elev)

dat_plot_mmm %>% 
  ggplot(aes(mod_mean, elev, 
             xmin = mod_min, xmax = mod_max,
             colour = ff, fill = ff))+
  geom_tile(inherit.aes = F, data = dat_ds_pixremoved[glac == ">=20%"],
            aes(x = 0.5, width = Inf, y = elev, height = 50),
            fill = "grey95")+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_path()+
  geom_path(inherit.aes = F, data= dat_plot_alps,
            aes(value, elev))+
  facet_wrap(~fp)+
  theme_bw()




# bc, ds, amun, alps, rofental --------------------------------------------








# EOF ---------------------------------------------------------------------






