# plot subset of CP

library(raster)
# library(stars)
# library(sf)
library(data.table)
library(ggplot2)
library(magrittr)
library(scico)

icell_sample <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/sample-pixels-02-high-res.rds")
rr_alt <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_alt[rr_alt < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_alt[rr_corine == 0] <- NA

# try different samples (from sample-pixels-02...R)
dat_sample <- dat[!is.na(scd), sample(icell, 5), alt_f]
icell_sample <- dat_sample$V1

# alt_sample <- rr_alt[icell_sample]



f_data_cp <- function(icell_sample){
  
  mat_values <- rr_cp[icell_sample]
  colnames(mat_values) <- band_lbl
  
  dt_values <- mat_values %>% 
    as.data.table()
  
  dt_values[, icell := icell_sample]
  dt_values[, alt := rr_alt[icell_sample]]
  
  dt_values %>% 
    melt(id.vars = c("icell", "alt")) -> dt_values2
  
  dt_values2[value < 0, value := NA]
  dt_values2[, c("scf_lb", "scf_ub") := tstrsplit(variable, ",")]
  dt_values2[, scf_lb := readr::parse_number(scf_lb)]
  dt_values2[, scf_ub := readr::parse_number(scf_ub)]
  dt_values2[, scf_middle := (scf_lb + scf_ub) / 2]
  dt_values2
}

# sample pixels ---------------------------------------------------------------




# 10bin
rr_cp <- brick("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/cp_10bins.tif")
cut(0:100/100, breaks = c(0, 0.001, 1:9/10, 0.999, 1), include.lowest = T) %>% 
  levels -> band_lbl

# 20bin
rr_cp <- brick("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/cp_20bins.tif")
cut(0:100/100, breaks = c(0, 0.001, 1:19/20, 0.999, 1), include.lowest = T) %>% 
  levels -> band_lbl
brks <- c(0, 0.001, 1:19/20, 0.999, 1)
scf_middle <- (head(brks, -1) + tail(brks, -1))/2
# names(rr_cp) <- band_lbl


dt_values2 <- f_data_cp(icell_sample)


dt_values2 %>% 
  ggplot(aes(scf_middle, value))+
  geom_point()+
  geom_line(aes(group = icell))+
  theme_bw()

dt_values2 %>% 
  ggplot(aes(scf_middle, value))+
  geom_point()+
  # geom_smooth(method = lm, formula = y ~ ns(x, 3), se = F)+
  geom_line(aes(group = icell))+
  theme_bw()+
  facet_wrap(~alt)



dt_values2[, 
           .(scf_middle = c(0, 1),
             value = c(0,1)),
           .(icell, alt)] %>% 
  rbind(dt_values2, fill = T) -> dt_values3
dt_values3 %>% 
  ggplot(aes(scf_middle, value))+
  geom_point()+
  geom_smooth(method = lm, formula = y ~ ns(x, 3), se = F)+
  geom_line(aes(group = icell))+
  theme_bw()+
  facet_wrap(~alt)

# no splines



# calc # na per pixel -----------------------------------------------------

rr_cp_na <- calc(rr_cp, function(x) sum(x < 0))
plot(rr_cp_na)

dat_na <- data.table(icell = 1:ncell(rr_cp),
                     n_na = rr_cp_na[],
                     alt = rr_alt[])

# dat_na %>% summary

dat_na %>% 
  ggplot(aes(alt, n_na))+
  geom_bin2d(binwidth = c(100, 1))+
  scale_fill_viridis_c(trans = "log10")+
  theme_bw()


# -> more distributed than thought...


# sample low elev only ----------------------------------------------------

dat_na[, alt_f := cut(alt, c(-150, 1:9*500, 4800), dig.lab = 5)]

dat_sample <- dat_na[!is.na(n_na) & n_na > 10 & alt < 4500, sample(icell, 5), alt_f]
icell_sample <- dat_sample$V1

dt_low <- f_data_cp(icell_sample)

dt_low %>% 
  ggplot(aes(scf_middle, value))+
  geom_point()+
  geom_line(aes(group = icell))+
  theme_bw()+
  facet_wrap(~alt)

dt_low[, alt_f := cut(alt, c(-150, 1:9*500, 4800), dig.lab = 5)]

dt_low %>% 
  ggplot(aes(scf_middle, value, colour = alt))+
  geom_point()+
  geom_line(aes(group = icell))+
  scale_color_viridis_c()+
  theme_bw()+
  facet_wrap(~alt_f)



# sample more ----------------------------------------------------

dat_na[, alt_f := cut(alt, c(-150, 1:9*500, 4800), dig.lab = 5)]

dat_sample <- dat_na[!is.na(n_na) & !is.na(alt) , sample(icell, 10), alt_f]
icell_sample <- dat_sample$V1

dt_more <- f_data_cp(icell_sample)

dt_more[, alt_f := cut(alt, c(-150, 1:9*500, 4800), dig.lab = 5)]

dt_more %>% 
  ggplot(aes(scf_middle, value, colour = alt))+
  # geom_point()+
  geom_line(aes(group = icell))+
  scale_color_viridis_c()+
  theme_bw()+
  facet_wrap(~alt_f)



# with scf app
dat_sample <- dat_na[!is.na(n_na) & !is.na(alt) , sample(icell, 5), alt_f]
icell_sample <- dat_sample$V1
dt_more2 <- f_data_cp(icell_sample)

dt_more2_app <- dt_more2[, 
                         .(y = cp50_scf(value)), 
                         .(icell, alt)]


dt_more2 %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, colour = "grey")+
  geom_point()+
  geom_line(aes(group = icell))+
  geom_vline(data = dt_more2_app, aes(xintercept = y), colour = "red")+
  theme_bw()+
  facet_wrap(~alt)

# calc cp50 scf value -----------------------------------------------------

# using linear interpolation where it works. 
# no additional 0 and 1 superimposed yet



brks <- c(0, 0.001, 1:19/20, 0.999, 1)
scf_middle <- (head(brks, -1) + tail(brks, -1))/2

vals <- dt_values2[icell == 2669217, value]


app <- approx(scf_middle, vals, xout = seq(0,1,by=0.01))
plot(scf_middle, vals)
lines(app$x, app$y)

app <- approx(vals, scf_middle, xout = seq(0,1,by=0.01))
plot(vals, scf_middle)
lines(app$x, app$y)
approx(vals, scf_middle, xout = 0.5)

dt_values2_app <- dt_values2[, 
                             .(y = cp50_scf(value)), 
                             .(icell, alt)]

dt_values2 %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, colour = "grey")+
  geom_point()+
  geom_line(aes(group = icell))+
  geom_vline(data = dt_values2_app, aes(xintercept = y), colour = "red")+
  theme_bw()+
  facet_wrap(~alt)

dt_values2 %>% 
  ggplot(aes(value, scf_middle))+
  geom_vline(xintercept = 0.5, colour = "grey")+
  geom_point()+
  geom_line(aes(group = icell))+
  geom_hline(data = dt_values2_app, aes(yintercept = y), colour = "red")+
  theme_bw()+
  facet_wrap(~alt)

# remove first and last pixel
cp50_scf <- function(vals, ind = 2:21){
  
  vals_ind <- vals[ind]
  vals_ind_na <- is.na(vals_ind)
  if(sum(!vals_ind_na) >= 2 & length(unique(vals_ind[!vals_ind_na])) >= 2) {
    approx(vals_ind, scf_middle[ind], 
           xout = 0.5, ties = min)$y
  } else {
    NA_real_
  }
  
}

# remove near 0 and 1 pixels (if incomplete cp)
cp50_scf <- function(vals, eps = 0.001){
  
  if(sum(is.na(vals)) == 0){
    
    approx(vals, scf_middle, 
           xout = 0.5, ties = min)$y
    
  } else {
    
    ind <- vals > eps & vals < (1-eps)
    vals_ind <- vals[ind]
    vals_ind_na <- is.na(vals_ind)
    if(sum(!vals_ind_na) >= 2 & length(unique(vals_ind[!vals_ind_na])) >= 2) {
      approx(vals_ind, scf_middle[ind], 
             xout = 0.5, ties = min)$y
    } else {
      NA_real_
    }
    
  }

}

rr_cp




# test cp fun na ----------------------------------------------------------

for(i in 1:1000){
  icell <- sample(1:ncell(rr_cp), 1)
  print(icell)
  cp50_scf(rr_cp[icell])
}

for(i in 1:1000){
  # icell <- sample(1:ncell(rr_cp), 1)
  icell <- i*50
  cat(icell, " | ")
  xx <- cp50_scf(rr_cp[icell])
  cat(xx, "\n")
}







# EOF ---------------------------------------------------------------------


