# compare past downscale to MODIS SCD

library(data.table)
library(magrittr)
library(raster)
library(stars)
library(ggplot2)
library(fs)
library(foreach)


path_out_map <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/fig/ds/compare-ds-clim-maps/"
path_out_scat <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/fig/ds/compare-ds-clim-past-scatter/"

rs_modis <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")
rs_th <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v05_impute_level.tif")

ds_stars <- 10


gg0 <- ggplot()+
  geom_stars(data = rs_modis, downsample = ds_stars)+
  scale_fill_viridis_c("", na.value = NA)+
  theme_void()+
  ggtitle("MODIS")

# overall -----------------------------------------------------------------


c(
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/02_qm/",
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/03_qdm/"#,
  #"/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/04_mbc_qdm/"
) %>% 
  lapply(function(x){
    
    data.table(bc_var = path_file(x),
               file_rcm = dir_ls(x))
    
  }) %>% rbindlist() -> dat_loop


dat_loop[, c("institute_rcm", "gcm", "experiment", "ensemble", "dsr", "fp") := 
           tstrsplit(path_ext_remove(path_file(file_rcm)), "_")]

dat_loop2 <- dat_loop[, .(bc_var, institute_rcm, gcm, experiment)] %>% unique


mitmatmisc::init_parallel_ubuntu(12)

dat_metrics <- foreach(
  i = 1:nrow(dat_loop2),
  .final = rbindlist
) %dopar% {
  
  dat_loop2[i, ] %>% 
    merge(dat_loop) -> dat_i
  
  rs_rcm_past <- read_stars(dat_i[fp == "past", file_rcm])
  rs_rcm_future <- read_stars(dat_i[fp == "future", file_rcm])
  
  fn_gg_map <- path(path_out_map,
                    dat_loop2[i, bc_var],
                    dat_loop2[i, paste0(institute_rcm, "_", gcm, "_", experiment, ".png")])
  dir_create(path_dir(fn_gg_map))
  
  # maps
  gg1 <- ggplot()+
    geom_stars(data = rs_rcm_past, downsample = ds_stars)+
    scale_fill_viridis_c("", na.value = NA)+
    theme_void()+
    ggtitle(dat_i[fp == "past", path_file(file_rcm)])
  
  gg2 <- ggplot()+
    geom_stars(data = rs_rcm_future, downsample = ds_stars)+
    scale_fill_viridis_c("", na.value = NA)+
    theme_void()+
    ggtitle(dat_i[fp == "future", path_file(file_rcm)])
  
  gg1_diff <- ggplot()+
    geom_stars(data = rs_rcm_past - rs_modis, downsample = ds_stars)+
    scale_fill_gradient2("")+
    theme_void()
  
  gg2_diff <- ggplot()+
    geom_stars(data = rs_rcm_future - rs_modis, downsample = ds_stars)+
    scale_fill_gradient2("")+
    theme_void()
  
  
  gg_out <- cowplot::plot_grid(gg0, gg1, gg2,
                               NULL, gg1_diff, gg2_diff)

  ggsave(gg_out,
         filename = fn_gg_map,
         width = 20,
         height = 10)  
  
  
  # summary metrics
  dat1 <- data.table(scd_modis = as.vector(rs_modis[[1]]),
                     scd_past = as.vector(rs_rcm_past[[1]]),
                     scd_future = as.vector(rs_rcm_future[[1]]),
                     th_level = as.vector(rs_th[[1]]))

  dat2 <- dat1[, 
               .(th_level = 0L,
                 bias_past = mean(scd_past - scd_modis, na.rm = T),
                 bias_future = mean(scd_future - scd_modis, na.rm = T),
                 mae_past = mean(abs(scd_past - scd_modis), na.rm = T),
                 mae_future = mean(abs(scd_future - scd_modis), na.rm = T))]
  
  dat2l <- dat1[!is.na(th_level), 
               .(bias_past = mean(scd_past - scd_modis, na.rm = T),
                 bias_future = mean(scd_future - scd_modis, na.rm = T),
                 mae_past = mean(abs(scd_past - scd_modis), na.rm = T),
                 mae_future = mean(abs(scd_future - scd_modis), na.rm = T)),
               th_level]

  
  # scatter plot
  
  dat_plot_scatter <- rbind(dat1[!is.na(th_level), .(scd_modis, scd_past, th_level = 0L)],
                            dat1[!is.na(th_level), .(scd_modis, scd_past, th_level)])
  

  fn_gg_scat <- path(path_out_scat,
                     dat_loop2[i, bc_var],
                    dat_loop2[i, paste0(institute_rcm, "_", gcm, "_", experiment, ".png")])
  dir_create(path_dir(fn_gg_scat))
  
  gg_scat <- ggplot(dat_plot_scatter, 
                     aes(scd_modis, scd_past))+
    geom_bin2d(binwidth = 5)+
    scale_fill_viridis_c(trans = "log10")+
    geom_abline()+
    facet_wrap(~th_level)+
    theme_bw()+
    ggtitle(dat_i[fp == "past", path_file(file_rcm)])
  
  ggsave(gg_scat,
         filename = fn_gg_scat,
         width = 14,
         height = 12)
  
  
  rbind(dat2, dat2l) %>% 
    cbind(dat_loop2[i, ])
  

}

saveRDS(dat_metrics,
        file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/ds/metrics-past.rds")



