# identify how upscale x and y correspond in MODIS


library(raster)
library(lubridate)
library(magrittr)
library(mapview)



# get raster from Products
rr <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/05_temporal_complete_max10d/20000315_120000.tif")

# remove water
rr_na <- rr
rr_na[rr_na > 3 | rr == 0] <- NA

# get snow fraction
rr_temp <- rr_na == 1
rr_snow <- aggregate(rr_temp, fact = c(37, 53), fun = mean)


rr_snow
rr

2867/53
4901/37

mapview(rr_snow)
mapview(rr)


mapview(rr_snow)+mapview(rr)




# try stars ---------------------------------------------------------------

rs1 <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/05_temporal_complete_max10d/20000315_120000.tif")
rs2 <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/modis_10d/modis_snc_55x133_2000-03-15.tif")

xl <- c(3847098, 4987079)
yl <- c(2197169, 2872448)

ggplot()+
  geom_stars(data = rs1, downsample = 20)+
  coord_sf(xlim = xl, ylim = yl)+
  theme(legend.position = "none")
  
ggplot()+
  geom_stars(data = rs2[,,,1])+
  coord_sf(xlim = xl, ylim = yl)+
  theme(legend.position = "none")


# check upscaling ---------------------------------------------------------

rr1 <- raster(rr)
rr1[] <- 1L
rr1

rr1_up <- aggregate(rr1, fact = c(37, 53), fun = sum)

mat_rr1_up <- as.matrix(rr1_up)
mat_rr1_up %>% dim

# 55x133 
# 2867x4901   -  53,37
# 54*53 + 5 x 132*37 + 17

mat_rr2 <- mat_rr1_up[rep(1:55, each = 53),
                      rep(1:133, each = 37)]
mat_rr2 %>% dim

mat_rr3 <- mat_rr2[1:2867, 1:4901]
mat_rr3 %>% dim


