# much too slow if mat_cp is not fully loaded

# -------------- #

# implement filling of scf_th based on similar lr-pixels

# search up to 50 similar pixels 
#   (optional : sub with wasserstein dist < 200-300, 
#    but for highest elev that does not work good...)
# add buffer around alt_f (+- 100-200m) 

library(raster)
library(data.table)
library(magrittr)
library(foreach)

source("R/functions/cp_helpers.R")
source("R/functions/cp_mae.R")


# settings ----------------------------------------------------------------

n_sim_pixels <- 50 # number of lr pixels to look for
alt_buffer <- 150 # meters to look for similar cp



# prep data ---------------------------------------------------------------

rr_alt <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_alt[rr_alt < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_alt[rr_corine == 0] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
# rr_alt[rr_water == 1] <- NA #problems with aggregate later

rr_scf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/scf_cp50_v02_monotonic.tif")
rr_scf
# plot(rr_scf)
# plot(is.na(rr_scf))

dat_scf <- data.table(icell = 1:ncell(rr_scf),
                      water = rr_water[],
                      scf_th = rr_scf[],
                      alt = rr_alt[])
dat_scf[, alt_f := cut(alt, c(-150, 0, 1:47*100, 4800), dig.lab = 5)]




# add alt_diff
rr_alt_lr <- aggregate(rr_alt,
                       fact = c(37, 53),
                       fun = mean,
                       na.rm = F)

mat_alt_lr <- raster::as.matrix(rr_alt_lr)
mat_alt_lr2hr <- mat_alt_lr[rep(1:55, each = 53),
                            rep(1:133, each = 37)]
mat_alt_lr2hr <- mat_alt_lr2hr[1:2867, 1:4901]
rr_alt_lr2hr <- raster(rr_alt)
rr_alt_lr2hr[] <- mat_alt_lr2hr

dat_scf[, alt_lr := rr_alt_lr2hr[]]
dat_scf[, alt_diff := alt_lr - alt]

# add lr2hr icell
rr_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")
rr_lr[] <- 1:ncell(rr_lr)
mat_icell_lr <- raster::as.matrix(rr_lr)
mat_icell_lr2hr <- mat_icell_lr[rep(1:55, each = 53),
                                rep(1:133, each = 37)]
mat_icell_lr2hr <- mat_icell_lr2hr[1:2867, 1:4901]
rr_icell_lr2hr <- raster(rr_alt)
rr_icell_lr2hr[] <- mat_icell_lr2hr

dat_scf[, icell_lr := rr_icell_lr2hr[]]


# remove water and not available pixels
dat_scf_eval <- dat_scf[!is.na(alt) & !is.na(alt_lr) & water == 0]


# prep for loop 
dat_scf_eval_missing <- dat_scf_eval[, 
                                     .(n_na = sum(is.na(scf_th)),
                                       n_avail = sum(!is.na(scf_th))),
                                     .(icell_lr)]

lr_icell <- dat_scf_eval_missing[n_na > 0, icell_lr]



# cp data -----------------------------------------------------------------

rr_cp <- brick("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/cp_20bins_monotonic.tif")
band_lbl <- levels(cut(0:100/100, 
                       breaks = c(0, 0.001, 1:19/20, 0.999, 1), 
                       include.lowest = T))

# mat_cp_full <- raster::as.matrix(rr_cp) # icell*l
# 
# mat_cp_full[mat_cp_full < 0] <- NA
# colnames(mat_cp_full) <- band_lbl
# rownames(mat_cp_full) <- 1:ncell(rr_cp)

source("R/functions/cp_rr2dt.R")

# dist lr icells
mat_dist <- as.matrix(readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/dist-01-wasserstein.rds"))

# main loop ---------------------------------------------------------------

mitmatmisc::init_parallel_ubuntu(4)

dat_icell_minmae <- foreach(
  i_lr_icell = lr_icell[c(1:10, 13:26)], # test n_na>=10
  .inorder = F,
  .final = rbindlist
) %dopar% {
  
  i_lr_icell_similar <- icell_min_dist(i_lr_icell, n_sim_pixels)$icell_with_min_dist
  
  i_hr_missing <- dat_scf_eval[icell_lr == i_lr_icell & is.na(scf_th), icell]
  
  dat_scf_eval_i <- dat_scf_eval[!is.na(scf_th) & 
                         icell_lr %in% i_lr_icell_similar]
  
  lapply(i_hr_missing[1:10], function(x){
    
    x_cp <- cp_rr2mat(x)
    x_alt <- dat_scf_eval[icell == x, alt]
    
    icell_to_check <- dat_scf_eval_i[between(alt, 
                                             x_alt - alt_buffer, 
                                             x_alt + alt_buffer), 
                                  icell]
    mat_cp_to_check <- cp_rr2mat(icell_to_check)
    
    cp_mae(x, x_cp, mat_cp_to_check)
    
  }) %>% rbindlist -> dat_next_icell
  
  dat_next_icell
  
}



# save --------------------------------------------------------------------

# icells with mae
saveRDS(dat_icell_minmae, 
        "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/dist-02-mae-hr.rds")

# new scf
dat_new_th <- merge(dat_icell_minmae,dat_scf, 
        by.x = "icell_min_cp_mae", by.y = "icell")

rr_scf_new <- raster(rr_scf)
rr_scf_new[dat_new_th$icell_orig] <- dat_new_th$scf_th

rr_scf_new[dat_scf_eval[!is.na(scf_th), icell]] <- dat_scf_eval[!is.na(scf_th), scf_th]

writeRaster(rr_scf_new, 
            "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/scf_cp50_v03_imputed_similar.tif",
            overwrite = T)

# mae imputed
# -> (can be calculated from imputed cell data)


# wasser_dist of next icell 
# -> (can be computed from imputed cell data)



# EOF ---------------------------------------------------------------------


