# compare flo data to ds

# library(stars)
library(raster)
library(data.table)
library(magrittr)
library(ggplot2)
library(cowplot)
library(fs)
library(foreach)




# check that rca4 has two missing large-scale pixels (which elev?  --------



rr_template_ds <- raster("data/rofental_modis_laea_flo.tif")
rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA

rr_elev_rofental <- crop(rr_elev, rr_template_ds)

rr_rca4 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/02_qm/SMHI-RCA4_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future.tif")
rr_rca4_rofental <- crop(rr_rca4, rr_template_ds)
rr_elev_rofental_rca4 <- rr_elev_rofental
rr_elev_rofental_rca4[is.na(rr_rca4_rofental)] <- NA

dat_elev <- data.table(icell = 1:ncell(rr_elev_rofental),
                       elev = rr_elev_rofental[],
                       elev_rca4 = rr_elev_rofental_rca4[])


# qplot(dat_elev$elev)
# summary(dat_elev$elev)
dat_elev[, elev_f := cut(elev, c(seq(800, 3500, by = 100), 3800), dig.lab = 5)]
# dat_elev[, .N, keyby = elev_f]

# mitmatmisc::init_parallel_ubuntu(5)



c("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds/03_qdm/",
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds/02_qm/") %>% 
  lapply(function(x){
    
    data.table(bc_var = path_file(x),
               fn = dir_ls(x))
    
  }) %>% rbindlist -> dat_loop

dat_loop[, c("institute_rcm", "gcm", "experiment", "ensemble", "dsr", "fp") := 
           tstrsplit(path_ext_remove(path_file(fn)), "_")]



# ds data elev ------------------------------------------------------------


dat_ds_elev <- foreach(
  i = seq_len(nrow(dat_loop)),
  .inorder = F,
  .final = rbindlist
) %dopar% {
  
  dat_i <- readRDS(dat_loop[i, fn])
  dat_i2 <- merge(dat_i, dat_elev[!is.na(elev_f), .(icell, elev_f)])
  
  dat_i2_elev <- dat_i2[, .(scf = mean(snc == 1, na.rm = T)), elev_f]
  
  dat_i2_elev %>% 
    cbind(dat_loop[i, .(bc_var, institute_rcm, gcm, experiment, fp)])
  
}


saveRDS(dat_ds_elev,
        "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/ds/rofental-summary-01-elev.rds")


# ds data month -----------------------------------------------------------


dat_ds_month <- foreach(
  i = seq_len(nrow(dat_loop)),
  .inorder = F,
  .final = rbindlist
) %dopar% {
  
  dat_i <- readRDS(dat_loop[i, fn])
  
  dat_i_month <- dat_i[, .(scf = mean(snc == 1, na.rm = T)), month(date)]
  
  dat_i_month %>% 
    cbind(dat_loop[i, .(bc_var, institute_rcm, gcm, experiment, fp)])
  
}


saveRDS(dat_ds_month,
        "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/ds/rofental-summary-02-month.rds")