# compare flo data to ds

library(raster)
library(data.table)
library(magrittr)
library(ggplot2)
library(fs)


rr_template_ds <- raster("data/rofental_modis_laea.tif")

# overview ds ------------------------------------------------------------

c("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds/03_qdm/",
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds/02_qm/") %>% 
  lapply(function(x){
    
    data.table(bc_var = path_file(x),
               fn = dir_ls(x))
    
  }) %>% rbindlist -> dat_ds

dat_ds[, c("institute_rcm", "gcm", "experiment", "ensemble", "dsr", "fp") := 
         tstrsplit(path_ext_remove(path_file(fn)), "_")]

# overview amundsen -------------------------------------------------------


x <- "data-aux/amundsen-flo-hr/maps/"
dat_am <- data.table(fn = dir_ls(x))
dat_am[, c("zz", "gcm", "institute_rcm", "experiment", "period") := 
         tstrsplit(path_ext_remove(path_file(fn)), "_")]
dat_am[, fp := ifelse(period == "2000-2020", "past", "future")]



# in common? --------------------------------------------------------------

dat_loop <- dat_am[, .(gcm, institute_rcm, experiment)] %>% 
  unique %>% 
  merge(unique(dat_ds[, .(institute_rcm, gcm, experiment)]))

# plot maps ---------------------------------------------------------------

f_read_ds_clim <- function(fn, rr0 = rr_template_ds){
  
  dat <- readRDS(fn)
  dat2 <- dat[, .(snc = mean(snc)), .(icell)]
  rr_out <- raster(rr_template_ds)
  rr_out[dat2$icell] <- dat2$snc
  rr_out
  
}

foreach(
  i = seq_len(nrow(dat_loop))
) %do% {
  
  dat_i_am <- dat_loop[i, ] %>% merge(dat_am)
  dat_i_ds <- dat_loop[i, ] %>% merge(dat_ds)
  
  rr_rof_past <- raster(dat_i_am[fp == "past", fn])
  
  rr <- f_read_ds_clim(dat_i_ds[fp == "past" & bc_var == "02_qm", fn])
  
  rr2 <- projectRaster(rr, rr_rof_past)
  plot(stack(rr_rof_past, rr2))
  
}






