## template from package MBC function QDM ####

# changes made:
#
# - Added arguments m.c.eval/m.p.eval. If supplied, the ECDF's are computed
#   for the original m.c and m.p, but only evaluated at *.eval (useful e.g.
#   for a moving window approach). Return values are of lenght *.eval
#
# - Added runif for a max value (typically 1), so QDM works also for the 
#   upper quantiles (the problem was constant snow cover for a large range
#   of quantiles).
#   The same approach was used as was implemented for 0 and ratio variables
# - Added check for max value != 1 (some RCM's have max SNC e.g. 0.98)
#
# - Added trace check for max value, too (as was done for 0)
#
# - Added an additional update to 0 in mhat, if values were 0 in the
#   original series (motivation: if models have 0 snow cover, then 
#   no change should be possible). Otherwise values might be inflated in 
#   mhat because of the runif adjustment (which is necessary numerically)
#   **REMOVED** one can also "add" snow cover by the BC routine.
# 
# - remove ECBC and corresponding ties
#
# - added param "ties=mean" to approx, which is default values anyway, to remove warnings
# - made ratio by default TRUE



QDM_snc <- function (o.c, m.c, m.p, ratio = TRUE, trace = 0.05, trace.calc = 0.5 * trace, 
                     jitter.factor = 0, n.tau = NULL, ratio.max = 2, ratio.max.trace = 10 * trace, 
                    # ECBC = FALSE, ties = "first", 
                    subsample = NULL, pp.type = 7,
                    m.c.eval = NULL, m.p.eval = NULL) {
  
  # check if eval present
  if(is.null(m.c.eval)) m.c.eval <- m.c
  if(is.null(m.p.eval)) m.p.eval <- m.p
  
  if (jitter.factor == 0 && (length(unique(o.c)) == 1 || 
                             length(unique(m.c)) == 1 || 
                             length(unique(m.p)) == 1 || 
                             length(unique(m.c.eval)) == 1 ||
                             length(unique(m.p.eval)) == 1)) {
    jitter.factor <- sqrt(.Machine$double.eps)
  }
  if (jitter.factor > 0) {
    o.c <- jitter(o.c, jitter.factor)
    m.c <- jitter(m.c, jitter.factor)
    m.p <- jitter(m.p, jitter.factor)
    m.c.eval <- jitter(m.c.eval, jitter.factor)
    m.p.eval <- jitter(m.p.eval, jitter.factor)
  }
  
  
  
  # get max possible snow cover if != 1, but only if slightly lower (some RCM have e.g max=0.98)
  max_snc <- max(m.c)
  if(max_snc <= 0.95) max_snc <- 1
  
  # add random component (to deal with constant 0 and 1 values)
  if (ratio) {
    epsilon <- .Machine$double.eps
    # 0 limit (already was in)
    o.c[o.c < trace.calc] <- runif(sum(o.c < trace.calc), min = epsilon, max = trace.calc)
    m.c[m.c < trace.calc] <- runif(sum(m.c < trace.calc), min = epsilon, max = trace.calc)
    m.p[m.p < trace.calc] <- runif(sum(m.p < trace.calc), min = epsilon, max = trace.calc)
    m.c.eval[m.c.eval < trace.calc] <- runif(sum(m.c.eval < trace.calc), min = epsilon, max = trace.calc)
    m.p.eval[m.p.eval < trace.calc] <- runif(sum(m.p.eval < trace.calc), min = epsilon, max = trace.calc)
    # 1 (or max) limit (added by MM)
    o.c[(max_snc - o.c) < trace.calc] <- max_snc - runif(sum((max_snc - o.c) < trace.calc), min = epsilon, max = trace.calc)
    m.c[(max_snc - m.c) < trace.calc] <- max_snc - runif(sum((max_snc - m.c) < trace.calc), min = epsilon, max = trace.calc)
    m.p[(max_snc - m.p) < trace.calc] <- max_snc - runif(sum((max_snc - m.p) < trace.calc), min = epsilon, max = trace.calc)
    m.c.eval[(max_snc - m.c.eval) < trace.calc] <- max_snc - runif(sum((max_snc - m.c.eval) < trace.calc), min = epsilon, max = trace.calc)
    m.p.eval[(max_snc - m.p.eval) < trace.calc] <- max_snc - runif(sum((max_snc - m.p.eval) < trace.calc), min = epsilon, max = trace.calc)
    
  }
  n <- length(m.p)
  if (is.null(n.tau)) n.tau <- n
  tau <- seq(0, 1, length = n.tau)
  if (!is.null(subsample)) {
    quant.o.c <- rowMeans(apply(replicate(subsample, sample(o.c, size = length(tau))), 
                                2, quantile, probs = tau, type = pp.type))
    quant.m.c <- rowMeans(apply(replicate(subsample, sample(m.c, size = length(tau))), 
                                2, quantile, probs = tau, type = pp.type))
    quant.m.p <- rowMeans(apply(replicate(subsample, sample(m.p, size = length(tau))), 
                                2, quantile, probs = tau, type = pp.type))
  } else {
    quant.o.c <- quantile(o.c, tau, type = pp.type)
    quant.m.c <- quantile(m.c, tau, type = pp.type)
    quant.m.p <- quantile(m.p, tau, type = pp.type)
  }
  
  tau.m.p <- approx(quant.m.p, tau, m.p.eval, rule = 2, ties = mean)$y
  
  if (ratio) {
    approx.t.qmc.tmp <- approx(tau, quant.m.c, tau.m.p, rule = 2, ties = mean)$y
    delta.m <- m.p.eval/approx.t.qmc.tmp
    delta.m[(delta.m > ratio.max) & (approx.t.qmc.tmp < ratio.max.trace)] <- ratio.max
    mhat.p <- approx(tau, quant.o.c, tau.m.p, rule = 2, ties = mean)$y * delta.m
  } else {
    delta.m <- m.p.eval - approx(tau, quant.m.c, tau.m.p, rule = 2, ties = mean)$y
    mhat.p <- approx(tau, quant.o.c, tau.m.p, rule = 2, ties = mean)$y + delta.m
  }
  
  mhat.c <- approx(quant.m.c, quant.o.c, m.c.eval, rule = 2, ties = mean)$y
  
  
  if (ratio) {
    # 0 limit
    mhat.c[mhat.c < trace] <- 0
    mhat.p[mhat.p < trace] <- 0
    # 1 (or max) limit
    mhat.c[(max_snc - mhat.c) < trace] <- max_snc
    mhat.p[(max_snc - mhat.p) < trace] <- max_snc
    
    # 0 in original series (gets inflated because of runif above)
    # not necessarily, BC can also "add" SNC
    # mhat.c[m.c.eval < trace] <- 0
    # mhat.p[m.p.eval < trace] <- 0
  }
  
  
  # if (ECBC) {
  #   if (length(mhat.p) == length(o.c)) {
  #     mhat.p <- sort(mhat.p)[rank(o.c, ties.method = ties)]
  #   }
  #   else {
  #     stop("Schaake shuffle failed due to incompatible lengths")
  #   }
  # }
  list(mhat.c = mhat.c, mhat.p = mhat.p)
}
