library(magrittr)
library(fs)
library(data.table)
library(raster)

calc_clim_ds <- function(path_in, 
                         file_out, 
                         path_temp_proc = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/temp_r_proc/"){
  
  if(file_exists(file_out)) return("File exists")
  
  fns <- dir_ls(path_in)
  dt_fns <- data.table(fn = fns)
  dt_fns[, year_month := fn %>% 
           path_file %>% 
           path_ext_remove]
  dt_fns[, c("year", "month") := tstrsplit(year_month, "-", type.convert = T)]
  mitmatmisc::add_hydro_year(dt_fns)
  # dt_fns[, .N, hydro_year]
  # dt_fns_hydro <- dt_fns[hydro_year %in% 2001:2020]
  
  dir_create(path_temp_proc)
  
  for(i_year in unique(dt_fns$year)){
    
    fn_out <- path(path_temp_proc, i_year, ext = "tif")
    if(file_exists(fn_out)) next
    fns_i <- dt_fns[year == i_year, fn]
    rrs_i <- stack(fns_i)
    rrs_i_out <- calc(rrs_i, sum)
    writeRaster(rrs_i_out, fn_out)
    
  }
  
  rrs <- stack(dir_ls(path_temp_proc))
  rrs_clim <- calc(rrs, mean)
  rrs_clim[rrs_clim < 0] <- NA
  
  # plot(rrs_clim)
  
  writeRaster(rrs_clim, filename = file_out)
  dir_delete(path_temp_proc)
  
}

calc_clim_ds_hydroyear <- function(path_in, 
                                   file_out, 
                                   path_temp_proc = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/temp_r_proc/"){
  
  if(file_exists(file_out)) return("File exists")
  
  fns <- dir_ls(path_in)
  dt_fns <- data.table(fn = fns)
  dt_fns[, year_month := fn %>% 
           path_file %>% 
           path_ext_remove]
  dt_fns[, c("year", "month") := tstrsplit(year_month, "-", type.convert = T)]
  mitmatmisc::add_hydro_year(dt_fns, 8)
  # dt_fns[, .N, hydro_year]
  # dt_fns_hydro <- dt_fns[hydro_year %in% 2001:2020]
  
  dir_create(path_temp_proc)
  
  for(i_hydro_year in unique(dt_fns$hydro_year)){
    
    fn_out <- path(path_temp_proc, i_hydro_year, ext = "tif")
    if(file_exists(fn_out)) next
    fns_i <- dt_fns[hydro_year == i_hydro_year, fn]
    rrs_i <- stack(fns_i)
    rrs_i_out <- calc(rrs_i, sum)
    writeRaster(rrs_i_out, fn_out)
    
  }
  
  rrs <- stack(dir_ls(path_temp_proc))
  rrs_clim <- calc(rrs, mean)
  rrs_clim[rrs_clim < 0] <- NA
  
  # plot(rrs_clim)
  
  writeRaster(rrs_clim, filename = file_out)
  dir_delete(path_temp_proc)
  
}