library(magrittr)
library(fs)
library(data.table)
library(raster)

season_month <- list(DJF = c(12, 1,2),
                     MAM = 3:5,
                     JJA = 6:8,
                     SON = 9:11)

calc_clim_ds_seasonal <- function(path_in, 
                                  file_out, 
                                  path_temp_proc = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/temp_r_proc/",
                                  season){
  
  if(file_exists(file_out)) return("File exists")
  
  fns <- dir_ls(path_in)
  dt_fns <- data.table(fn = fns)
  dt_fns[, year_month := fn %>% 
           path_file %>% 
           path_ext_remove]
  dt_fns[, c("year", "month") := tstrsplit(year_month, "-", type.convert = T)]
  mitmatmisc::add_hydro_year(dt_fns)
  # dt_fns[, .N, hydro_year]
  # dt_fns_hydro <- dt_fns[hydro_year %in% 2001:2020]
  
  # subset to season
  dt_fns <- dt_fns[month %in% season_month[[season]]]
  
  dir_create(path_temp_proc)
  
  for(i_year in unique(dt_fns$year)){
    
    fn_out <- path(path_temp_proc, i_year, ext = "tif")
    if(file_exists(fn_out)) next
    fns_i <- dt_fns[year == i_year, fn]
    rrs_i <- stack(fns_i)
    rrs_i_out <- calc(rrs_i, sum)
    writeRaster(rrs_i_out, fn_out)
    
  }
  
  rrs <- stack(dir_ls(path_temp_proc))
  rrs_clim <- calc(rrs, mean)
  rrs_clim[rrs_clim < 0] <- NA
  
  # plot(rrs_clim)
  
  writeRaster(rrs_clim, filename = file_out)
  dir_delete(path_temp_proc)
  
}

calc_clim_ds_seasonal_hydro_year <- function(path_in, 
                                             file_out, 
                                             path_temp_proc = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/temp_r_proc/",
                                             season,
                                             na.rm = F){
  
  if(file_exists(file_out)) return("File exists")
  
  fns <- dir_ls(path_in)
  dt_fns <- data.table(fn = fns)
  dt_fns[, year_month := fn %>% 
           path_file %>% 
           path_ext_remove]
  dt_fns[, c("year", "month") := tstrsplit(year_month, "-", type.convert = T)]
  mitmatmisc::add_hydro_year(dt_fns, 8)
  # dt_fns[, .N, hydro_year]
  # dt_fns_hydro <- dt_fns[hydro_year %in% 2001:2020]
  
  # subset to season
  dt_fns <- dt_fns[month %in% season_month[[season]]]
  
  # since JJA over hydro_year month change, needs actual year for seasonal value
  if(season == "JJA"){
    dt_fns[, hydro_year := year]
  } 
  
  # complete seasons
  dt_fns[, nn := .N, hydro_year]
  dt_fns <- dt_fns[nn == 3]
  
  dir_create(path_temp_proc)
  
  for(i_hydro_year in unique(dt_fns$hydro_year)){
    
    fn_out <- path(path_temp_proc, i_hydro_year, ext = "tif")
    if(file_exists(fn_out)) next
    fns_i <- dt_fns[hydro_year == i_hydro_year, fn]
    rrs_i <- stack(fns_i)
    rrs_i_out <- calc(rrs_i, sum, na.rm = na.rm)
    writeRaster(rrs_i_out, fn_out)
    
  }
  
  rrs <- stack(dir_ls(path_temp_proc))
  rrs_clim <- calc(rrs, mean, na.rm = na.rm)
  rrs_clim[rrs_clim < 0] <- NA
  
  # plot(rrs_clim)
  
  writeRaster(rrs_clim, filename = file_out)
  dir_delete(path_temp_proc)
  
}