# longest subset a monotonic sequence
# not necessarily unique, but maximum length


# from:
# https://www.r-bloggers.com/2014/09/compute-longest-increasingdecreasing-subsequence-using-rcpp/


library(Rcpp)
# cppcode <- "n#include <vector>n#include <Rcpp.h>nusing namespace std;ntypedef int index_type;nn// [[Rcpp::export]]nvector<index_type> longest_subseq(SEXP X){n    Rcpp::NumericVector x(X);ntvector<index_type> P(x.size(), 0);ntvector<index_type> M(x.size()+1, 0);ntindex_type L(0), newL;ntfor(index_type i=0; i < x.size(); ++i) {nttindex_type lo(1), hi(L), mid;nttwhile( lo <= hi) {ntttmid = (lo + hi) / 2;ntttif (x[M[mid]] < x[i] ) {nttttlo = mid + 1;nttt} else {ntttthi = mid - 1;nttt}ntt}nttnewL = lo;nttP[i] = M[newL - 1];nttif (newL > L) {ntttM[newL] = i;ntttL = newL;ntt} else if (x[i] < x[M[newL]]) {ntttM[newL] = i;ntt}nt}n    vector<index_type> re(L);ntindex_type k(M[L]);ntfor(index_type i=L-1; i>=0; --i){nttre[i] = k + 1;nttk = P[k];nt}n    return(re);n}"
sourceCpp(file = "R/functions/longest_subseq.cpp")


# longest_subseq_na <- function(x){
#   
#   x_new <- x
#   x_i <- longest_subseq(x)
#   x_new[-x_i] <- NA
#   x_new
#   
# }

monotonic_cp <- function(x){
  
  # filter NAs
  i_na <- is.na(x)
  x_na_rm <- x[!i_na]
  
  # vec of monotonic values (filled with NA, where not)
  x_i <- longest_subseq(x_na_rm)
  x_na_rm[-x_i] <- NA
  
  # output same length as input 
  x_new <- x
  x_new[!i_na] <- x_na_rm
  x_new
  
}


monotonic_cp_buffer <- function(x, buf = 0.01){
  
  # make NAs in original file
  x[x < 0] <- NA
  
  # buffer nearly 1s and 0s
  x[x < buf] <- 0
  x[(1-x) < buf] <- 1
  
  # filter NAs
  i_na <- is.na(x)
  x_na_rm <- x[!i_na]
  
  # vec of monotonic values (filled with NA, where not)
  x_i <- longest_subseq(x_na_rm)
  x_na_rm[-x_i] <- NA
  
  # output same length as input 
  x_new <- x
  x_new[!i_na] <- x_na_rm
  x_new
  
}
