# taken from package qmap
# functions fitQmapQUANT and doQmapQUANT


fit_QM_snc <- function(obs, mod, wet.day = TRUE, qstep = 0.01, nboot = 1, ...){
  ys <- na.omit(obs)
  xs <- na.omit(mod)
  
  if (length(xs) != length(ys)) {
    hn <- min(length(xs), length(ys))
    ys <- quantile(ys, seq(0, 1, length.out = hn), type = 8)
    xs <- quantile(xs, seq(0, 1, length.out = hn), type = 8)
  } else {
    xs <- sort(xs)
    ys <- sort(ys)
  }
  
  if (is.numeric(wet.day)) {
    q0 <- ys >= wet.day
    ys <- ys[q0]
    xs <- xs[q0]
  } else if (is.logical(wet.day)) {
    if (wet.day) {
      q0 <- ys > 0
      ys <- ys[q0]
      xs <- xs[q0]
      wet.day <- xs[1]
      names(wet.day) <- NULL
    } else {
      wet.day <- NULL
    }
  } else stop("'wet.day' should be 'numeric' or 'logical'")
  
  nn <- length(ys)
  newx <- quantile(xs, probs = seq(0, 1, by = qstep), type = 8)
  fit <- array(NA, dim = c(length(newx), 2, nboot))
  if (nboot > 1) {
    booty <- replicate(nboot, sample(ys, size = nn, replace = FALSE))
    booty <- apply(booty, 2, quantile, probs = seq(0, 1, 
                                                   by = qstep), type = 8)
    booty <- rowMeans(booty, na.rm = TRUE)
  }
  else {
    booty <- quantile(ys, probs = seq(0, 1, by = qstep), type = 8)
  }
  ppar <- list(modq = matrix(newx, ncol = 1), fitq = matrix(booty, ncol = 1))
  op <- list(par = ppar, wet.day = wet.day)
  class(op) <- "fitQmapQUANT"
  return(op)
}




do_QM_snc <- function (x, fobj, type = c("linear", "tricub"), ...) {
  
  type <- match.arg(type)
  wet <- if (!is.null(fobj$wet.day)) {
    x >= fobj$wet.day
  } else {
    rep(TRUE, length(x))
  }
  out <- rep(NA, length.out = length(x))
  if (type == "linear") {
    out[wet] <- approx(x = fobj$par$modq[, 1], y = fobj$par$fitq[, 1],
                       xout = x[wet], method = "linear", rule = 2, 
                       ties = mean)$y
    nq <- nrow(fobj$par$modq)
    largex <- x > fobj$par$modq[nq, 1]
    if (any(largex)) {
      max.delta <- fobj$par$modq[nq, 1] - fobj$par$fitq[nq, 1]
      out[largex] <- x[largex] - max.delta
    }
  } else if (type == "tricub") {
    sfun <- splinefun(x = fobj$par$modq[, 1], y = fobj$par$fitq[, 1], method = "monoH.FC")
    out[wet] <- sfun(x[wet])
  }
  out[!wet] <- 0
  if (!is.null(fobj$wet.day)) out[out < 0] <- 0
  
  return(out)
}

