# plot time series of SNC changes
# BC data 0.11

library(fs)
library(data.table)
library(foreach)
library(magrittr)
library(ggplot2)
library(forcats)
library(flextable)
library(officer)

load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/clim-02.rda")
rcm_short <- readRDS("data/auxiliary-01-rcm-names.rds")

dat_aux <- readRDS("data/auxiliary-03-modis.rds")

# alpine wide avg ---------------------------------------------------------

# only take pixels in common to all RCMs
dat_th <- readRDS("data/auxiliary-02-snow-max-threshold.rds")
dat_th[snow_plausible == "yes" & 
         !rcm_short %in% c("WRF331F", "HIRHAM5", "RACMO22E")] %>% 
  dcast(icell ~ rcm_short, value.var = "snow_plausible") %>% 
  na.omit %>% 
  .[, icell] -> icell_common_rcm
icell_modis <- sort(unique(clim_modis$icell))
icell_common <- intersect(icell_common_rcm, icell_modis)


# add
clim_modis[, season := mitmatmisc::season_fct(month)]
clim_qdm[, season := mitmatmisc::season_fct(month)]
clim_qm[, season := mitmatmisc::season_fct(month)]
dat_delta2[, season := mitmatmisc::season_fct(month)]

dat_aux[icell %in% icell_common, alt] %>% qplot(binwidth = 100)
dat_aux[, alt_f := cut(alt, breaks = c(-10, seq(200, 2600, by = 200), 3000), dig.lab = 5)]

clim_modis2 <- merge(clim_modis[icell %in% icell_common], 
                    dat_aux, by = "icell")
dat_delta22 <- merge(dat_delta2[icell %in% icell_common], 
                   dat_aux, by = "icell")
clim_qm2 <- merge(clim_qm[icell %in% icell_common], 
                   dat_aux, by = "icell")
clim_qdm2 <- merge(clim_qdm[icell %in% icell_common], 
                  dat_aux, by = "icell")


dt_modis <- clim_modis2[season %in% c("DJF", "MAM"), 
                       .(snc = mean(snc),
                         nn = .N),
                       .(alt_f, season)]

dt_qdm <- clim_qdm2[season %in% c("DJF", "MAM"), 
                   .(snc = mean(snc_qdm),
                     nn = .N), 
                   .(alt_f, season, rcm, gcm, experiment, fp)]

dt_qm <- clim_qm2[season %in% c("DJF", "MAM"), 
                  .(snc = mean(snc_qm),
                    nn = .N), 
                  .(alt_f, season, rcm, gcm, experiment, fp)]

dt_dc <- dat_delta22[season %in% c("DJF", "MAM"),
                    .(snc = mean(snc_future_delta),
                      nn = .N,
                      fp = "future"),
                    .(alt_f, season, rcm, gcm, experiment)]

dt_raw <- clim_qdm2[season %in% c("DJF", "MAM"), 
                   .(snc = mean(snc),
                     nn = .N), 
                   .(alt_f, season, rcm, gcm, experiment, fp)]



dat_full <- rbindlist(list(
  # modis = dt_modis,
  # mbc = dt_mbc,
  raw = dt_raw,
  qdm = dt_qdm,
  qm = dt_qm,
  dc = dt_dc
), use.names = T, fill = T, idcol = "ff")

dat_full[, rcm_short := rcm_short[rcm]]

# WRF331F not anymore on ESGF
# RACMO has so many "bad" cells
dat_full <- dat_full[is.na(rcm_short) |
                       !rcm_short %in% c("WRF331F", "RACMO22E")] 
# plot -------------------------------------------------------------


# dat_plot <- dt_qdm[!(fp == "past" & experiment == "rcp26")]
dat_plot <- dat_full[fp == "future"]
dat_plot[, experiment_f := ifelse(experiment == "rcp26", "RCP2.6", "RCP8.5")]
dat_plot[, lbl_time := ifelse(fp == "past", 
                              "2000-2020",
                              paste0("2071-2100: ", experiment_f))]
dat_plot[, lbl_bc := factor(toupper(ff), levels = toupper(c("raw", "dc", "qm", "qdm")))]



gg <-
dat_plot %>% droplevels %>% 
  ggplot(aes(snc, alt_f))+
  geom_point(data = droplevels(dt_modis), aes(shape = "MODIS"))+
  geom_line(aes(colour = rcm_short, linetype = gcm, group = paste0(rcm,gcm)))+
  scale_color_brewer("RCM", palette = "Set1")+
  scale_linetype("GCM")+
  scale_shape("2000-2020 Obs.")+
  facet_grid(lbl_time + season ~ lbl_bc)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("Snow cover fraction [0-1]")+
  ylab("Elevation [m]")



ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/bc-elev-season.png",
       gg, 
       width = 12,
       height = 11)



# plot diff ---------------------------------------------------------------

# diff to modis

dat_plot_diff <- dat_plot %>%
  merge(dt_modis[, .(season, alt_f, snc_ref = snc)])

# or diff to past?
# dat_plot_diff <- dat_plot[fp == "future"] %>%
#   merge(dat_plot[fp == "past", .(ff, month_fct, rcm, gcm, snc_ref = snc)])

dat_plot_diff[, snc_diff := snc - snc_ref]
dat_plot_diff[, snc_ratio := snc / snc_ref]

gg <-
dat_plot_diff %>%
  ggplot(aes(snc_diff, alt_f))+
  geom_vline(xintercept = 0)+
  geom_path(aes(colour = rcm_short, linetype = gcm, group = paste0(rcm,gcm)))+
  scale_color_brewer("RCM", palette = "Set1")+
  scale_linetype("GCM")+
  facet_grid(experiment_f + season ~ lbl_bc)+
  # scale_y_continuous(breaks = seq(-1, 1, by = 0.2))+
  # facet_grid(experiment_f + season ~ lbl_bc, scales = "free_y", space = "free_y")+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("Absolute change in snow cover fraction")+
  ylab("Elevation [m]")


ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/bc-elev-season-diff.png",
       gg, 
       width = 12,
       height = 11)





# table -------------------------------------------------------------------

dat_table <- dat_plot[,
         .(mod_mean = mean(snc),
           mod_min = min(snc),
           mod_max = max(snc),
           mod_n = .N),
         .(alt_f, season, lbl_bc, experiment_f)]

dat_table[, mmm := sprintf("%2.1f (%2.1f, %2.1f)", 100*mod_mean, 100*mod_min, 100*mod_max)]

dat_table2 <- dcast(dat_table,
                    alt_f + season ~ lbl_bc + experiment_f, 
                    value.var = "mmm")

setorder(dat_table2, season, -alt_f)
dat_table2 %>% 
  flextable() %>% 
  fontsize(size = 6) %>% 
  autofit



# table diff --------------------------------------------------------------


dat_table_diff <- dat_plot_diff[,
                                .(mod_mean = mean(snc_diff),
                                  mod_min = min(snc_diff),
                                  mod_max = max(snc_diff),
                                  mod_n = .N),
                                .(alt_f, season, lbl_bc, experiment_f)]

dat_table_diff[, mmm := sprintf("%2.1f (%2.1f, %2.1f)", 100*mod_mean, 100*mod_min, 100*mod_max)]

dat_table_diff2 <- dcast(dat_table_diff,
                         season + experiment_f + alt_f ~ lbl_bc, 
                         value.var = "mmm")

setorder(dat_table_diff2, season, experiment_f, -alt_f)

dat_table_diff2[season == "DJF"] %>% 
  dplyr::select(-season) %>% 
  flextable() %>% 
  set_header_labels(experiment_f = "Scenario",
                    alt_f = "Elevation [m]") %>% 
  merge_v(j = "experiment_f") %>% 
  valign(j = "experiment_f", valign  = "top") %>% 
  fix_border_issues() %>% 
  fontsize(size = 8, part = "all") %>% 
  autofit -> ft


read_docx() %>% 
  body_add_flextable(ft) %>% 
  print(target = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/bc-elev-season-diff-DJF.docx")



dat_table_diff2[season == "MAM"] %>% 
  dplyr::select(-season) %>% 
  flextable() %>% 
  set_header_labels(experiment_f = "Scenario",
                    alt_f = "Elevation [m]") %>% 
  merge_v(j = "experiment_f") %>% 
  valign(j = "experiment_f", valign  = "top") %>% 
  fix_border_issues() %>% 
  fontsize(size = 8, part = "all") %>% 
  autofit -> ft


read_docx() %>% 
  body_add_flextable(ft) %>% 
  print(target = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/bc-elev-season-diff-MAM.docx")



# numbers -----------------------------------------------------------------

dt_modis[, weighted.mean(snc, nn), season]
dt_modis[, round(100*weighted.mean(snc, nn)), season]
dt_modis[, round(100*weighted.mean(snc, nn), 1), season]


# model means
dat_full[,
         .(alps_avg = weighted.mean(snc, nn)),
         .(ff, season, fp, rcm, gcm, experiment)] %>% 
  .[,
    .(mod_mean = round(100*mean(alps_avg),1),
      mod_min = round(100*min(alps_avg),1),
      mod_max = round(100*max(alps_avg),1),
      mod_n = .N),
    .(ff, season, fp)]




# differences
dat_plot_diff[,
              .(alps_avg = weighted.mean(snc_diff, nn)),
              .(lbl_time, season, lbl_bc, rcm, gcm, experiment)] %>% 
  .[,
    .(diff_mod_mean = round(100*mean(alps_avg),1),
      diff_mod_min = round(100*min(alps_avg),1),
      diff_mod_max = round(100*max(alps_avg),1),
      diff_mod_n = .N),
    .(lbl_time, season, lbl_bc)] 
