# plot time series of SNC changes
# BC data 0.11

library(fs)
library(data.table)
library(foreach)
library(magrittr)
library(ggplot2)
library(forcats)

library(flextable)


load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/clim-02.rda")
rcm_short <- readRDS("data/auxiliary-01-rcm-names.rds")

dat_aux <- readRDS("data/auxiliary-03-modis.rds")

# alpine wide avg ---------------------------------------------------------

# only take pixels in common to all RCMs
dat_th <- readRDS("data/auxiliary-02-snow-max-threshold.rds")
dat_th[snow_plausible == "yes" & 
         !rcm_short %in% c("WRF331F", "HIRHAM5", "RACMO22E")] %>% 
  dcast(icell ~ rcm_short, value.var = "snow_plausible") %>% 
  na.omit %>% 
  .[, icell] -> icell_common_rcm
icell_modis <- sort(unique(clim_modis$icell))
icell_common <- intersect(icell_common_rcm, icell_modis)


# add
# clim_modis[, season := mitmatmisc::season_fct(month)]
# clim_qdm[, season := mitmatmisc::season_fct(month)]
# clim_qm[, season := mitmatmisc::season_fct(month)]

dat_aux[icell %in% icell_common, alt] %>% qplot(binwidth = 100)
dat_aux[, alt_f := cut(alt, breaks = c(-10, seq(200, 2600, by = 100), 3000), dig.lab = 5)]

clim_modis2 <- merge(clim_modis[icell %in% icell_common], 
                    dat_aux, by = "icell")
dat_delta22 <- merge(dat_delta2[icell %in% icell_common], 
                   dat_aux, by = "icell")
clim_qm2 <- merge(clim_qm[icell %in% icell_common], 
                   dat_aux, by = "icell")
clim_qdm2 <- merge(clim_qdm[icell %in% icell_common], 
                  dat_aux, by = "icell")


dt_modis <- clim_modis2[, 
                       .(snc = mean(snc)),
                       .(alt_f)]

dt_qdm <- clim_qdm2[, 
                   .(snc = mean(snc_qdm)), 
                   .(alt_f, rcm, gcm, experiment, fp)]

dt_qm <- clim_qm2[, 
                  .(snc = mean(snc_qm)), 
                  .(alt_f, rcm, gcm, experiment, fp)]

dt_dc <- dat_delta22[,
                    .(snc = mean(snc_future_delta), fp = "future"),
                    .(alt_f, rcm, gcm, experiment)]

dt_raw <- clim_qdm2[, 
                   .(snc = mean(snc)), 
                   .(alt_f, rcm, gcm, experiment, fp)]



dat_full <- rbindlist(list(
  # modis = dt_modis,
  # mbc = dt_mbc,
  raw = dt_raw,
  qdm = dt_qdm,
  qm = dt_qm,
  dc = dt_dc
), use.names = T, fill = T, idcol = "ff")

dat_full[, rcm_short := rcm_short[rcm]]

# WRF331F not anymore on ESGF
# RACMO has so many "bad" cells
dat_full <- dat_full[is.na(rcm_short) |
                       !rcm_short %in% c("WRF331F", "RACMO22E")] 
# plot -------------------------------------------------------------


# dat_plot <- dt_qdm[!(fp == "past" & experiment == "rcp26")]
dat_plot <- dat_full[fp == "future"]
dat_plot[, experiment_f := ifelse(experiment == "rcp26", "RCP2.6", "RCP8.5")]
dat_plot[, lbl_time := ifelse(fp == "past", 
                              "2000-2020",
                              paste0("2071-2100: ", experiment_f))]
dat_plot[, lbl_bc := factor(toupper(ff), levels = toupper(c("raw", "dc", "qm", "qdm")))]



gg <-
dat_plot %>% droplevels %>% 
  ggplot(aes(snc, alt_f))+
  geom_point(data = droplevels(dt_modis), aes(shape = "MODIS"))+
  geom_line(aes(colour = rcm_short, linetype = gcm, group = paste0(rcm,gcm)))+
  scale_color_brewer("RCM", palette = "Set1")+
  scale_linetype("GCM")+
  scale_shape("2000-2020 Obs.")+
  facet_grid(lbl_time ~ lbl_bc)+
  xlim(0,1)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("Snow cover fraction [0-1]")+
  ylab("Elevation [m]")



ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/bc-elev-annual.png",
       gg, 
       width = 12,
       height = 6)



# plot diff ---------------------------------------------------------------

# diff to modis

dat_plot_diff <- dat_plot %>%
  merge(dt_modis[, .(alt_f, snc_ref = snc)])

# or diff to past?
# dat_plot_diff <- dat_plot[fp == "future"] %>%
#   merge(dat_plot[fp == "past", .(ff, month_fct, rcm, gcm, snc_ref = snc)])

dat_plot_diff[, snc_diff := snc - snc_ref]
dat_plot_diff[, snc_ratio := snc / snc_ref]

gg <-
dat_plot_diff %>%
  ggplot(aes(snc_diff, alt_f))+
  geom_vline(xintercept = 0)+
  geom_path(aes(colour = rcm_short, linetype = gcm, group = paste0(rcm,gcm)))+
  scale_color_brewer("RCM", palette = "Set1")+
  scale_linetype("GCM")+
  facet_grid(experiment_f ~ lbl_bc)+
  # scale_y_continuous(breaks = seq(-1, 1, by = 0.2))+
  # facet_grid(experiment_f + season ~ lbl_bc, scales = "free_y", space = "free_y")+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("Absolute change in snow cover fraction")+
  ylab("Elevation [m]")


ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/bc-elev-annual-diff.png",
       gg, 
       width = 12,
       height = 6)


# numbers -----------------------------------------------------------------


num_qdm <- clim_qdm2[!rcm %in% c("IPSL-INERIS-WRF331F", "KNMI-RACMO22E"), 
                     .(snc_bc = mean(snc_qdm)),
                     .(gcm, rcm, experiment, fp)]
modis_avg <- clim_modis2[, mean(snc)]
num_qdm[, snc_bc_diff := snc_bc - modis_avg]
num_qdm[, snc_bc_diff := (snc_bc - modis_avg)/modis_avg]

num_qdm[, .(mean(snc_bc_diff), min(snc_bc_diff), max(snc_bc_diff)), .(fp, experiment)] %>% 
  flextable


modis_avg * 0.49138 * 365

num_qdm[, .(mean(snc_bc), min(snc_bc), max(snc_bc)), .(fp, experiment)] %>% 
  flextable
