# plot ST subset of CP

# library(raster)
library(stars)
library(sf)
library(data.table)
library(ggplot2)
library(magrittr)
library(cowplot)

rs_cp <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/cp_20bins.tif")

# extent ST
sf_ita <- read_sf("/mnt/CEPH_BASEDATA/GIS/NATIONAL/ITALY/BOUNDARIES/IT_bnd_prov_pol_ISTAT_2011.shp")
sf_bz <- sf_ita %>% dplyr::filter(NOME_PRO == "BOLZANO/BOZEN")
sf_bz_rs <- st_transform(sf_bz, st_crs(rs_cp))

rs_cp_crop <- st_crop(rs_cp, sf_bz_rs)
rs_plot <- st_as_stars(rs_cp_crop)

# 20bin
cut(0:100/100, breaks = c(0, 0.001, 1:19/20, 0.999, 1), include.lowest = T) %>% 
  levels -> band_lbl
rs_plot <- rs_plot %>% 
  st_set_dimensions("band", values = band_lbl)

# ggplot()+
#   geom_stars(data = rs_plot, downsample = c(3,3,2.5), na.rm = T)+
#   scale_fill_viridis_c("CP", limits = c(0,1), na.value = "white")+
#   facet_wrap(~band)+
#   coord_equal()+
#   theme_void()

# cp plot

gg_cp <- ggplot()+
  geom_stars(data = rs_plot, downsample = c(1, 1, 2.5), na.rm = T)+
  scale_fill_viridis_c("P(hr snow | lr SCF)", limits = c(0,1), na.value = "white")+
  facet_wrap(~band)+
  coord_equal()+
  theme_void()


# elev

rs_elev <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rs_elev[rs_elev < -30000] <- NA
rs_water <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rs_elev[rs_water == 1] <- NA

rs_elev_plot <- st_as_stars(st_crop(rs_elev, 
                                    st_transform(sf_bz, st_crs(rs_elev))))

gg_elev <- ggplot()+
  geom_stars(data = rs_elev_plot, downsample = 0, na.rm = T)+
  scale_fill_scico("m.a.s.l.", 
                   palette = "oleron", na.value = NA, rescaler = scales::rescale_mid)+
  coord_equal()+
  theme_void()



gg_out <- plot_grid(gg_elev, gg_cp,
                    labels = paste0("(", c("a", "b"), ")"),
                    rel_widths = c(1.5, 3))


ggsave(gg_out,
       filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/cp-southtyrol.png",
       width = 12, height = 4)
