# plot time series of SNC changes
# BC data 0.11

library(fs)
library(data.table)
library(foreach)
library(magrittr)
library(ggplot2)
library(forcats)


load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/clim-02.rda")
rcm_short <- readRDS("data/auxiliary-01-rcm-names.rds")

dat_aux <- readRDS("data/auxiliary-03-modis.rds")

# alpine wide avg ---------------------------------------------------------

# only take pixels in common to all RCMs
dat_th <- readRDS("data/auxiliary-02-snow-max-threshold.rds")
dat_th[snow_plausible == "yes" & 
         !rcm_short %in% c("WRF331F", "HIRHAM5", "RACMO22E")] %>% 
  dcast(icell ~ rcm_short, value.var = "snow_plausible") %>% 
  na.omit %>% 
  .[, icell] -> icell_common_rcm
icell_modis <- sort(unique(clim_modis$icell))
icell_common <- intersect(icell_common_rcm, icell_modis)

# icell_common <- icell_modis # for testing

dt_modis <- clim_modis[icell %in% icell_common, 
                       .(snc = mean(snc)),
                       month]
# dt_mbc <- clim_mbc[icell %in% icell_common, 
#                    .(snc = mean(snc_mbc)), 
#                    .(month, rcm, gcm, experiment, fp)]
dt_qdm <- clim_qdm[icell %in% icell_common, 
                   .(snc = mean(snc_qdm)), 
                   .(month, rcm, gcm, experiment, fp)]
dt_qm <- clim_qm[icell %in% icell_common,
                 .(snc = mean(snc_qm)),
                 .(month, rcm, gcm, experiment, fp)]
dt_dc <- dat_delta2[icell %in% icell_common,
                    .(snc = mean(snc_future_delta), fp = "future"),
                    .(month, rcm, gcm, experiment)]
dt_raw <- clim_qdm[icell %in% icell_common, 
                   .(snc = mean(snc)), 
                   .(month, rcm, gcm, experiment, fp)]

dat_full <- rbindlist(list(
  # modis = dt_modis,
  # mbc = dt_mbc,
  raw = dt_raw,
  qdm = dt_qdm,
  qm = dt_qm,
  dc = dt_dc
), use.names = T, fill = T, idcol = "ff")

mitmatmisc::add_month_fct(dat_full, 10)
dat_full[, rcm_short := rcm_short[rcm]]

# WRF331F not anymore on ESGF
# RACMO has so many "bad" cells
dat_full <- dat_full[is.na(rcm_short) |
                       !rcm_short %in% c("WRF331F", "RACMO22E")] 


# plot -------------------------------------------------------------

dat_full[fp == "past", .N, ff]

dat_plot <- dat_full[!(fp == "past" & experiment == "rcp26")]
dat_plot[, experiment_f := ifelse(experiment == "rcp26", "RCP2.6", "RCP8.5")]
dat_plot[, lbl_time := ifelse(fp == "past", 
                              "2000-2020",
                              paste0("2071-2100: ", experiment_f))]
dat_plot[, lbl_bc := factor(toupper(ff), levels = toupper(c("raw", "dc", "qm", "qdm")))]

mitmatmisc::add_month_fct(dt_modis, 10)

gg <- dat_plot %>% 
  ggplot(aes(month_fct, snc))+
  geom_point(data = dt_modis, aes(shape = "MODIS"))+
  geom_line(aes(colour = rcm_short, linetype = gcm, group = paste0(rcm,gcm)))+
  scale_color_brewer("RCM", palette = "Set1")+
  scale_linetype("GCM")+
  scale_shape("Observations")+
  facet_grid(lbl_time ~ lbl_bc, scales = "free_y", space = "free_y")+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab(NULL)+
  ylab("Mean snow cover fraction [0-1]")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/bc-alps-month.png",
       gg, 
       width = 16,
       height = 8)



# plot diff ---------------------------------------------------------------

# diff to modis 

dat_plot_diff <- dat_plot[fp == "future"] %>% 
  merge(dt_modis[, .(month_fct, snc_ref = snc)])

# or diff to past?
# dat_plot_diff <- dat_plot[fp == "future"] %>% 
#   merge(dat_plot[fp == "past", .(ff, month_fct, rcm, gcm, snc_ref = snc)])

dat_plot_diff[, snc_ratio := snc/snc_ref]

dat_plot_diff %>% 
  ggplot(aes(month_fct, snc_ratio))+
  geom_line(aes(colour = rcm_short, linetype = gcm, group = paste0(rcm,gcm)))+
  scale_color_brewer("RCM", palette = "Set1")+
  scale_linetype("GCM")+
  facet_grid(lbl_time ~ lbl_bc)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab(NULL)+
  ylab("Mean snow cover fraction [0-1]")



