# table and figure of plausible snow


library(foreach)
library(fs)
library(data.table)
library(magrittr)

library(ggplot2)
library(flextable)
library(officer)

# models used (from file)
rcm_used <- c("CLMcom-CCLM4-8-17", "CLMcom-ETH-COSMO-crCLIM-v1-1", "CNRM-ALADIN63",
              "IPSL-WRF381P", "KNMI-RACMO22E", "SMHI-RCA4")



rcm_short_names <- readRDS("data/auxiliary-01-rcm-names.rds")

dat_aux <- readRDS("data/auxiliary-03-modis.rds")

dat_th <- readRDS("data/auxiliary-02-snow-max-threshold.rds")


dat_th_aux <- merge(dat_th, dat_aux, by = "icell")
dat_th_aux[, rcm_short := rcm_short_names[rcm]]





gg_plaus <-
dat_th_aux[rcm %in% rcm_used] %>% 
  ggplot(aes(x, y, fill = snow_plausible))+
  geom_raster()+
  facet_wrap(~rcm_short)+
  theme_void()+
  theme(legend.position = "bottom")+
  coord_equal()

ggsave(gg_plaus, file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/snow-plausible.png",
       width = 7, height = 4, units = "in")

# table
dat_th_aux[rcm %in% rcm_used, .(.N), .(rcm, snow_plausible)] %>% 
  dcast(rcm ~ snow_plausible, value.var = "N") -> dat_th_table
dat_th_table[is.na(no), no := 0]
dat_th_table[, n_total := no + yes]
dat_th_table[, no_frac := no / n_total]

setorder(dat_th_table, rcm)

flextable(dat_th_table,
          col_keys = c("rcm", "n_total", "yes", "no", "no_frac")) %>%
  set_header_labels(rcm = "RCM",
                    n_total = "Total [#]",
                    yes = "Kept [#]",
                    no = "Removed [#]",
                    no_frac = "Removed, fraction") %>% 
  set_formatter(no_frac = scales::percent_format(accuracy = .1)) %>% 
  autofit() -> ft

read_docx() %>% 
  body_add_flextable(ft) %>% 
  print(target = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/snow-plausible.docx")
