# table of models used



library(foreach)
library(fs)
library(data.table)
library(magrittr)

library(ggplot2)
library(flextable)
library(officer)


# from inventory
dat_inv <- readRDS("data/eurocordex-inventory-v02/snc.rds")
dat_inv[, list_files := NULL]
dat_inv[experiment %in% c("rcp26", "rcp85"), .(institute_rcm, gcm, experiment)] %>% 
  unique


# from ds folder
dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full/02_qm/") %>% 
  path_file %>% 
  tstrsplit("_") %>% 
  as.data.table -> dat_info_ds
setnames(dat_info_ds, c("institute_rcm", "gcm", "experiment", 
                        "ensemble", "downscale_realisation", "fp"))

dat_info_ds[, .(institute_rcm, gcm, experiment)] %>% 
  unique



# take ds folder info -----------------------------------------------------

# remove WRF331F because superseded

dat_info_ds[institute_rcm != "IPSL-INERIS-WRF331F", 
            .(institute_rcm, rcm_short = rcm_short_names[institute_rcm], 
              gcm, experiment, available = "X")] %>% 
  unique %>% 
  dcast(institute_rcm + gcm ~ experiment, value.var = "available") -> dat_table



dat_table %>% 
  flextable() %>% 
  set_header_labels(institute_rcm = "RCM",
                    gcm = "GCM",
                    rcp26 = "RCP2.6",
                    rcp85 = "RCP8.5") %>% 
  autofit() -> ft

read_docx() %>% 
  body_add_flextable(ft) %>% 
  print(target = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/01-initial/models-used.docx")



# numbers -----------------------------------------------------------------

# rcp85
dat_table[, unique(institute_rcm)]
dat_table[, unique(gcm)]
dat_table %>% nrow

# rcp26
dat_table[!is.na(rcp26), unique(institute_rcm)]
dat_table[!is.na(rcp26), unique(gcm)]
dat_table[!is.na(rcp26)] %>% nrow
