# prep data for zenodo




library(fs)
library(data.table)
library(foreach)
library(magrittr)
library(raster)

# library(ggplot2)
# library(forcats)
# library(cowplot)



# settings ----------------------------------------------------------------

path_out <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/zenodo/data_biascorrection/"





# data --------------------------------------------------------------------


rr0 <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")

# rcm_short <- readRDS("data/auxiliary-01-rcm-names.rds")
dat_aux <- readRDS("data/auxiliary-03-modis.rds")


# export elev aux ---------------------------------------------------------

rr_elev <- rr0
rr_elev[dat_aux$icell] <- dat_aux$alt

fn_out <- path(path_out, "aux_elev", "elevation", ext = "tif")
dir_create(path_dir(fn_out))
writeRaster(rr_elev, fn_out, overwrite = T)


## 2071-2100 ---------------------------------------------------------------


load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/clim-02.rda")


dat_71 <- rbindlist(list(
  raw = clim_qdm[, .(icell, month, rcm, gcm, experiment, fp, snc = snc)],
  qdm = clim_qdm[, .(icell, month, rcm, gcm, experiment, fp, snc = snc_qdm)],
  qm = clim_qm[, .(icell, month, rcm, gcm, experiment, fp, snc = snc_qm)],
  dc = dat_delta2[, .(icell, month, rcm, gcm, experiment, fp = "future", snc = snc_future_delta)]
), use.names = T, fill = T, idcol = "bc_var")

dat_71[, period := "2071-2100"]

## 2041-2071 ---------------------------------------------------------------


load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full_2041_2070/clim-02.rda")


dat_41 <- rbindlist(list(
  raw = clim_qdm[, .(icell, month, rcm, gcm, experiment, fp, snc = snc)],
  qdm = clim_qdm[, .(icell, month, rcm, gcm, experiment, fp, snc = snc_qdm)],
  qm = clim_qm[, .(icell, month, rcm, gcm, experiment, fp, snc = snc_qm)],
  dc = dat_delta2[, .(icell, month, rcm, gcm, experiment, fp = "future", snc = snc_future_delta)]
), use.names = T, fill = T, idcol = "bc_var")

dat_41[, period := "2041-2071"]



## combine  ---------------------------------------------------------------


dat_bc <- rbind(dat_71, dat_41)

# WRF331F not anymore on ESGF
dat_bc <- dat_bc[rcm != "IPSL-INERIS-WRF331F"]

dat_loop_bc <- unique(dat_bc[, .(period, bc_var, experiment, fp, gcm, rcm)])

# export MODIS  -----------------------------------------------------------

# year

dt_year <- clim_modis[, 
                      .(snc = mean(snc)), 
                      .(icell)]

rr_out <- raster(rr0)
rr_out[dt_year$icell] <- dt_year$snc

fn_out <- path(path_out,
               "MODIS",
               "annual",
               "annual",
               ext = "tif")
dir_create(path_dir(fn_out))
writeRaster(rr_out, fn_out, overwrite = T)

# month

dt_month <- clim_modis[, 
                       .(snc = mean(snc)), 
                       .(icell, month)]

for(i in 1:12){
  
  rr_out <- raster(rr0)
  rr_out[dt_month[month == i, icell]] <- dt_month[month == i, snc]
  
  
  
  fn_out <- path(path_out,
                 "MODIS",
                 "month",
                 sprintf("%02i-%s", i, month.abb[i]),
                 ext = "tif")
  dir_create(path_dir(fn_out))
  writeRaster(rr_out, fn_out, overwrite = T)
  
  
  
}

# season

dt_season <- clim_modis[, 
                        .(snc = mean(snc)), 
                        .(icell, season = mitmatmisc::season_fct(month))]

all_seasons <- levels(dt_season$season)
for(i in seq_along(all_seasons)){
  
  rr_out <- raster(rr0)
  rr_out[dt_season[season == all_seasons[i], icell]] <- dt_season[season == all_seasons[i], snc]
  
  
  
  fn_out <- path(path_out,
                 "MODIS",
                 "season",
                 sprintf("%02i-%s", i, all_seasons[i]),
                 ext = "tif")
  dir_create(path_dir(fn_out))
  writeRaster(rr_out, fn_out, overwrite = T)
  
  
  
}


# export BC ---------------------------------------------------------------

# year

dt_year <- dat_bc[,
                  .(snc = mean(snc)),
                  .(period, bc_var, experiment, fp, gcm, rcm, icell)]

for(i in seq_len(nrow(dat_loop_bc))){
  
  dat_loop_bc[i, ] %>% 
    merge(dt_year) -> dat_i
  
  rr_out <- raster(rr0)
  rr_out[dat_i$icell] <- dat_i$snc
  
  fn_out <- dat_loop_bc[i, 
                        path(path_out, "annual",
                             period, bc_var, experiment, fp,
                             paste0(rcm, "_", gcm),
                             ext = "tif")]
  dir_create(path_dir(fn_out))
  
  writeRaster(rr_out, fn_out, overwrite = T)
  
}


# month

dt_month <- dat_bc[,
                   .(snc = mean(snc)),
                   .(month, period, bc_var, experiment, fp, gcm, rcm, icell)]

for(i in seq_len(nrow(dat_loop_bc))){
  
  for(i_month in 1:12){
    
    dat_loop_bc[i, ] %>% 
      merge(dt_month[month == i_month]) -> dat_i
    
    rr_out <- raster(rr0)
    rr_out[dat_i$icell] <- dat_i$snc
    
    fn_out <- dat_loop_bc[i, 
                          path(path_out, "month",
                               period, bc_var, experiment, fp,
                               paste0(rcm, "_", gcm),
                               sprintf("%02i-%s", i_month, month.abb[i_month]),
                               ext = "tif")]
    dir_create(path_dir(fn_out))
    
    writeRaster(rr_out, fn_out, overwrite = T)
    
    
    
  }
  
 
}


# season

dt_season <- dat_bc[,
                   .(snc = mean(snc)),
                   .(season = mitmatmisc::season_fct(month),
                     period, bc_var, experiment, fp, gcm, rcm, icell)]

for(i in seq_len(nrow(dat_loop_bc))){
  
  all_seasons <- levels(dt_season$season)
  for(i_season in seq_along(all_seasons)){
    
    dat_loop_bc[i, ] %>% 
      merge(dt_season[season == all_seasons[i_season]]) -> dat_i
    
    rr_out <- raster(rr0)
    rr_out[dat_i$icell] <- dat_i$snc
    
    fn_out <- dat_loop_bc[i, 
                          path(path_out, "season",
                               period, bc_var, experiment, fp,
                               paste0(rcm, "_", gcm),
                               sprintf("%02i-%s", i_season, all_seasons[i_season]),
                               ext = "tif")]
    dir_create(path_dir(fn_out))
    
    writeRaster(rr_out, fn_out, overwrite = T)
    
    
    
  }
  
  
}


