# ds bias: map and elev

library(stars)
library(raster)
library(data.table)
library(magrittr)
library(ggplot2)
library(cowplot)
library(foreach)
library(fs)
library(graticule)

library(flextable)
library(officer)


elev_int <- 100 

rcms_common <- c("CLMcom-CCLM4-8-17", "CLMcom-ETH-COSMO-crCLIM-v1-1",
                 "CNRM-ALADIN63", "IPSL-WRF381P", "SMHI-RCA4")

# BC data -----------------------------------------------------------------


load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/clim-02.rda")
dat_aux <- readRDS("data/auxiliary-03-modis.rds")


# only take pixels in common to all RCMs
dat_th <- readRDS("data/auxiliary-02-snow-max-threshold.rds")
dat_th[snow_plausible == "yes" & 
         rcm %in% rcms_common] %>% 
  dcast(icell ~ rcm_short, value.var = "snow_plausible") %>% 
  na.omit %>% 
  .[, icell] -> icell_common_rcm
icell_modis <- sort(unique(clim_modis$icell))
icell_common <- intersect(icell_common_rcm, icell_modis)



# icell means
clim_modis_icell <- clim_modis[icell %in% icell_common,
                               .(snc_modis = mean(snc)),
                               icell]

clim_qdm_icell <- clim_qdm[icell %in% icell_common & fp == "past" & rcm %in% rcms_common,
         .(snc = mean(snc_qdm)),
         .(icell, rcm, gcm, experiment, fp)] %>% 
  merge(clim_modis_icell, by = "icell")

clim_qdm_icell_ensmean <- clim_qdm_icell[, 
                                         .(snc = mean(snc),
                                           snc_modis = mean(snc_modis)),
                                         .(icell, experiment)]


# elev means
dat_aux[, alt_f := cut(alt, breaks = c(-10, seq(200, 2600, by = elev_int), 3000), dig.lab = 5)]

clim_qdm2 <- merge(clim_qdm[icell %in% icell_common], 
                   dat_aux, by = "icell")

dat_bc_elev_alps <- clim_qdm2[,
                              .(snc = mean(snc_qdm)),
                              .(alt_f, rcm, gcm, experiment, fp)]




# plot --------------------------------------------------------------------


clim_qdm_icell[fp == "past", qplot(snc - snc_modis)]
clim_qdm_icell_ensmean[, qplot(snc - snc_modis)]


clim_qdm_icell[experiment == "rcp85"] %>% 
  merge(dat_aux) %>% 
  ggplot(aes(x,y,fill = snc - snc_modis))+
  geom_raster()+
  scale_fill_gradient2()+
  # theme_bw()+
  facet_wrap(~rcm + gcm)

clim_qdm_icell[experiment == "rcp26"] %>% 
  merge(dat_aux) %>% 
  ggplot(aes(x,y,fill = (snc - snc_modis)/snc_modis))+
  geom_raster()+
  scale_fill_gradient2()+
  # theme_bw()+
  facet_wrap(~rcm + gcm)


clim_qdm_icell_ensmean %>% 
  merge(dat_aux, by = "icell") %>% 
  ggplot(aes(x,y,fill = snc - snc_modis))+
  geom_raster()+
  scale_fill_gradient2()+
  # theme_bw()+
  facet_wrap(~ experiment)

clim_qdm_icell_ensmean %>% 
  merge(dat_aux, by = "icell") %>% 
  ggplot(aes(x,y,fill = (snc - snc_modis)/snc_modis))+
  geom_raster()+
  scale_fill_gradient2()+
  # theme_bw()+
  facet_wrap(~ experiment)

# data --------------------------------------------------------------------


# maps
rr_ds <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/alps-clim/2000-10-01_2019-09-30.tif")

rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA

rr_modis <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")


rs_modis <- st_as_stars(rr_modis)
rs_ds <- st_as_stars(rr_ds)
rs_diff_abs <- rs_ds - rs_modis
rs_diff_rel <- (rs_ds - rs_modis)/rs_modis

# more than 100% -> 100%
rs_diff_rel[rs_diff_rel > 1] <- 1


# elev summary

dat_modis <- data.table(icell = 1:ncell(rr_modis),
                        scd_modis = rr_modis[],
                        scd_ds = rr_ds[],
                        elev = rr_elev[])

dat_modis[, elev_f := cut(elev, breaks = c(seq(-200, 4800, by = elev_int)), dig.lab = 5)]

dat_modis2 <- dat_modis[!is.na(scd_ds) & !is.na(scd_modis), 
                        .(scd_modis = mean(scd_modis),
                          scd_ds = mean(scd_ds),
                          nn = .N),
                        .(elev_f)]


# plot maps --------------------------------------------------------------------



prj <- st_crs(rs_modis)$proj4string

lons <- seq(10, 12, by = 2)
lats <- seq(46, 48, by = 1)
xl <- range(lons) + c(-0.4, 0.4)
yl <- range(lats) + c(-0.4, 0.4)
grat <- graticule(lons, lats, proj = prj, xlim = xl, ylim = yl)
labs <- graticule_labels(lons, lats, xline = min(xl), yline = max(yl), proj = prj)


# -> maps almost not distinguishable



# gg_map_modis <- 
# ggplot()+
#   geom_sf(data = st_as_sf(grat), colour = NA)+
#   geom_stars(data = rs_modis, downsample = ds_stars)+
#   scale_fill_scico("SCD [d]", limits = c(0, 366),
#                    palette = "davos", na.value = NA, direction = 1)+
#   coord_sf(expand = F)+
#   theme_minimal()+
#   xlab(NULL)+ylab(NULL)+
#   scale_x_continuous(labels = function(x) paste0(x, "° E"))+
#   scale_y_continuous(labels = function(x) paste0(x, "° N"))


# gg_map_ds <-
# ggplot()+
#   geom_sf(data = st_as_sf(grat), colour = NA)+
#   geom_stars(data = rs_ds, downsample = ds_stars)+
#   scale_fill_scico("SCD [d]", limits = c(0, 366),
#                    palette = "davos", na.value = NA, direction = 1)+
#   coord_sf(expand = F)+
#   theme_minimal()+
#   xlab(NULL)+ylab(NULL)+
#   scale_x_continuous(labels = function(x) paste0(x, "° E"))+
#   scale_y_continuous(labels = function(x) paste0(x, "° N"))



gg_map_diff_abs <-
  ggplot()+
  geom_sf(data = st_as_sf(grat), colour = NA)+
  geom_stars(data = rs_diff_abs, downsample = ds_stars)+
  scale_fill_scico("dSCD [d]", 
                   palette = "vik", na.value = NA, direction = -1,
                   # limits = c(-50, 50),
                   rescaler = scales::rescale_mid)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"))


gg_map_diff_rel <-
  ggplot()+
  geom_sf(data = st_as_sf(grat), colour = NA)+
  geom_stars(data = rs_diff_rel*100, downsample = ds_stars)+
  scale_fill_scico("dSCD [%]", 
                   palette = "vik", na.value = NA, direction = -1,
                   # limits = c(-1,1)*100,
                   rescaler = scales::rescale_mid)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"))



# 
# 
# gg_map_comb <- plot_grid(gg_map_hr, gg_map_lr,
#                          gg_map_diff_abs, gg_map_diff_rel,
#                          align = "hv",
#                          labels = c("(a)", "(b)", "(c)", "(d)"))
# 
# 
# ggsave(filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/scale-issues-02-map.png",
#        gg_map_comb,
#        width = 12,
#        height = 7)
# 



# plot elev ---------------------------------------------------------------

cols <- scales::brewer_pal(palette = "Set1")(4)[3:4]

dat_modis2[, elev := tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans]

dat_plot <- dat_modis2 %>% 
  melt(id.vars = c("elev", "nn"), measure.vars = c("scd_modis", "scd_ds"))

dat_plot[, variable_f := fct_recode(variable,
                                    "Original" = "scd_modis",
                                    "Downscaled" = "scd_ds")]
setorder(dat_plot, variable, elev)  


dat_plot_diff <- copy(dat_modis2)
dat_plot_diff[, diff_abs := scd_ds - scd_modis]
dat_plot_diff[, diff_rel := (scd_ds - scd_modis)/scd_modis]
setorder(dat_plot_diff, elev)  


gg_scd <- dat_plot %>% 
  ggplot(aes(value, elev, colour = variable_f))+
  geom_path()+
  scale_color_manual("", values = cols)+
  theme_bw()+
  theme(legend.position = c(0.8, 0.2),
        legend.background = element_rect(colour = "black"),
        legend.title = element_blank())+
  xlab("Average SCD [d]")+
  ylab("Elevation [m]")
  
  


gg_diff_abs <-
dat_plot_diff %>% 
  ggplot(aes(diff_abs, elev))+
  geom_vline(xintercept = 0, linetype = "dashed")+
  geom_path()+
  geom_point()+
  theme_bw()+
  xlab("Absolute difference in SCD (downscaled - original) [d]")+
  ylab("Elevation [m]")


gg_diff_rel <-
dat_plot_diff %>% 
  ggplot(aes(diff_rel, elev))+
  geom_vline(xintercept = 0, linetype = "dashed")+
  geom_path()+
  geom_point()+
  scale_x_continuous(labels = scales::percent_format())+
  theme_bw()+
  xlab("Relative difference in SCD (downscaled - original)")+
  ylab("Elevation [m]")




gg_comb <- plot_grid(plot_grid(gg_map_diff_abs, gg_map_diff_rel,
                               nrow = 1,
                               align = "hv",
                               labels = c("(a)", "(b)")),
                     plot_grid(gg_scd, gg_diff_abs, gg_diff_rel,
                               nrow = 1,
                               align = "hv",
                               labels = c("(c)", "(d)", "(e)")),
                     ncol = 1)


ggsave(filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/bias-past/ds-modis-map-elev.png",
       gg_comb,
       width = 14,
       height = 7)




# table and summary -------------------------------------------------------

dat_plot_diff[, .(elev_f, scd_modis, scd_ds, diff_abs, diff_rel = 100*diff_rel)] %>% 
  rbind(data.table(elev_f = "Average", 
                   scd_modis = dat_plot_diff[, weighted.mean(scd_modis, nn)],
                   scd_ds = dat_plot_diff[, weighted.mean(scd_ds, nn)],
                   diff_abs = dat_plot_diff[, weighted.mean(diff_abs, nn)],
                   diff_rel = 100*dat_plot_diff[, weighted.mean(diff_rel, nn)]),
        fill = T) %>% 
  flextable() %>% 
  colformat_double(j = c("scd_modis", "scd_ds"), digits = 1) %>% 
  colformat_double(j = c("diff_abs"), digits = 1) %>% 
  colformat_double(j = c("diff_rel"), digits = 1, suffix = "%") %>% 
  autofit() -> ft

read_docx() %>% 
  body_add_flextable(ft) %>% 
  print(target = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/bias-past/table-ds-modis-map-elev.docx")



