# check scale issues DS
# is there a difference between BA-DS-model and DS-(upscaled)-OBS

library(raster)
library(stars)
library(ggplot2)
library(data.table)
library(magrittr)
library(foreach)

# old cal year
# rr_obs <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")
# rr_ds_obs <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/_eval_upscaled_modis/alps-clim/2000-10-01_2019-09-30.tif")


# new hydro year
rr_obs <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_scd/clim/2000-08-01_2020-07-31.tif")
rr_ds_obs <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/alps-clim/2000-08-01_2020-07-31.tif")
rr_ds_mod <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_past/rcp26.tif")

# rr_ds_mod1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/03_qdm/CLMcom-CCLM4-8-17_ICHEC-EC-EARTH_rcp85_r12i1p1_v1_past.tif")
# rr_ds_mod2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/03_qdm/CLMcom-CCLM4-8-17_ICHEC-EC-EARTH_rcp26_r12i1p1_v1_past.tif")
# rr_ds_mod2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/03_qdm/SMHI-RCA4_ICHEC-EC-EARTH_rcp85_r3i1p1_v1_past.tif")
# plot(rr_ds_mod1-rr_ds_mod2)

# rr_ds_mod1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_past/rcp26.tif")
# rr_ds_mod2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_past/rcp85.tif")
# plot(rr_ds_mod1-rr_ds_mod2)


# future
# rr_ds_mod <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp26_noglacier.tif")
# rr_ds_mod <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp26.tif")
# rr_ds_mod <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp85.tif")


rs_diff1 <- st_as_stars(rr_ds_obs - rr_obs)
rs_diff1_rel <-  st_as_stars((rr_ds_obs - rr_obs)/rr_obs)


ggplot()+
  geom_stars(data = rs_diff1, downsample = 5)+
  scale_fill_gradient2()+
  theme_bw()

ggplot()+
  geom_stars(data = rs_diff1_rel, downsample = 5)+
  scale_fill_gradient2(limits = c(-1, 2))+
  theme_bw()



rs_diff2 <- st_as_stars(rr_ds_mod - rr_ds_obs)
rs_diff2_rel <-  st_as_stars((rr_ds_mod - rr_ds_obs)/rr_ds_obs)


ggplot()+
  geom_stars(data = rs_diff2, downsample = 5)+
  scale_fill_gradient2()+
  theme_bw()

ggplot()+
  geom_stars(data = rs_diff2_rel, downsample = 5)+
  scale_fill_gradient2(limits = c(NA, 2))+
  theme_bw()


# elev summary ------------------------------------------------------------


rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA


elev_int <- 100 

dat_eval <- data.table(icell = 1:ncell(rr_obs),
                        scd_obs = rr_obs[],
                        scd_ds_obs = rr_ds_obs[],
                        scd_ds_mod = rr_ds_mod[],
                        elev = rr_elev[])

dat_eval[, elev_f := cut(elev, breaks = c(seq(-200, 4800, by = elev_int)), dig.lab = 5)]

dat_eval2 <- dat_eval[!is.na(scd_ds_obs) & !is.na(scd_ds_mod) & !is.na(scd_obs), 
                        .(scd_obs = mean(scd_obs),
                          scd_ds_obs = mean(scd_ds_obs),
                          scd_ds_mod = mean(scd_ds_mod),
                          nn = .N),
                        .(elev_f)]

dat_eval2 %>% 
  melt(id.vars = c("elev_f", "nn")) %>% 
  ggplot(aes(elev_f, value, colour = variable))+
  geom_point()+
  geom_line(aes(group = variable))+
  theme_bw()


dat_eval2 %>% 
  # melt(id.vars = c("elev_f", "nn")) %>% 
  # ggplot(aes(elev_f, scd_ds_obs - scd_obs))+
  ggplot(aes(elev_f, scd_ds_mod - scd_ds_obs))+
  # ggplot(aes(elev_f, (scd_ds_mod - scd_ds_obs)/scd_ds_obs))+
  geom_hline(yintercept = 0, linetype = "dashed")+
  geom_point()+
  geom_line(aes(group = 1))+
  theme_bw()



# seasonal? ---------------------------------------------------------------

rr_obs_djf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_scd/clim-seasonal/JJA.tif")
rr_ds_obs_djf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/alps-clim-season/JJA.tif")
rr_ds_mod_djf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim_seasonal/JJA/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.tif")



rs_diff4 <- st_as_stars(rr_ds_mod_djf - rr_ds_obs_djf)
rs_diff4_rel <-  st_as_stars((rr_ds_mod_djf - rr_ds_obs_djf)/rr_ds_obs_djf)


ggplot()+
  geom_stars(data = rs_diff4, downsample = 5)+
  scale_fill_gradient2()+
  theme_bw()

ggplot()+
  geom_stars(data = rs_diff4_rel, downsample = 5)+
  scale_fill_gradient2(limits = c(NA, 2))+
  theme_bw()




## elev ---------------------------------------------------------------------



rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA


elev_int <- 100 


rr_obs_seas <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_scd/clim-seasonal/JJA.tif")
rr_ds_obs_seas <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/alps-clim-season/JJA.tif")
rr_ds_mod_seas <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim_seasonal/JJA/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.tif")



dat_eval_seas <- data.table(icell = 1:ncell(rr_elev),
                            scd_obs = rr_obs_seas[],
                            scd_ds_obs = rr_ds_obs_seas[],
                            scd_ds_mod = rr_ds_mod_seas[],
                            elev = rr_elev[])

dat_eval_seas[, elev_f := cut(elev, breaks = c(seq(-200, 4800, by = elev_int)), dig.lab = 5)]

dat_eval_seas2 <- dat_eval_seas[!is.na(scd_ds_obs) &  !is.na(scd_ds_mod) & !is.na(scd_obs), 
                                .(scd_obs = mean(scd_obs),
                                  scd_ds_obs = mean(scd_ds_obs),
                                  scd_ds_mod = mean(scd_ds_mod),
                                  nn = .N),
                                .(elev_f)]

dat_eval_seas2 %>% 
  melt(id.vars = c("elev_f", "nn")) %>% 
  ggplot(aes(elev_f, value, colour = variable))+
  geom_point()+
  geom_line(aes(group = variable))+
  theme_bw()


dat_eval_seas2 %>% 
  # melt(id.vars = c("elev_f", "nn")) %>% 
  # ggplot(aes(elev_f, scd_ds_obs - scd_obs))+
  ggplot(aes(elev_f, (scd_ds_obs - scd_obs)/scd_obs))+
  # ggplot(aes(elev_f, scd_ds_mod - scd_ds_obs))+
  # ggplot(aes(elev_f, (scd_ds_mod - scd_ds_obs)/scd_ds_obs))+
  geom_hline(yintercept = 0, linetype = "dashed")+
  geom_point()+
  geom_line(aes(group = 1))+
  theme_bw()



## all season --------------------------------------------------------------

dat_eval_seas_full <- foreach(
  i_season = c("SON", "DJF", "MAM", "JJA"),
  .final = rbindlist
) %do% {
  
  
  
  rr_obs_seas <- raster(paste0("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_scd/clim-seasonal/", i_season, ".tif"))
  rr_ds_obs_seas <- raster(paste0("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/alps-clim-season/", i_season, ".tif"))
  rr_ds_mod_seas <- raster(paste0("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim_seasonal/", i_season, "/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.tif"))
  
  
  
  dat_eval_seas <- data.table(icell = 1:ncell(rr_elev),
                              scd_obs = rr_obs_seas[],
                              scd_ds_obs = rr_ds_obs_seas[],
                              scd_ds_mod = rr_ds_mod_seas[],
                              elev = rr_elev[])
  
  dat_eval_seas[, elev_f := cut(elev, breaks = c(seq(-200, 4800, by = elev_int)), dig.lab = 5)]
  
  dat_eval_seas[!is.na(scd_ds_obs) &  !is.na(scd_ds_mod) & !is.na(scd_obs), 
                .(season = i_season,
                  scd_obs = mean(scd_obs),
                  scd_ds_obs = mean(scd_ds_obs),
                  scd_ds_mod = mean(scd_ds_mod),
                  nn = .N),
                .(elev_f)]
  

  
}

dat_eval_seas_full[, season_fct := forcats::fct_inorder(season)]


dat_eval_seas_full %>% 
  melt(id.vars = c("elev_f", "nn", "season", "season_fct")) %>% 
  ggplot(aes(elev_f, value, colour = variable))+
  geom_point()+
  geom_line(aes(group = variable))+
  theme_bw()+
  facet_wrap(~season_fct)


dat_eval_seas_full %>% 
  # melt(id.vars = c("elev_f", "nn")) %>% 
  # ggplot(aes(elev_f, scd_ds_obs - scd_obs))+
  # ggplot(aes(elev_f, (scd_ds_obs - scd_obs)/scd_obs))+ylim(NA,1)+
  # ggplot(aes(elev_f, scd_ds_mod - scd_ds_obs))+
  ggplot(aes(elev_f, (scd_ds_mod - scd_ds_obs)/scd_ds_obs))+
  geom_hline(yintercept = 0, linetype = "dashed")+
  geom_point()+
  geom_line(aes(group = 1))+
  theme_bw()+
  facet_wrap(~season_fct)

