# comp DS with BC

# library(stars)
library(raster)
library(data.table)
library(magrittr)
library(ggplot2)
library(cowplot)
library(foreach)
library(fs)


elev_int <- 100 
# elev_int <- 200
# -> no large difference


# alpine wide: DS ensemble ------------------------------------------------



# common icell 

dat_icell <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/ds/common-icell.rds")
names(dat_icell)
rcms_common <- c("MODIS", "CLMcom-CCLM4-8-17", "CLMcom-ETH-COSMO-crCLIM-v1-1",
                 "CNRM-ALADIN63", "IPSL-WRF381P", "SMHI-RCA4")
lgl_icell_common <- dat_icell[, 
                              apply(.SD, 1, all), 
                              .SDcols = rcms_common]


# glacier 


rr_rgi <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/rgi/modis-hr.tif")
lgl_icell_noglacier <- rr_rgi[] <= 0.1

# modis data 

rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA

rr_modis <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")


dat_modis <- data.table(icell = 1:ncell(rr_modis),
                        scd = rr_modis[],
                        elev = rr_elev[])
# dat_modis <- dat_modis[!is.na(scd)]
# dat_modis$elev %>% qplot
# dat_modis[lgl_icell_common & elev > 3000, elev] %>% qplot
# dat_modis[lgl_icell_common & lgl_icell_noglacier & elev > 3000, elev] %>% qplot
# dat_modis[lgl_icell_common & lgl_icell_noglacier, elev] %>% summary

dat_modis[, elev_f := cut(elev, breaks = c(seq(-200, 3600, by = elev_int), 4200), dig.lab = 5)]

dat_modis2 <- dat_modis[lgl_icell_common & lgl_icell_noglacier, 
                        .(scd = mean(scd),
                          nn = .N),
                        .(elev_f)]

# dat_modis2 %>% ggplot(aes(scd, elev_f))+geom_point()


# qdm data 

x <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/03_qdm/"
dat_loop <- data.table(bc_var = path_file(x),
                       file_rcm = dir_ls(x))

dat_loop[, c("institute_rcm", "gcm", "experiment", "ensemble", "dsr", "fp") := 
           tstrsplit(path_ext_remove(path_file(file_rcm)), "_")]

# dat_loop2 <- dat_loop[institute_rcm %in% rcms_common & fp == "future"]
dat_loop2 <- dat_loop[institute_rcm %in% rcms_common]


dat_rcm2 <- foreach(
  i = seq_len(nrow(dat_loop2)),
  .final = rbindlist
) %do% {
  
  # i_rcm <- dat_loop_rcm[i, institute_rcm]
  
  i_rr <- raster(dat_loop2[i, file_rcm])
  
  dat_i <- data.table(dat_modis[, .(icell, elev, elev_f)],
                      scd = i_rr[])
  
  dat_i2 <- dat_i[lgl_icell_common & lgl_icell_noglacier,  # 
                  .(scd = mean(scd),
                    nn = .N),
                  .(elev_f)]
  
  dat_i3 <- dat_i2 %>% 
    cbind(dat_loop2[i, ])
  dat_i3[, file_rcm := NULL]
  dat_i3
  
  
}




# alpine wide: BC ensemble ------------------------------------------------

load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/clim-02.rda")
dat_aux <- readRDS("data/auxiliary-03-modis.rds")


# only take pixels in common to all RCMs
dat_th <- readRDS("data/auxiliary-02-snow-max-threshold.rds")
dat_th[snow_plausible == "yes" & 
         !rcm_short %in% c("WRF331F", "HIRHAM5", "RACMO22E")] %>% 
  dcast(icell ~ rcm_short, value.var = "snow_plausible") %>% 
  na.omit %>% 
  .[, icell] -> icell_common_rcm
icell_modis <- sort(unique(clim_modis$icell))
icell_common <- intersect(icell_common_rcm, icell_modis)


# elev means
dat_aux[, alt_f := cut(alt, breaks = c(-10, seq(200, 2600, by = elev_int), 3000), dig.lab = 5)]

clim_qdm2 <- merge(clim_qdm[icell %in% icell_common], 
                   dat_aux, by = "icell")

dat_bc_elev_alps <- clim_qdm2[,
                              .(snc = mean(snc_qdm)),
                              .(alt_f, rcm, gcm, experiment, fp)]

# days in month (-> no sensible difference)
# clim_qdm2[, days_month := lubridate:::N_DAYS_IN_MONTHS[month]]
# 
# dat_bc_elev_alps <- clim_qdm2[, 
#                               .(snc = weighted.mean(snc_qdm, days_month)), 
#                               .(alt_f, rcm, gcm, experiment, fp)]


# combine all -------------------------------------------------------------

dat_plot_rcm <- rbind(

  dat_rcm2[, .(gcm, institute_rcm, experiment, fp, 
               elev = tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans, 
               scf = scd/365.25,
               region = "GAR", ff = "Downscaling")]
  ,
  dat_bc_elev_alps[, .(gcm, institute_rcm = rcm, experiment, fp, 
                       elev = tstrsplit(alt_f, ",") %>% sapply(readr::parse_number) %>% rowMeans, 
                       scf = snc,
                       region = "GAR", ff = "Bias correction")]
  
)







dat_plot_obs <- dat_modis2[, .(elev = tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans,
                               scf = scd/365.25,
                               region = "GAR")]





# subset to rcms with (almost) full rofental coverage --------------------------------


rcms_to_keep <- c(
  "CLMcom-CCLM4-8-17", 
  "CLMcom-ETH-COSMO-crCLIM-v1-1",
  "CNRM-ALADIN63", # 7 lr pixels missing
  "IPSL-WRF381P",
  "SMHI-RCA4" # 2 lr pixels missing
)


dat_plot_rcm2 <- dat_plot_rcm[institute_rcm %in% rcms_to_keep]


dat_plot_rcm2[, .(ff, institute_rcm, gcm, experiment)] %>% 
  unique %>% 
  with(table(ff, experiment))





# model minmeanmax --------------------------------------------------------


dat_plot_mmm <- dat_plot_rcm2[,
                              .(mod_mean = mean(scf),
                                mod_min = min(scf),
                                mod_max = max(scf)),
                              .(ff, fp, experiment, region, elev)]

setorder(dat_plot_mmm, ff, fp, experiment, region, elev)

# dat_plot_mmm[, ff_region := paste0(ff, " ", region)]

dat_plot_mmm[, period := ifelse(fp == "past", "2000-2020", "2071-2100")]
dat_plot_mmm[, experiment_f := ifelse(experiment == "rcp26", "RCP2.6", "RCP8.5")]

dat_plot_mmm[, period_experiment := paste0(period, " (", experiment_f, ")")]






gg <- dat_plot_mmm[elev < 3700 & !(experiment == "rcp85" & fp == "past")] %>%
  ggplot(aes(mod_mean, elev,
             xmin = mod_min, xmax = mod_max,
             colour = ff, fill = ff))+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_path()+
  geom_point(inherit.aes = F,
             data = dat_plot_obs[elev < 3700],
             aes(scf, elev, shape = "MODIS"))+
  facet_wrap(~ period_experiment)+
  scale_x_continuous(labels = scales::percent)+
  scale_color_brewer("Method",
                     palette = "Dark2", aesthetics = c("color", "fill"))+
  scale_shape("Obs. (2000-2020)")+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("Snow cover fraction")+
  ylab("Elevation [m]")


# 
# ggsave(filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/comp-bc-ds.png",
#        gg,
#        width = 11,
#        height = 3)



# plot diff ---------------------------------------------------------------

# diff to modis

dat_plot_mmm %>% 
  merge(dat_plot_obs[, .(elev, scf_modis = scf)]) -> dat_plot_diff

dat_plot_diff[, ":="(mod_mean_diff = mod_mean - scf_modis,
                     mod_min_diff = mod_min - scf_modis,
                     mod_max_diff = mod_max - scf_modis)]






gg_diff <- dat_plot_diff[elev < 3700 & !(experiment == "rcp85" & fp == "past")] %>% 
  ggplot(aes(mod_mean_diff*100, elev, 
             xmin = mod_min_diff*100, xmax = mod_max_diff*100,
             colour = ff, fill = ff))+
  geom_vline(xintercept = 0)+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_path()+
  facet_wrap(~ period_experiment)+
  # scale_x_continuous(labels = scales::percent)+
  scale_color_brewer("Method", 
                     palette = "Dark2", aesthetics = c("color", "fill"))+
  scale_shape("Obs. (2000-2020)")+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("Difference in snow cover fraction to MODIS [percentage points]")+
  ylab("Elevation [m]")

# gg_diff_comb <- plot_grid(gg, gg_diff,
#                           ncol = 1,
#                           align = "hv",
#                           labels = c("(a)", "(b)"))
# 
# 
# ggsave(filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/comp-bc-ds-diff.png",
#        gg_diff_comb,
#        width = 11,
#        height = 6)




# plot diff rel ---------------------------------------------------------------

# diff to modis

dat_plot_mmm %>% 
  merge(dat_plot_obs[, .(elev, scf_modis = scf)]) -> dat_plot_diff2

dat_plot_diff2[, ":="(mod_mean_diff = (mod_mean - scf_modis) / scf_modis,
                     mod_min_diff = (mod_min - scf_modis) / scf_modis,
                     mod_max_diff = (mod_max - scf_modis) / scf_modis)]






gg_diff2 <-
dat_plot_diff2[elev < 3700 & !(experiment == "rcp85" & fp == "past") &
                 scf_modis*365 > 5] %>% 
  ggplot(aes(mod_mean_diff, elev, 
             xmin = mod_min_diff, xmax = mod_max_diff,
             colour = ff, fill = ff))+
  geom_vline(xintercept = 0)+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_path()+
  facet_wrap(~ period_experiment)+
  scale_x_continuous(labels = scales::percent)+
  ylim(range(dat_plot_diff[elev < 3700 & !(experiment == "rcp85" & fp == "past"), elev]))+
  scale_color_brewer("Method", 
                     palette = "Dark2", aesthetics = c("color", "fill"))+
  scale_shape("Obs. (2000-2020)")+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("Relative difference in snow cover fraction to MODIS")+
  ylab("Elevation [m]")

gg_diff_comb_rel <- plot_grid(gg, gg_diff, gg_diff2,
                              ncol = 1,
                              align = "hv",
                              labels = c("(a)", "(b)"))


ggsave(filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/comp-bc-ds-diff-rel.png",
       gg_diff_comb_rel,
       width = 11,
       height = 9)


# numbers -----------------------------------------------------------------


dat_plot_mmm %>% 
  dcast(fp + experiment + elev ~ ff, value.var = "mod_mean") %>% 
  .[, diff := Downscaling - `Bias correction`] %>% 
  split(by = c("fp", "experiment")) %>% 
  lapply(summary)



dat_plot_mmm %>% 
  dcast(fp + experiment + elev ~ ff, value.var = "mod_mean") %>% 
  .[, 
    .(mean(Downscaling - `Bias correction`, na.rm = T)),
    .(fp, experiment)]


# EOF ---------------------------------------------------------------------






