# compare scf_th seasonal

library(raster)
# library(stars)
# library(sf)
library(data.table)
library(ggplot2)
library(magrittr)
library(scico)
library(cowplot)


# data --------------------------------------------------------------------


rr_annual <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v02_monotonic.tif")
rr_mamjja <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp_seasonal/scf_cp50_v02_monotonic_MAMJJA.tif")
rr_sondjf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp_seasonal/scf_cp50_v02_monotonic_SONDJF.tif")


rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA

dat <- data.table(icell = 1:ncell(rr_annual),
                  value_annual = rr_annual[],
                  value_mamjja = rr_mamjja[],
                  value_sondjf = rr_sondjf[],
                  elev = rr_elev[])

dat[, d_mamjja_sondjf := value_mamjja - value_sondjf]

dat[, elev_fct := cut(elev, breaks = seq(-150, 4850, by = 100), dig.lab = 5)]
dat[, elev_fct2 := cut(elev, breaks = seq(-150, 4850, by = 1000), dig.lab = 5)]

dat_summ <- dat[!is.na(value_annual),
                .(d_mamjja_sondjf = mean(value_mamjja - value_sondjf, na.rm = T),
                  d_mamjja_annual = mean(value_mamjja - value_annual, na.rm = T),
                  d_sondjf_annual = mean(value_sondjf - value_annual, na.rm = T),
                  nn = .N),
                elev_fct]


# plot --------------------------------------------------------------------

dat$d_mamjja_sondjf %>% summary
dat$d_mamjja_sondjf %>% qplot


dat %>% 
  ggplot(aes(d_mamjja_sondjf))+
  geom_histogram()+
  facet_wrap(~elev_fct2)
