# downscale upscaled MODIS

library(raster)
library(data.table)
library(magrittr)
library(fs)
library(foreach)


path_out_main <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/"



# scf and rasters ---------------------------------------------------------

rr_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")

rr_scf_th <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v04_imputed_alt.tif")
mat_scf_th <- raster::as.matrix(rr_scf_th)


# MODIS only -----------------------------------------------------------------

dat_modis <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/modis-10d-02-snc.rds")
dat_modis[, year := year(date)]
dat_modis[, month := month(date)]

dat_nn <- dat_modis[, .(n_days = length(unique(date))), .(year, month)]
dat_loop <- dat_nn[n_days >= 28]


for(i in 1:nrow(dat_loop)){
  
  
  i_year <- dat_loop[i, year]
  i_month <- dat_loop[i, month]
  
  dat_modis_i <- dat_modis[year == i_year & month == i_month]
  
  fn_out_monthly <- path(path_out_main,
                         "alps-monthly/",
                         sprintf("%4i-%02i.tif", i_year, i_month))
  
  
  i_dates <- sort(unique(dat_modis_i$date))
  mat_scd <- matrix(0, ncol = ncol(mat_scf_th), nrow = nrow(mat_scf_th))
  
  for(d in seq_along(i_dates)){
    
    i_date <- i_dates[d]
    dat_modis_i2 <- dat_modis_i[date == i_date]
    
    # upscale
    rr_lr_bc <- raster(rr_lr)
    rr_lr_bc[dat_modis_i2$icell] <- dat_modis_i2$snc
    
    mat_lr_bc <- raster::as.matrix(rr_lr_bc)
    mat_lr_bc_2hr <- mat_lr_bc[rep(1:55, each = 53),
                               rep(1:133, each = 37)]
    mat_lr_bc_2hr <- mat_lr_bc_2hr[1:2867, 1:4901]
    
    # binary snow
    mat_binary_snow <- (mat_lr_bc_2hr > mat_scf_th)
    
    
    
    # save daily
    fn_out_daily <- path(path_out_main,
                         "alps-daily",
                         i_date, ext = "tif")
    rr_out_daily <- raster(rr_scf_th)
    rr_out_daily[] <- mat_binary_snow
    writeRaster(rr_out_daily, fn_out_daily, overwrite = T)
    
    
    # update monthly scd
    mat_scd <- mat_scd + mat_binary_snow
    
    
  }
  
  # save monthly
  
  # mat_scd[is.na(mat_scf_th)] <- NA # not needed
  rr_out <- raster(rr_scf_th)
  rr_out[] <- mat_scd
  
  
  writeRaster(rr_out, fn_out_monthly, overwrite = T)
  
  
  
}




