# viz and numbers downscaled upscaled modis


library(data.table)
library(magrittr)
library(fs)
library(foreach)

library(ggplot2)
library(scico)


dat_summ <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/summary-daily-elev.rds")


# number of nodata/cloud/water in orig data negligible
dat_summ %>% 
  ggplot(aes(N))+
  geom_histogram(aes(fill = factor(value_ds)))+
  facet_wrap(~value_orig, scales = "free")

dat_summ[, .(n_orig_snowland = sum(N[value_orig %in% c(1,2)]),
             n_orig_other = sum(N[!value_orig %in% c(1,2)]),
             n_total = sum(N))] %>% 
  .[, .(n_orig_snowland/n_total, n_orig_other/n_total)]



# data prep ---------------------------------------------------------------

dat_summ2 <- dat_summ[value_orig %in% c(1, 2)]
dat_summ2[, orig_fct := factor(ifelse(value_orig == 1, "snow", "land"))]
dat_summ2[, ds_fct := factor(ifelse(value_ds == 1, "snow", "land"))]
dat_summ2[, c("elev_min", "elev_max") := tstrsplit(elev_fct, "\\(|\\]|,", 
                                                   keep = 2:3, 
                                                   type.convert = T)]
dat_summ2[, elev_middle := (elev_min+elev_max)/2]


dat_tf <- dat_summ2[, 
                    .(n_true = sum(N[orig_fct == ds_fct]),
                      n_false = sum(N[orig_fct != ds_fct]),
                      n_total = sum(N)),
                    .(elev_middle, month(date))]
dat_tf[, frac_true := n_true / n_total]
dat_tf[, frac_false := n_false / n_total]
dat_tf[, month_fct := mitmatmisc::month_fct(month, 10)]




# plot --------------------------------------------------------------------

dat_tf %>% 
  melt(id.vars = c("elev_middle", "month", "month_fct"),
       measure.vars = c("frac_true", "frac_false")) %>% 
  ggplot(aes(elev_middle, value, colour = variable))+
  geom_line()+
  facet_wrap(~month_fct)+
  theme_bw()


dat_tf %>% 
  ggplot(aes(elev_middle, frac_false))+
  geom_line()+
  facet_wrap(~month_fct)+
  theme_bw()

dat_tf %>% 
  ggplot(aes(elev_middle, n_false))+
  geom_line()+
  facet_wrap(~month_fct)+
  theme_bw()

dat_tf %>% 
  ggplot(aes(elev_middle, n_total))+
  geom_line()+
  geom_line(aes(y = n_false, color = "false"))+
  facet_wrap(~month_fct)+
  theme_bw()

dat_tf %>% 
  melt(id.vars = c("elev_middle", "month", "month_fct"),
       measure.vars = c("n_true", "n_false")) %>% 
  ggplot(aes(elev_middle, value, fill = variable))+
  geom_col(width = 100)+
  facet_wrap(~month_fct)+
  theme_bw()





