# viz and numbers downscaled upscaled modis


library(data.table)
library(magrittr)
library(fs)
library(foreach)
library(forcats)

library(ggplot2)
library(scico)


dat_summ <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/02-summary-daily-elev-th.rds")



# data prep ---------------------------------------------------------------

dat_summ2 <- dat_summ[value_orig %in% c(1, 2) & !is.na(th_level)]
dat_summ2[, orig_fct := factor(ifelse(value_orig == 1, "snow", "land"))]
dat_summ2[, ds_fct := factor(ifelse(value_ds == 1, "snow", "land"))]
dat_summ2[, c("elev_min", "elev_max") := tstrsplit(elev_fct, "\\(|\\]|,", 
                                                   keep = 2:3, 
                                                   type.convert = T)]
dat_summ2[, elev_middle := (elev_min+elev_max)/2]



# overall true false

dat_tf <- dat_summ2[, 
                    .(n_true = sum(N[orig_fct == ds_fct]),
                      n_false = sum(N[orig_fct != ds_fct]),
                      n_total = sum(N)),
                    # .(elev_middle, month(date), th_level)]
                    .(elev_middle, month(date))]
dat_tf[, frac_true := n_true / n_total]
dat_tf[, frac_false := n_false / n_total]
dat_tf[, month_fct := mitmatmisc::month_fct(month, 10)]




# detailed
# pos = snow, neg = land, orig = truth
dat_xt <- dat_summ2[, 
                    .(n_tp = sum(N[orig_fct == "snow" & ds_fct == "snow"]),
                      n_tn = sum(N[orig_fct == "land" & ds_fct == "land"]),
                      n_fp = sum(N[orig_fct == "land" & ds_fct == "snow"]),
                      n_fn = sum(N[orig_fct == "snow" & ds_fct == "land"]),
                      n_total = sum(N)),
                    # .(elev_middle, month(date), th_level)]
                    .(elev_middle, month(date))]
dat_xt[, frac_tp := n_tp / n_total]
dat_xt[, frac_tn := n_tn / n_total]
dat_xt[, frac_fp := n_fp / n_total]
dat_xt[, frac_fn := n_fn / n_total]

dat_xt[, month_fct := mitmatmisc::month_fct(month, 10)]




# plot (xt) ---------------------------------------------------------------


dat_xt %>% 
  ggplot(aes(elev_middle))+
  geom_line(aes(y = frac_fp, colour = "fp"))+
  geom_line(aes(y = frac_fn, colour = "fn"))+
  facet_grid(th_level~month_fct)+
  theme_bw()

dat_xt %>% 
  ggplot(aes(elev_middle))+
  geom_line(aes(y = n_fp, colour = "fp"))+
  geom_line(aes(y = n_fn, colour = "fn"))+
  facet_grid(th_level~month_fct)+
  theme_bw()

dat_xt %>% 
  ggplot(aes(elev_middle, n_total))+
  geom_line()+
  geom_line(aes(y = n_fp, colour = "fp"))+
  geom_line(aes(y = n_fn, colour = "fn"))+
  facet_grid(th_level~month_fct)+
  theme_bw()

dat_xt %>% 
  melt(id.vars = c("elev_middle", "month", "month_fct", "th_level"),
       measure.vars = c("n_tp", "n_tn", "n_fp", "n_fn")) %>% 
  ggplot(aes(elev_middle, value, fill = variable))+
  geom_col(width = 100)+
  facet_grid(th_level~month_fct)+
  theme_bw()

dat_xt %>% 
  melt(id.vars = c("elev_middle", "month", "month_fct", "th_level"),
       measure.vars = c("n_tp", "n_tn", "n_fp", "n_fn")) %>% 
  ggplot(aes(elev_middle, value, colour = variable))+
  geom_line()+
  facet_grid(th_level~month_fct)+
  theme_bw()


# summary metrics ---------------------------------------------------------


dat_xt[, .(
  sens = sum(n_tp) / sum(n_tp + n_fn),
  spec = sum(n_tn) / sum(n_tn + n_fp),
  ppv = sum(n_tp) / sum(n_tp + n_fp),
  npv = sum(n_tn) / sum(n_tn + n_fn),
  acc = sum(n_tp + n_tn) / sum(n_tp + n_fp + n_tn + n_fn)
  )]

dat_xt[, .(
  sens = sum(n_tp) / sum(n_tp + n_fn),
  spec = sum(n_tn) / sum(n_tn + n_fp),
  ppv = sum(n_tp) / sum(n_tp + n_fp),
  npv = sum(n_tn) / sum(n_tn + n_fn),
  acc = sum(n_tp + n_tn) / sum(n_tp + n_fp + n_tn + n_fn)
), .(th_level)]

dat_xt[, .(
  sens = sum(n_tp) / sum(n_tp + n_fn),
  spec = sum(n_tn) / sum(n_tn + n_fp),
  ppv = sum(n_tp) / sum(n_tp + n_fp),
  npv = sum(n_tn) / sum(n_tn + n_fn),
  acc = sum(n_tp + n_tn) / sum(n_tp + n_fp + n_tn + n_fn)
), .(elev_middle, month_fct, th_level)] %>% 
  ggplot(aes(elev_middle))+
  geom_line(aes(y = ppv, colour = "ppv"))+
  geom_line(aes(y = npv, colour = "npv"))+
  facet_grid(th_level~month_fct)+
  theme_bw()

dat_xt[, .(
  sens = sum(n_tp) / sum(n_tp + n_fn),
  spec = sum(n_tn) / sum(n_tn + n_fp),
  ppv = sum(n_tp) / sum(n_tp + n_fp),
  npv = sum(n_tn) / sum(n_tn + n_fn),
  acc = sum(n_tp + n_tn) / sum(n_tp + n_fp + n_tn + n_fn)
), .(elev_middle, month_fct, th_level)] %>% 
  ggplot(aes(elev_middle))+
  geom_line(aes(y = acc, colour = "acc"))+
  facet_grid(th_level~month_fct)+
  theme_bw()

