# downscaling method figure
# - 1 lr pixel
# - hr elev
# - hr snc threshold
# - some probability curves with estimated snc threshold
# - maybe imputed values too...

library(raster)
library(stars)
# library(sf)
library(data.table)
library(ggplot2)
library(magrittr)
library(scico)
library(cowplot)


# cp rr -------------------------------------------------------------------

source("R/functions/cp_rr2dt.R")
rr_cp <- brick("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/cp_20bins_monotonic.tif")
band_lbl <- levels(cut(0:100/100, 
                       breaks = c(0, 0.001, 1:19/20, 0.999, 1), 
                       include.lowest = T))

# prep scf data ---------------------------------------------------------------

rr_scf1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v02_monotonic.tif")
rr_scf2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v03_imputed_similar.tif")
rr_scf3 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v04_imputed_alt.tif")


rr_alt <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_alt[rr_alt < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_alt[rr_corine == 0] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
# rr_alt[rr_water == 1] <- NA #problems with aggregate later



dat_scf <- data.table(icell = 1:ncell(rr_scf1),
                      water = rr_water[],
                      scf1 = rr_scf1[],
                      scf2 = rr_scf2[],
                      scf3 = rr_scf3[],
                      alt = rr_alt[])
# add alt_diff
rr_alt_lr <- aggregate(rr_alt,
                       fact = c(37, 53),
                       fun = mean,
                       na.rm = F)

mat_alt_lr <- raster::as.matrix(rr_alt_lr)
mat_alt_lr2hr <- mat_alt_lr[rep(1:55, each = 53),
                            rep(1:133, each = 37)]
mat_alt_lr2hr <- mat_alt_lr2hr[1:2867, 1:4901]
rr_alt_lr2hr <- raster(rr_alt)
rr_alt_lr2hr[] <- mat_alt_lr2hr

dat_scf[, alt_lr := rr_alt_lr2hr[]]
dat_scf[, alt_diff := alt_lr - alt]

# add lr2hr icell
rr_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")
rr_lr[] <- 1:ncell(rr_lr)
mat_icell_lr <- raster::as.matrix(rr_lr)
mat_icell_lr2hr <- mat_icell_lr[rep(1:55, each = 53),
                                rep(1:133, each = 37)]
mat_icell_lr2hr <- mat_icell_lr2hr[1:2867, 1:4901]
rr_icell_lr2hr <- raster(rr_alt)
rr_icell_lr2hr[] <- mat_icell_lr2hr

dat_scf[, icell_lr := rr_icell_lr2hr[]]


# remove water and not available pixels
dat_scf_eval <- dat_scf[!is.na(alt) & water == 0]


# example lr pixel -----------------------------------------------------------


# 2730 lr icell or 2208, 3627

ex_icell <- 2730

rr_alt_lr %>% plot
rr_1cell <- rr_alt_lr
rr_1cell[ex_icell] <- 10000
plot(rr_1cell)

xy_cell <- xyFromCell(rr_1cell, ex_icell)
xy_res <- res(rr_alt_lr)
ext <- extent(c(xmin = xy_cell[1, "x"] - xy_res[1]/2,
                xmax = xy_cell[1, "x"] + xy_res[1]/2, 
                ymin = xy_cell[1, "y"] - xy_res[2]/2,
                ymax = xy_cell[1, "y"] + xy_res[2]/2))

rr_alt_1cell <- crop(rr_alt, ext)
plot(rr_alt_1cell)

rr_scf1_1cell <- crop(rr_scf1, rr_alt_1cell)
rr_scf2_1cell <- crop(rr_scf2, rr_alt_1cell)
rr_scf3_1cell <- crop(rr_scf3, rr_alt_1cell)
plot(stack(rr_alt_1cell, rr_scf1_1cell, rr_scf2_1cell, rr_scf3_1cell))




# example hr pixel within -------------------------------------------------

which(rr_alt_1cell[] > 1450, rr_alt_1cell[] < 1550)
rr_alt_1cell[is.na(rr_scf1_1cell)] %>% qplot

which(rr_alt_1cell[] > 1450 & rr_alt_1cell[] < 1550 & is.na(rr_scf1_1cell[]))
which(rr_alt_1cell[] > 1450 & rr_alt_1cell[] < 1550 & !is.na(rr_scf1_1cell[]))
which(rr_alt_1cell[] > 1800 & !is.na(rr_scf1_1cell[]))
which(rr_alt_1cell[] < 800 & !is.na(rr_scf1_1cell[]))


# ex_icell_hr <- c(280, 395, 1125)
ex_icell_hr <- c(244, 356, 1153)

# add some more (high elev, low elev)
ex_icell_hr <- c(244, 356, 1153, 715, 1582)


rr_check_loc <- rr_scf1_1cell
rr_check_loc[ex_icell_hr] <- 2
plot(rr_check_loc)

rr_scf2_1cell[ex_icell_hr]

# get cp curves
rr_cp_1cell <- crop(rr_cp, rr_alt_1cell)
dt_cp <- cp_rr2dt(ex_icell_hr, rr_cp_1cell, rr_alt_1cell)

# ggplot(dt_cp, aes(scf_middle, value))+
#   geom_point()+
#   facet_wrap(~icell)





# plot --------------------------------------------------------------------



## maps --------------------------------------------------------------------



rs_elev <- st_as_stars(rr_alt_1cell) %>% st_transform(4326)
rs_scf1 <- st_as_stars(rr_scf1_1cell) %>% st_transform(4326)
rs_scf2 <- st_as_stars(rr_scf2_1cell) %>% st_transform(4326)


xyFromCell(rr_alt_1cell, ex_icell_hr) %>% 
  sf_project(st_crs(rr_alt_1cell), st_crs(rs_elev), .) %>% 
  as.data.table() %>% 
  cbind(icell = ex_icell_hr, 
        scf_th = rr_scf2_1cell[ex_icell_hr]) -> dt_ex_icell
setnames(dt_ex_icell, c("V1", "V2"), c("long", "lat"))
setorder(dt_ex_icell, long)
dt_ex_icell[, lbl := paste0("(", 1:5, ")")]
# dt_ex_icell[, th_source := c("original", "original", "imputed", "original", "original")]
dt_ex_icell[, th_source := ifelse(icell == 244, "imputed", "original")]
dt_ex_icell[, th_source_f := factor(th_source, levels = c("original", "imputed"))]


gg_elev <-
ggplot()+
  geom_stars(data = rs_elev)+
  geom_text(data = dt_ex_icell, aes(long, lat, label = lbl))+
  theme_minimal()+
  scale_fill_scico("m.a.s.l.",
                   palette = "oleron", na.value = NA, rescaler = scales::rescale_mid)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"),
                     breaks = c(11.55, 11.6, 11.65))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"),
                     breaks = c(46.65, 46.7, 46.75))

gg_scf1 <-
ggplot()+
  geom_stars(data = rs_scf1)+
  geom_text(data = dt_ex_icell, aes(long, lat, label = lbl))+
  theme_minimal()+
  scale_fill_distiller("SNCp50",
                       limits = c(0,1), palette = "YlOrRd", direction = 1)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"),
                     breaks = c(11.55, 11.6, 11.65))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"),
                     breaks = c(46.65, 46.7, 46.75))

gg_scf2 <- 
ggplot()+
  geom_stars(data = rs_scf2)+
  geom_text(data = dt_ex_icell, aes(long, lat, label = lbl))+
  theme_minimal()+
  scale_fill_distiller("SNCp50",
                       limits = c(0,1), palette = "YlOrRd", direction = 1)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"),
                     breaks = c(11.55, 11.6, 11.65))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"),
                     breaks = c(46.65, 46.7, 46.75))



## cp curves ---------------------------------------------------------------

dt_cp %>% merge(dt_ex_icell) -> dt_cp2

# get impute hr icell used
dat_minmae <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/dist-02-mae-hr.rds")

icell_imp_orig <- cellFromXY(rr_alt, xyFromCell(rr_alt_1cell, 244))
icell_imp_imp <- dat_minmae[icell_orig == icell_imp_orig, icell_min_cp_mae]

dt_cp_imp <- cp_rr2dt(icell_imp_imp, rr_cp, rr_alt)
dt_cp_imp[, lbl := dt_ex_icell[th_source == "imputed", lbl]]


gg_cp <-
  dt_cp2 %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, linetype = "dotted", color = "grey50")+
  geom_point()+
  geom_line()+
  geom_point(data = dt_cp_imp, aes(colour = "similar pixel"),
             shape = 17)+
  geom_line(data = dt_cp_imp, aes(colour = "similar pixel"))+
  geom_vline(data = dt_ex_icell, aes(xintercept = scf_th, linetype = th_source_f))+
  scale_linetype("SNCp50")+
  scale_color_discrete("Prob. from")+
  facet_wrap(~ lbl, nrow = 1)+
  theme_bw()+
  theme(panel.spacing.x = unit(12, "pt"),
        panel.grid.minor = element_blank())+
  xlab("Low-resolution snow cover fraction (SNC)")+
  ylab("Empirical probability \n that high-resolution pixel \n is snow covered")



## combine -----------------------------------------------------------------

gg_top <- plot_grid(gg_elev, gg_scf1, gg_scf2, 
          labels = paste0("(", letters[1:3], ")"), nrow = 1)

gg_out <- plot_grid(gg_top, gg_cp,
                    labels = c("", "(d)"), ncol = 1, rel_heights = c(1.5, 1))


ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/downscale-method.png",
       gg_out,
       width = 12, height = 6)
