# ds bias: map and elev

library(stars)
library(raster)
library(data.table)
library(magrittr)
library(ggplot2)
library(cowplot)
library(foreach)
library(fs)
library(graticule)

library(flextable)
library(officer)


elev_int <- 100 


# ds_stars <- 10
ds_stars <- 5 # production


# data --------------------------------------------------------------------


# maps
rr_ds <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/alps-clim/2000-08-01_2020-07-31.tif")

rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA

rr_modis <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_scd/clim/2000-08-01_2020-07-31.tif")

rr_th <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v05_impute_level.tif")
rr_rgi <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/rgi/modis-hr.tif")

rs_modis <- st_as_stars(rr_modis)
rs_ds <- st_as_stars(rr_ds)
rs_diff_abs <- rs_ds - rs_modis
rs_diff_rel <- (rs_ds - rs_modis)/rs_modis

# more than 100% -> 100%
rs_diff_rel[rs_diff_rel > 1] <- 1


# elev summary

dat_modis <- data.table(icell = 1:ncell(rr_modis),
                        scd_modis = rr_modis[],
                        scd_ds = rr_ds[],
                        th_level = rr_th[],
                        frac_glacier = rr_rgi[],
                        elev = rr_elev[])

dat_modis[, elev_f := cut(elev, breaks = c(seq(-200, 4800, by = elev_int)), dig.lab = 5)]

dat_modis2 <- dat_modis[!is.na(scd_ds) & !is.na(scd_modis), 
                        .(scd_modis = mean(scd_modis),
                          scd_ds = mean(scd_ds),
                          nn = .N),
                        .(elev_f)]


# plot maps --------------------------------------------------------------------



prj <- st_crs(rs_modis)$proj4string

lons <- seq(10, 12, by = 2)
lats <- seq(46, 48, by = 1)
xl <- range(lons) + c(-0.4, 0.4)
yl <- range(lats) + c(-0.4, 0.4)
grat <- graticule(lons, lats, proj = prj, xlim = xl, ylim = yl)
labs <- graticule_labels(lons, lats, xline = min(xl), yline = max(yl), proj = prj)


# -> maps almost not distinguishable



# gg_map_modis <- 
# ggplot()+
#   geom_sf(data = st_as_sf(grat), colour = NA)+
#   geom_stars(data = rs_modis, downsample = ds_stars)+
#   scale_fill_scico("SCD [d]", limits = c(0, 366),
#                    palette = "davos", na.value = NA, direction = 1)+
#   coord_sf(expand = F)+
#   theme_minimal()+
#   xlab(NULL)+ylab(NULL)+
#   scale_x_continuous(labels = function(x) paste0(x, "° E"))+
#   scale_y_continuous(labels = function(x) paste0(x, "° N"))


# gg_map_ds <-
# ggplot()+
#   geom_sf(data = st_as_sf(grat), colour = NA)+
#   geom_stars(data = rs_ds, downsample = ds_stars)+
#   scale_fill_scico("SCD [d]", limits = c(0, 366),
#                    palette = "davos", na.value = NA, direction = 1)+
#   coord_sf(expand = F)+
#   theme_minimal()+
#   xlab(NULL)+ylab(NULL)+
#   scale_x_continuous(labels = function(x) paste0(x, "° E"))+
#   scale_y_continuous(labels = function(x) paste0(x, "° N"))



gg_map_diff_abs <-
  ggplot()+
  geom_sf(data = st_as_sf(grat), colour = NA)+
  geom_stars(data = rs_diff_abs, downsample = ds_stars)+
  scale_fill_scico("dSCD [d]", 
                   palette = "vik", na.value = NA, direction = -1,
                   # limits = c(-50, 50),
                   rescaler = scales::rescale_mid)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"))


gg_map_diff_rel <-
  ggplot()+
  geom_sf(data = st_as_sf(grat), colour = NA)+
  geom_stars(data = rs_diff_rel*100, downsample = ds_stars)+
  scale_fill_scico("dSCD [%]", 
                   palette = "vik", na.value = NA, direction = -1,
                   # limits = c(-1,1)*100,
                   rescaler = scales::rescale_mid)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"))



# 
# 
# gg_map_comb <- plot_grid(gg_map_hr, gg_map_lr,
#                          gg_map_diff_abs, gg_map_diff_rel,
#                          align = "hv",
#                          labels = c("(a)", "(b)", "(c)", "(d)"))
# 
# 
# ggsave(filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/scale-issues-02-map.png",
#        gg_map_comb,
#        width = 12,
#        height = 7)
# 



# plot elev ---------------------------------------------------------------

cols <- scales::brewer_pal(palette = "Set1")(4)[3:4]

dat_modis2[, elev := tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans]

dat_plot <- dat_modis2 %>% 
  melt(id.vars = c("elev", "nn"), measure.vars = c("scd_modis", "scd_ds"))

dat_plot[, variable_f := fct_recode(variable,
                                    "Original" = "scd_modis",
                                    "Downscaled" = "scd_ds")]
setorder(dat_plot, variable, elev)  


dat_plot_diff <- copy(dat_modis2)
dat_plot_diff[, diff_abs := scd_ds - scd_modis]
dat_plot_diff[, diff_rel := (scd_ds - scd_modis)/scd_modis]
setorder(dat_plot_diff, elev)  


gg_scd <- dat_plot %>% 
  ggplot(aes(value, elev, colour = variable_f))+
  geom_path()+
  scale_color_manual("", values = cols)+
  theme_bw()+
  theme(legend.position = c(0.8, 0.2),
        legend.background = element_rect(colour = "black"),
        legend.title = element_blank())+
  xlab("Average SCD [d]")+
  ylab("Elevation [m]")
  
  


gg_diff_abs <-
dat_plot_diff %>% 
  ggplot(aes(diff_abs, elev))+
  geom_vline(xintercept = 0, linetype = "dashed")+
  geom_path()+
  geom_point()+
  theme_bw()+
  xlab("Absolute difference in SCD (downscaled - original) [d]")+
  ylab("Elevation [m]")


gg_diff_rel <-
dat_plot_diff %>% 
  ggplot(aes(diff_rel, elev))+
  geom_vline(xintercept = 0, linetype = "dashed")+
  geom_path()+
  geom_point()+
  scale_x_continuous(labels = scales::percent_format())+
  theme_bw()+
  xlab("Relative difference in SCD (downscaled - original)")+
  ylab("Elevation [m]")




gg_comb <- plot_grid(plot_grid(gg_map_diff_abs, gg_map_diff_rel,
                               nrow = 1,
                               align = "hv",
                               labels = c("(a)", "(b)")),
                     plot_grid(gg_scd, gg_diff_abs, gg_diff_rel,
                               nrow = 1,
                               align = "hv",
                               labels = c("(c)", "(d)", "(e)")),
                     ncol = 1)


ggsave(filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/bias-past/ds-modis-map-elev.png",
       gg_comb,
       width = 14,
       height = 7)




# table and summary -------------------------------------------------------

dat_plot_diff[, .(elev_f, scd_modis, scd_ds, diff_abs, diff_rel = 100*diff_rel)] %>% 
  rbind(data.table(elev_f = "Average", 
                   scd_modis = dat_plot_diff[, weighted.mean(scd_modis, nn)],
                   scd_ds = dat_plot_diff[, weighted.mean(scd_ds, nn)],
                   diff_abs = dat_plot_diff[, weighted.mean(diff_abs, nn)],
                   diff_rel = 100*dat_plot_diff[, weighted.mean(diff_rel, nn)]),
        fill = T) %>% 
  flextable() %>% 
  colformat_double(j = c("scd_modis", "scd_ds"), digits = 1) %>% 
  colformat_double(j = c("diff_abs"), digits = 1) %>% 
  colformat_double(j = c("diff_rel"), digits = 1, suffix = "%") %>% 
  autofit() -> ft

read_docx() %>% 
  body_add_flextable(ft) %>% 
  print(target = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/bias-past/table-ds-modis-map-elev.docx")




# elev plot by glacier and impute level -----------------------------------

# better use original RCM clim plot
# 
# dat_gi <- dat_modis[!is.na(th_level)]
# 
# 
# dat_gi[is.na(frac_glacier), frac_glacier := 0]
# 
# dat_gi$elev %>% summary
# dat_gi[, elev_f := cut(elev, breaks = c(seq(-200, 4800, by = elev_int)), dig.lab = 5)]
# dat_gi[, th_level_f := fct_recode(factor(th_level),
#                                   "Original" = "1",
#                                   "Impute: similar prob. curves" = "2",
#                                   "Impute: elevation" = "3")]
# dat_gi[frac_glacier > 0, qplot(frac_glacier)]
# dat_gi[, frac_glacier_f := ifelse(frac_glacier < 0.1, 
#                                   "<10% glacier in pixel", 
#                                   ">=10% glacier in pixel")]
# 
# dat_gi2 <- dat_gi[!is.na(scd_ds) & !is.na(scd_modis), 
#                   .(scd_modis = mean(scd_modis),
#                     scd_ds = mean(scd_ds),
#                     nn = .N),
#                   .(elev_f, th_level_f, frac_glacier_f)]
# 
# dat_gi2[, elev := tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans]
# 
# dat_gi2[, diff_abs := scd_ds - scd_modis]
# dat_gi2[, diff_rel := (scd_ds - scd_modis)/scd_modis]
# setorder(dat_gi2, elev)  
# 
# dat_gi2[nn < 10]
# 
# # gg_diff_abs <-
# dat_gi2[nn > 5] %>% 
#   ggplot(aes(diff_abs, elev))+
#   geom_vline(xintercept = 0, linetype = "dashed")+
#   geom_path()+
#   geom_point()+
#   theme_bw()+
#   facet_grid(th_level_f ~ frac_glacier_f)+
#   xlab("Absolute difference in SCD (downscaled - original) [d]")+
#   ylab("Elevation [m]")
# 
# 
# # gg_diff_rel <-
# dat_gi2[nn > 10] %>% 
#   ggplot(aes(diff_rel, elev))+
#   geom_vline(xintercept = 0, linetype = "dashed")+
#   geom_path()+
#   geom_point()+
#   scale_x_continuous(labels = scales::percent_format())+
#   theme_bw()+
#   facet_grid(frac_glacier_f ~ th_level_f)+
#   xlab("Relative difference in SCD (downscaled - original)")+
#   ylab("Elevation [m]")
# 
# 
# ggplot(dat_gi, 
#        aes(elev, scd_ds - scd_modis))+
#   geom_bin2d(binwidth = c(100, 5))+
#   scale_fill_viridis_c(trans = "log10")+
#   geom_hline(yintercept = 0)+
#   facet_grid(frac_glacier_f~th_level_f)+
#   theme_bw()+
#   xlab("Elevation [m]")+
#   ylab("SCD difference (RCM - MODIS) [d]")


# EOF ---------------------------------------------------------------------




