

library(raster)
# library(stars)
# library(sf)
library(data.table)
library(ggplot2)
library(magrittr)
library(scico)
library(forcats)

mat_dist <- as.matrix(readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/dist-01-wasserstein.rds"))

icell_sample <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/sample-pixels-01-stratified-elev.rds")

icell_sample



# prep data ---------------------------------------------------------------

rr_alt <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_alt[rr_alt < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_alt[rr_corine == 0] <- NA



rr_scf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v02_monotonic.tif")
rr_scf
# plot(rr_scf)
# plot(is.na(rr_scf))

dat_scf <- data.table(icell = 1:ncell(rr_scf),
                      scf_th = rr_scf[],
                      alt = rr_alt[])
dat_scf[, alt_f := cut(alt, c(-150, 0, 1:47*100, 4800), dig.lab = 5)]




# add alt_diff
rr_alt_lr <- aggregate(rr_alt, 
                       fact = c(37, 53), 
                       fun = mean,
                       na.rm = F)

mat_alt_lr <- as.matrix(rr_alt_lr)
mat_alt_lr2hr <- mat_alt_lr[rep(1:55, each = 53),
                            rep(1:133, each = 37)]
mat_alt_lr2hr <- mat_alt_lr2hr[1:2867, 1:4901]
rr_alt_lr2hr <- raster(rr_alt)
rr_alt_lr2hr[] <- mat_alt_lr2hr

dat_scf[, alt_lr := rr_alt_lr2hr[]]
dat_scf[, alt_diff := alt_lr - alt]

# add lr2hr icell
rr_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")
rr_lr[] <- 1:ncell(rr_lr)
mat_icell_lr <- as.matrix(rr_lr)
mat_icell_lr2hr <- mat_icell_lr[rep(1:55, each = 53),
                                rep(1:133, each = 37)]
mat_icell_lr2hr <- mat_icell_lr2hr[1:2867, 1:4901]
rr_icell_lr2hr <- raster(rr_alt)
rr_icell_lr2hr[] <- mat_icell_lr2hr

dat_scf[, icell_lr := rr_icell_lr2hr[]]


dat_scf_sample <- dat_scf[icell_lr %in% icell_sample]
dat_scf_sample[, icell_lr_fct := fct_reorder(factor(icell_lr), alt_lr)]
# dat_scf_sample[, current := F]



# select some -------------------------------------------------------------

i_select <- 6009
i_compare <- c(i_select, 3660, 1300, 2589, 5372)



wasser_dist <- mat_dist[i_compare, i_select]
icell_min_dist <- which.min(wasser_dist[wasser_dist > 0]) %>% names %>% as.numeric
dat_dist <- data.table(wasser_dist = wasser_dist,
                       icell_lr = i_compare)

dat_plot <- merge(dat_scf_sample,
                  dat_dist)
dat_plot[, current := ifelse(icell_lr == i_select, "reference", "compared to")]
dat_plot[, wasser_dist := round(wasser_dist, 1)]
# dat_plot[icell_lr == icell_min_dist, current := "min_dist"]

dat_plot_text <- unique(dat_plot[, .(icell_lr_fct, wasser_dist)])

dat_plot %>% 
  ggplot(aes(alt))+
  geom_histogram(aes(fill = current), binwidth = 50)+
  geom_text(data = dat_plot_text, 
            aes(x = 3000, y = 200, label = paste0("wd  = ", round(wasser_dist, 1))))+
  facet_wrap(~icell_lr_fct, ncol = 1)+
  scale_fill_discrete("")+
  theme_bw()+
  xlab("Elevation [m]")+
  ylab("Count")

ggsave(filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/wasser-dist-example.png",
       height = 7, width = 4)



# plot maps?
# -> maybe not
# 
# # levels(dat_plot$icell_lr_fct)
# i_compare_sorted <- c(5372, 1300, 6009, 3660, 2589) #manual!
# 
# el_limits <- c(500, 3800)
# 
# l_gg <- lapply(i_compare_sorted, function(ex_icell){
#   
#   xy_cell <- xyFromCell(rr_alt_lr, ex_icell)
#   xy_res <- res(rr_alt_lr)
#   ext <- extent(c(xmin = xy_cell[1, "x"] - xy_res[1]/2,
#                   xmax = xy_cell[1, "x"] + xy_res[1]/2, 
#                   ymin = xy_cell[1, "y"] - xy_res[2]/2,
#                   ymax = xy_cell[1, "y"] + xy_res[2]/2))
#   
#   rr_alt_1cell <- crop(rr_alt, ext)
#   
#   rs_alt <- st_as_stars(rr_alt_1cell) %>% st_transform(4326)
#   
#   ggplot()+
#     geom_stars(data = rs_alt)+
#     theme_minimal()+
#     scale_fill_scico("m.a.s.l.",
#                      palette = "oleron", na.value = NA, rescaler = scales::rescale_mid,
#                      limits = el_limits)+
#     coord_sf(expand = F)+
#     xlab(NULL)+ylab(NULL)+
#     ggtitle(ex_icell)+
#     theme(plot.title = element_text(hjust = 0.5))
# 
# })
# 
# 
# cowplot::plot_grid(plotlist = l_gg)
