# plots for methods overview


library(stars)
library(ggplot2)
library(magrittr)
library(scico)

# ds_stars <- 10
ds_stars <- 5

# cols_modis_binary_grey <- setNames(c("#deebf7", "#a1d99b", "#cccccc", "#cccccc", "#cccccc"),
#                               c("1", "2", "3", "4", "5"))

cols_modis_binary_white <- setNames(c("white", "#deebf7", "#a1d99b", "white", "white", "white"),
                              c("0", "1", "2", "3", "4", "5"))
# modis hr  -------------------------------------------------------

# 20100218_120000.tif (much snow)
# 20050130_120000.tif (much snow)
# 20130225_120000.tif (much snow)
# 20190228_120000.tif (center alps)

plot(raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/05_temporal_complete_max10d/20130603_120000.tif"))

# other
# rr1 <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/05_temporal_complete_max10d/20130603_120000.tif")

# main
rr1 <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/05_temporal_complete_max10d/20190228_120000.tif")


gg1 <- ggplot()+
  geom_stars(data = rr1, downsample = ds_stars)+
  coord_sf()+
  scale_fill_manual("", values = cols_modis_binary_white)+
  theme_void()+
  theme(legend.position = "none")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/modis-binary-other.png",
       gg1, 
       width = 5, height = 3)


# modis lr snc ------------------------------------------------------------


dat_modis <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/modis-10d-02-snc.rds")
dat_aux <- readRDS("data/auxiliary-03-modis.rds")

dat_plot <- dat_modis[date == "2019-02-28"] %>% 
  merge(dat_aux)

gg2 <- dat_plot %>% 
  ggplot(aes(x, y, fill = snc))+
  geom_raster()+
  scale_fill_viridis_c("SNC", option = "E", limits = c(0,1))+
  theme_void()+
  theme(legend.position = "none")


ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/modis-snc.png",
       gg2, 
       width = 5, height = 3)


# modis upscaled ----------------------------------------------------------



rr2 <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/_eval_upscaled_modis/alps-daily/2019-02-28.tif")

gg3 <-
  ggplot()+
  geom_stars(data =  rr2 < 0.5, downsample = ds_stars)+
  coord_sf()+
  scale_fill_manual("", values = unname(cols_modis_binary_white[2:3]))+
  theme_void()+
  theme(legend.position = "none")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/modis-downscaled.png",
       gg3, 
       width = 5, height = 3)




# cp maps -----------------------------------------------------------------


rr_cp <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/cp_20bins.tif")

cut(0:100/100, breaks = c(0, 0.001, 1:19/20, 0.999, 1), include.lowest = T) %>% 
  levels -> band_lbl
rr_cp <- rr_cp %>% 
  st_set_dimensions("band", values = band_lbl)


gg_cp <- ggplot()+
  geom_stars(data = rr_cp[,,,floor(seq(1, 22, by = 1))], 
             downsample = c(ds_stars, ds_stars, 1))+
  # coord_sf()+
  scale_fill_viridis_c("P(hr snow | lr SCF)", limits = c(0,1), na.value = "white")+
  facet_wrap(~band)+
  theme_void()+
  theme(legend.position = "none")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/cp.png",
       gg_cp, 
       width = 5, height = 3)



# scf th map --------------------------------------------------------------


rr_th <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v04_imputed_alt.tif")

gg_th <-
  ggplot()+
  geom_stars(data =  rr_th, downsample = ds_stars)+
  coord_sf()+
  # scale_fill_scico(na.value = "white")+
  scale_fill_distiller("SNCp50",
                       limits = c(0,1), palette = "YlOrRd", direction = 1, na.value = "white")+
  theme_void()+
  theme(legend.position = "none")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/scf-th.png",
       gg_th, 
       width = 5, height = 3)



