# plots methods sub: from cp to scf th

library(stars)
library(ggplot2)
library(magrittr)
library(scico)




# cp scf data -----------------------------------------------------------------
source("R/functions/cp50_scf.R")

source("R/functions/cp_rr2dt.R")
rr_cp1 <- brick("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/cp_20bins.tif")
rr_cp2_mon <- brick("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/cp_20bins_monotonic.tif")
band_lbl <- levels(cut(0:100/100, 
                       breaks = c(0, 0.001, 1:19/20, 0.999, 1), 
                       include.lowest = T))



rr_scf1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v02_monotonic.tif")
rr_scf2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v03_imputed_similar.tif")
rr_scf3 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v04_imputed_alt.tif")


rr_alt <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_alt[rr_alt < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_alt[rr_corine == 0] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
# rr_alt[rr_water == 1] <- NA #problems with aggregate later



dat_scf <- data.table(icell = 1:ncell(rr_scf1),
                      water = rr_water[],
                      scf1 = rr_scf1[],
                      scf2 = rr_scf2[],
                      scf3 = rr_scf3[],
                      alt = rr_alt[])




# add alt_diff
rr_alt_lr <- aggregate(rr_alt,
                       fact = c(37, 53),
                       fun = mean,
                       na.rm = F)

mat_alt_lr <- raster::as.matrix(rr_alt_lr)
mat_alt_lr2hr <- mat_alt_lr[rep(1:55, each = 53),
                            rep(1:133, each = 37)]
mat_alt_lr2hr <- mat_alt_lr2hr[1:2867, 1:4901]
rr_alt_lr2hr <- raster(rr_alt)
rr_alt_lr2hr[] <- mat_alt_lr2hr

dat_scf[, alt_lr := rr_alt_lr2hr[]]
dat_scf[, alt_diff := alt_lr - alt]

# add lr2hr icell
rr_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")
rr_lr[] <- 1:ncell(rr_lr)
mat_icell_lr <- raster::as.matrix(rr_lr)
mat_icell_lr2hr <- mat_icell_lr[rep(1:55, each = 53),
                                rep(1:133, each = 37)]
mat_icell_lr2hr <- mat_icell_lr2hr[1:2867, 1:4901]
rr_icell_lr2hr <- raster(rr_alt)
rr_icell_lr2hr[] <- mat_icell_lr2hr

dat_scf[, icell_lr := rr_icell_lr2hr[]]


# remove water and not available pixels
dat_scf_eval <- dat_scf[!is.na(alt) & !is.na(alt_lr) & water == 0]





# cp curve plots -------------------------------------------------------------------



## 1 mono ------------------------------------------------------------------



# set.seed(1234)
dat1 <- dat_scf_eval[alt > 1000 & alt < 2000 & !is.na(scf1)]
# icell1 <- dat1[scf1 > 0.3 | scf1 < 0.7, sample(icell, 1)]

# 6367312
# 3895071 (up and down)
# 6107608 (long flat)
icell1 <- 3895071

dat1_cp1 <- cp_rr2dt(icell1, rr_cp1, rr_alt)
dat1_cp2 <- cp_rr2dt(icell1, rr_cp2_mon, rr_alt)

# dat1_cp1$value %>% plot(type = "o")
# dat1_cp2$value %>% plot

# icell1_scf_cp1 <- dat1_cp1[, cp50_scf(value)]
icell1_scf_cp1 <- 0.392712121 # manually
icell1_scf_cp2 <- dat1[icell == icell1, scf1]


dat1_cp1 %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, linetype = "dashed", color = "grey20")+
  geom_point()+
  geom_line()+
  # geom_vline(xintercept = icell1_scf_cp1, colour = "#fd8d3c")+
  xlim(0,1)+ylim(0,1)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("lo-res SNC")+
  ylab("P(hi-res snow)")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/ts-cp-1-1.png",
       width = 5, height = 3)


dat1_cp2 %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, linetype = "dashed", color = "grey20")+
  geom_point()+
  geom_line()+
  # geom_vline(xintercept = icell1_scf_cp2, colour = "#fd8d3c")+
  xlim(0,1)+ylim(0,1)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("lo-res SNC")+
  ylab("P(hi-res snow)")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/ts-cp-1-2.png",
       width = 5, height = 3)

dat1_cp2 %>% 
  .[!(value > 0.001 & value < 0.999), value := NA] %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, linetype = "dashed", color = "grey20")+
  geom_point()+
  geom_line()+
  # geom_vline(xintercept = icell1_scf_cp2, colour = "#fd8d3c")+
  xlim(0,1)+ylim(0,1)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("lo-res SNC")+
  ylab("P(hi-res snow)")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/ts-cp-1-3.png",
       width = 5, height = 3)



dat1_cp2 %>% 
  .[!(value > 0.001 & value < 0.999), value := NA] %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, linetype = "dashed", color = "grey20")+
  geom_point()+
  geom_line()+
  geom_vline(xintercept = icell1_scf_cp2, colour = "#fd8d3c")+
  xlim(0,1)+ylim(0,1)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("lo-res SNC")+
  ylab("P(hi-res snow)")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/ts-cp-1-4.png",
       width = 5, height = 3)

## 2 sim pixel ------------------------------------------------------------------



# set.seed(1234)
dat2 <- dat_scf_eval[alt > 1000 & alt < 2500 & is.na(scf1) & !is.na(scf2)]
icell2 <- dat2[scf2 > 0.1 | scf2 < 0.9, sample(icell, 1)]

# 5540442, 12037860 (all below 0.5)
# 4667879 (exact 1 removed)
# 11974010, 7422159 (cross 0.5 two times, and no above 0.5 monotonic)
icell2 <- 7422159

dat2_cp1 <- cp_rr2dt(icell2, rr_cp1, rr_alt)
dat2_cp2 <- cp_rr2dt(icell2, rr_cp2_mon, rr_alt)

# dat2_cp1$value %>% plot(type = "o")
# dat2_cp2$value %>% plot

# icell1_scf_cp1 <- dat1_cp1[, cp50_scf(value)]
icell1_scf_cp1 <- c(0.322090909, 0.3375) # manually
icell2_scf_cp2 <- dat2[icell == icell2, scf2]




# get impute hr icell used
dat_minmae <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/dist-02-mae-hr.rds")
dat2_cp2_imp <- cp_rr2dt(dat_minmae[icell_orig == icell2, icell_min_cp_mae], rr_cp2_mon, rr_alt)
dat2_cp2_imp[!(value > 0.001 & value < 0.999), value := NA] 


dat2_cp1 %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, linetype = "dashed", color = "grey20")+
  geom_point()+
  geom_line()+
  # geom_vline(xintercept = icell1_scf_cp1, colour = "#fd8d3c")+
  xlim(0,1)+ylim(0,1)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("lo-res SNC")+
  ylab("P(hi-res snow)")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/ts-cp-2-1.png",
       width = 5, height = 3)

dat2_cp2 %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, linetype = "dashed", color = "grey20")+
  # geom_point(data = dat2_cp2_imp[!is.na(value)], colour = "grey50", shape = 17)+
  # geom_line(data = dat2_cp2_imp[!is.na(value)], colour = "grey50")+
  geom_point()+
  geom_line()+
  # geom_vline(xintercept = icell2_scf_cp2, colour = "#fd8d3c")+
  xlim(0,1)+ylim(0,1)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("lo-res SNC")+
  ylab("P(hi-res snow)")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/ts-cp-2-2.png",
       width = 5, height = 3)


dat2_cp2 %>% 
  .[!(value > 0.001 & value < 0.999), value := NA] %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, linetype = "dashed", color = "grey20")+
  # geom_point(data = dat2_cp2_imp[!is.na(value)], colour = "grey50", shape = 17)+
  # geom_line(data = dat2_cp2_imp[!is.na(value)], colour = "grey50")+
  geom_point()+
  geom_line()+
  # geom_vline(xintercept = icell2_scf_cp2, colour = "#fd8d3c")+
  xlim(0,1)+ylim(0,1)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("lo-res SNC")+
  ylab("P(hi-res snow)")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/ts-cp-2-3.png",
       width = 5, height = 3)


dat2_cp2 %>% 
  .[!(value > 0.001 & value < 0.999), value := NA]  %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, linetype = "dashed", color = "grey20")+
  geom_point(data = dat2_cp2_imp, colour = "grey50", shape = 17)+
  geom_line(data = dat2_cp2_imp, colour = "grey50")+
  geom_point()+
  geom_line()+
  geom_vline(xintercept = icell2_scf_cp2, colour = "#fd8d3c")+
  xlim(0,1)+ylim(0,1)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("lo-res SNC")+
  ylab("P(hi-res snow)")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/ts-cp-2-4.png",
       width = 5, height = 3)


## 3 elev ------------------------------------------------------------------



# set.seed(1234)
dat3 <- dat_scf_eval[alt > 500 & alt < 4000 & is.na(scf1) & is.na(scf2) & !is.na(scf3)]
icell3 <- dat3[scf3 > 0.2 | scf3 < 0.8, sample(icell, 1)]

# 10704413 (one exact 1)
# 11709748 (all very low)
# 6872260 (one 0, many 1, nothing in between)
icell3 <- 6872260

dat3_cp1 <- cp_rr2dt(icell3, rr_cp1, rr_alt)
dat3_cp2 <- cp_rr2dt(icell3, rr_cp2_mon, rr_alt)

dat3_cp1$value %>% plot(type = "o")
dat3_cp2$value %>% plot

# icell1_scf_cp1 <- dat1_cp1[, cp50_scf(value)]
# icell1_scf_cp1 <- 0.392712121 # manually
icell3_scf_cp2 <- dat3[icell == icell3, scf3]


dat3_cp1 %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, linetype = "dashed", color = "grey20")+
  geom_point()+
  geom_line()+
  # geom_vline(xintercept = icell1_scf_cp1, colour = "#fd8d3c")+
  xlim(0,1)+ylim(0,1)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("lo-res SNC")+
  ylab("P(hi-res snow)")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/ts-cp-3-1.png",
       width = 5, height = 3)


dat3_cp2 %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, linetype = "dashed", color = "grey20")+
  geom_point()+
  geom_line()+
  # geom_vline(xintercept = icell3_scf_cp2, colour = "#fd8d3c")+
  xlim(0,1)+ylim(0,1)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("lo-res SNC")+
  ylab("P(hi-res snow)")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/ts-cp-3-2.png",
       width = 5, height = 3)


dat3_cp2 %>% 
  .[!(value > 0.001 & value < 0.999), value := NA] %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, linetype = "dashed", color = "grey20")+
  geom_point()+
  geom_line()+
  # geom_vline(xintercept = icell3_scf_cp2, colour = "#fd8d3c")+
  xlim(0,1)+ylim(0,1)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("lo-res SNC")+
  ylab("P(hi-res snow)")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/ts-cp-3-3.png",
       width = 5, height = 3)


dat3_cp2 %>% 
  .[!(value > 0.001 & value < 0.999), value := NA] %>% 
  ggplot(aes(scf_middle, value))+
  geom_hline(yintercept = 0.5, linetype = "dashed", color = "grey20")+
  geom_point()+
  geom_line()+
  geom_vline(xintercept = icell3_scf_cp2, colour = "#fd8d3c")+
  xlim(0,1)+ylim(0,1)+
  theme_bw()+
  theme(panel.grid.minor = element_blank())+
  xlab("lo-res SNC")+
  ylab("P(hi-res snow)")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/ts-cp-3-4.png",
       width = 5, height = 3)


# temp: manual cp50 -------------------------------------------------------


dat2_cp1[, approx(scf_middle, value, n = 100)]
