# plots methods sub ds proc

library(stars)
library(ggplot2)
library(magrittr)
library(scico)

cols_modis_binary <- c("#deebf7", "#a1d99b")



# scf 1 lr pixel ----------------------------------------------------------


rr_scf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v04_imputed_alt.tif")


rr_alt <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_alt[rr_alt < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_alt[rr_corine == 0] <- NA

rr_alt_lr <- aggregate(rr_alt,
                       fact = c(37, 53),
                       fun = mean,
                       na.rm = F)

ex_icell <- 2730

xy_cell <- xyFromCell(rr_alt_lr, ex_icell)
xy_res <- res(rr_alt_lr)
ext <- extent(c(xmin = xy_cell[1, "x"] - xy_res[1]/2,
                xmax = xy_cell[1, "x"] + xy_res[1]/2, 
                ymin = xy_cell[1, "y"] - xy_res[2]/2,
                ymax = xy_cell[1, "y"] + xy_res[2]/2))

rr_alt_1cell <- crop(rr_alt, ext)

rr_scf_1cell <- crop(rr_scf, rr_alt_1cell)
rs_scf <- st_as_stars(rr_scf_1cell) %>% st_transform(4326)

gg_scf_1cell <-
  ggplot()+
  geom_stars(data = rs_scf)+
  scale_fill_distiller("SNCp50",
                       limits = c(0,1), palette = "YlOrRd", direction = 1)+
  coord_sf(expand = F)+
  theme_void()

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/scf-th-1cell.png",
       gg_scf_1cell, 
       width = 4, height = 5)




# snc 1 cell colors -----------------------------------------------------



scales::show_col(scales::viridis_pal(option = "E")(100))

scales::viridis_pal(option = "E")(100)[c(10,50,90)]

ggplot()+
  geom_stars(data = rs_scf, fill = scales::viridis_pal(option = "E")(100)[25])+
  coord_sf(expand = F)+
  theme_void()+
  theme(legend.position = "none")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/snc-1cell-25.png",
       width = 3, height = 5)

ggplot()+
  geom_stars(data = rs_scf, fill = scales::viridis_pal(option = "E")(100)[50])+
  coord_sf(expand = F)+
  theme_void()+
  theme(legend.position = "none")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/snc-1cell-50.png",
       width = 3, height = 5)

ggplot()+
  geom_stars(data = rs_scf, fill = scales::viridis_pal(option = "E")(100)[75])+
  coord_sf(expand = F)+
  theme_void()+
  theme(legend.position = "none")

ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/snc-1cell-75.png",
       width = 3, height = 5)



# ds 1 cell ---------------------------------------------------------------


ggplot()+
  geom_stars(data = rs_scf > 0.25)+
  scale_fill_manual(values = cols_modis_binary)+
  coord_sf(expand = F)+
  theme_void()+
  theme(legend.position = "none")


ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/snow-binary-1cell-25.png",
       width = 3, height = 5)


ggplot()+
  geom_stars(data = rs_scf > 0.5)+
  scale_fill_manual(values = cols_modis_binary)+
  coord_sf(expand = F)+
  theme_void()+
  theme(legend.position = "none")


ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/snow-binary-1cell-50.png",
       width = 3, height = 5)


ggplot()+
  geom_stars(data = rs_scf > 0.75)+
  scale_fill_manual(values = cols_modis_binary)+
  coord_sf(expand = F)+
  theme_void()+
  theme(legend.position = "none")


ggsave("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/methods-overview/snow-binary-1cell-75.png",
       width = 3, height = 5)
