# check scf th (sncp50) by elev and grid cell



library(data.table)
library(magrittr)
library(raster)


rr_alt <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_alt[rr_alt < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_alt[rr_corine == 0] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
# rr_alt[rr_water == 1] <- NA #problems with aggregate later

# rr_scf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v03_imputed_similar.tif")
rr_scf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v04_imputed_alt.tif")
rr_scf
# plot(rr_scf)
# plot(is.na(rr_scf))

dat_scf <- data.table(icell = 1:ncell(rr_scf),
                      water = rr_water[],
                      scf_th = rr_scf[],
                      alt = rr_alt[])
dat_scf[, alt_f := cut(alt, c(-150, 0, 1:47*100, 4800), dig.lab = 5)]

# add alt_diff
rr_alt_lr <- aggregate(rr_alt,
                       fact = c(37, 53),
                       fun = mean,
                       na.rm = F)

mat_alt_lr <- raster::as.matrix(rr_alt_lr)
mat_alt_lr2hr <- mat_alt_lr[rep(1:55, each = 53),
                            rep(1:133, each = 37)]
mat_alt_lr2hr <- mat_alt_lr2hr[1:2867, 1:4901]
rr_alt_lr2hr <- raster(rr_alt)
rr_alt_lr2hr[] <- mat_alt_lr2hr

dat_scf[, alt_lr := rr_alt_lr2hr[]]
dat_scf[, alt_diff := alt_lr - alt]

# add lr2hr icell
rr_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")
rr_lr[] <- 1:ncell(rr_lr)
mat_icell_lr <- raster::as.matrix(rr_lr)
mat_icell_lr2hr <- mat_icell_lr[rep(1:55, each = 53),
                                rep(1:133, each = 37)]
mat_icell_lr2hr <- mat_icell_lr2hr[1:2867, 1:4901]
rr_icell_lr2hr <- raster(rr_alt)
rr_icell_lr2hr[] <- mat_icell_lr2hr

dat_scf[, icell_lr := rr_icell_lr2hr[]]




# remove water and not available pixels
dat_scf_eval <- dat_scf[!is.na(alt) & !is.na(alt_lr) & water == 0]



# check single lr icell ---------------------------------------------------

# 2730 lr icell or 2208, 3627

ex_icell <- 2730
ex_icell <- 2208
ex_icell <- 3627

rr_alt_lr %>% plot
rr_1cell <- rr_alt_lr
rr_1cell[ex_icell] <- 10000
plot(rr_1cell)


dat1 <- dat_scf_eval[icell_lr == ex_icell]


dat1 %>% 
  ggplot(aes(scf_th, alt))+
  geom_hline(yintercept = dat1$alt_lr[1])+
  geom_point()+
  theme_bw()

dat1 %>% 
  ggplot(aes(scf_th, alt_diff))+
  geom_point()+
  theme_bw()


dat1 %>% 
  ggplot(aes(scf_th, alt))+
  geom_bin2d()+
  scale_fill_viridis_c()+
  theme_bw()



# summary all -------------------------------------------------------------


dat_summ <- dat_scf_eval[!is.na(scf_th),
                         .(mean_scf_th = mean(scf_th)),
                         .(alt_f)]
dat_summ %>% 
  ggplot(aes(mean_scf_th, alt_f))+
  geom_point()

dat_scf_eval %>% 
  ggplot(aes(scf_th, alt))+
  geom_bin2d()+
  scale_fill_viridis_c(trans = "log10")+
  theme_bw()

dat_scf_eval %>% 
  ggplot(aes(alt_diff, alt))+
  geom_bin2d()+
  scale_fill_viridis_c(trans = "log10")+
  theme_bw()

dat_scf_eval %>% 
  ggplot(aes(alt_diff, alt_lr))+
  geom_bin2d()+
  scale_fill_viridis_c(trans = "log10")+
  theme_bw()

# dat_scf_eval %>% 
#   ggplot(aes(alt_lr, alt))+
#   geom_bin2d()+
#   geom_abline()+
#   scale_fill_viridis_c(trans = "log10")+
#   theme_bw()
