# get MODIS data


# library(raster)
library(stars)
library(data.table)
library(lubridate)
library(magrittr)
library(fs)
library(ggplot2)
library(forcats)
library(scico)
library(graticule)
library(patchwork)


# dsample <- 10 # for testing
dsample <- 5 # for final





# data --------------------------------------------------------------------


# rs_clim <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")
rs_clim <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_scd/clim/2000-08-01_2020-07-31.tif")


rs_elev <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rs_elev[rs_elev < -30000] <- NA

rs_water <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rs_elev[rs_water == 1] <- NA


rs_rcm <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_fixed/orog/orog_EUR-11_CNRM-CERFACS-CNRM-CM5_historical_r0i0p0_SMHI-RCA4_v1_fx.nc")
names(rs_rcm) <- "elev"


# rofental data -----------------------------------------------------------

rs_rofental <- read_stars("data/rofental_modis_laea_flo.tif")
sf_rofental <- st_as_sfc(st_bbox(rs_rofental))

rs_clim_rofental <- st_crop(rs_clim, rs_rofental)
st_crs(rs_rofental) <- st_crs(rs_elev)
rs_elev_rofental <- st_crop(rs_elev, rs_rofental)


# graticules --------------------------------------------------------------

prj <- st_crs(rs_clim)$proj4string

lons <- seq(10, 12, by = 2)
lats <- seq(46, 48, by = 1)
xl <- range(lons) + c(-0.4, 0.4)
yl <- range(lats) + c(-0.4, 0.4)
grat <- graticule(lons, lats, proj = prj, xlim = xl, ylim = yl)
labs <- graticule_labels(lons, lats, xline = min(xl), yline = max(yl), proj = prj)


# map: scd with graticule --------------------------------------------------------------------


gg_scd <-
  ggplot()+
  geom_sf(data = st_as_sf(grat))+
  geom_stars(data = rs_clim, downsample = dsample)+
  geom_sf(data = sf_rofental, fill = NA, color = "black")+
  scale_fill_scico("SCD [d]", limits = c(0, 366),
                   palette = "davos", na.value = NA, direction = 1)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"))

gg_scd_rofental <- 
ggplot()+
  geom_sf(data = st_crop(st_as_sf(grat), rs_clim_rofental))+
  geom_stars(data = rs_clim_rofental)+
  scale_fill_scico("SCD [d]", limits = c(0, 366),
                   palette = "davos", na.value = NA, direction = 1)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"),
                     breaks = c(10.5, 11))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"),
                     breaks = c(46.8, 46.9, 47))+
  theme(legend.position = "none",
        plot.margin = unit(c(5.5, 11, 5.5, 5.5), "points"))


# map: elev with graticule ------------------------------------------------

lim_elev <- range(rs_elev[[1]], na.rm = T)
# cols_elev <- terrain.colors(10)[2:9]
# cols_elev <- terrain.colors(8)[2:7]

gg_elev <-
  ggplot()+
  geom_sf(data = st_as_sf(grat))+
  geom_stars(data = rs_elev, downsample = dsample)+
  geom_sf(data = sf_rofental, fill = NA, color = "black")+
  scale_fill_scico("m.a.s.l.", limits = lim_elev,
                   palette = "oleron", na.value = NA, rescaler = scales::rescale_mid)+
  # scale_fill_gradientn("m.a.s.l.", limits = lim_elev,
  #                      colours = cols_elev, na.value = NA)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"))

gg_elev_rofental <-
ggplot()+
  geom_sf(data = st_crop(st_as_sf(grat), rs_clim_rofental))+
  geom_stars(data = rs_elev_rofental)+
  scale_fill_scico("m.a.s.l.", limits = lim_elev,
                   palette = "oleron", na.value = NA, rescaler = scales::rescale_mid)+
  # scale_fill_gradientn("m.a.s.l.", limits = lim_elev,
  #                      colours = cols_elev, na.value = NA)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"),
                     breaks = c(10.5, 11))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"),
                     breaks = c(46.8, 46.9, 47))+
  theme(legend.position = "none",
        plot.margin = unit(c(5.5, 11, 5.5, 5.5), "points"))


# map: elev rcm -----------------------------------------------------------

# 
# gg_elev_rcm <-
#   ggplot()+
#   geom_sf(data = st_as_sf(grat))+
#   geom_stars(data = rs_rcm, downsample = 1)+
#   scale_fill_scico("m.a.s.l.", limits = lim_elev,
#                    palette = "oleron", na.value = NA, rescaler = scales::rescale_mid)+
#   # scale_fill_gradientn("m.a.s.l.", limits = lim_elev,
#   #                      colours = cols_elev, na.value = NA)+
#   
#   coord_sf(expand = F)+
#   theme_minimal()+
#   xlab(NULL)+ylab(NULL)+
#   scale_x_continuous(labels = function(x) paste0(x, "° E"))+
#   scale_y_continuous(labels = function(x) paste0(x, "° N"))


# combine and save --------------------------------------------------------


# gg_out <- cowplot::plot_grid(gg_scd, gg_elev, gg_elev_rcm, ncol = 1,
#                              labels = paste0("(", letters[1:3], ")"))
# 
# 
# ggsave(gg_out,
#        file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/fig/paper-01-initial/map.png",
#        width = 4,
#        height = 6,
#        units = "in")


gg_out <- cowplot::plot_grid(gg_elev, gg_elev_rofental,
                             gg_scd, gg_scd_rofental,
                             ncol = 2, rel_widths = c(2,1),
                             labels = paste0("(", letters[1:4], ")"))


ggsave(gg_out,
       file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/map-with-rofental.png",
       width = 6,
       height = 4,
       units = "in")


ggsave(gg_out,
       file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/04-production/Figure 1.pdf",
       width = 6,
       height = 4,
       units = "in")


ggsave(gg_out,
       file = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/04-production/Figure 1.png",
       width = 6,
       height = 4,
       units = "in")