# compare scf_th seasonal

library(raster)
# library(stars)
# library(sf)
library(data.table)
# library(ggplot2)
library(magrittr)
# library(scico)
# library(cowplot)

library(flextable)
library(officer)


# data --------------------------------------------------------------------


rr_annual <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v02_monotonic.tif")
rr_mamjja <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp_seasonal/scf_cp50_v02_monotonic_MAMJJA.tif")
rr_sondjf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp_seasonal/scf_cp50_v02_monotonic_SONDJF.tif")


rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA

dat <- data.table(icell = 1:ncell(rr_annual),
                  value_annual = rr_annual[],
                  value_mamjja = rr_mamjja[],
                  value_sondjf = rr_sondjf[],
                  elev = rr_elev[])

dat[, d_mamjja_sondjf := value_mamjja - value_sondjf]
dat[, d_mamjja_annual := value_mamjja - value_annual]
dat[, d_sondjf_annual := value_sondjf - value_annual]

dat[, elev_fct := cut(elev, breaks = seq(-150, 4850, by = 100), dig.lab = 5)]
dat[, elev_fct2 := cut(elev, breaks = seq(-150, 4850, by = 1000), dig.lab = 5)]
dat[, elev_fct3 := cut(elev, breaks = seq(-150, 4850, by = 500), dig.lab = 5)]

dat_summ <- dat[!is.na(value_annual),
                .(d_mamjja_sondjf = mean(value_mamjja - value_sondjf, na.rm = T),
                  d_mamjja_annual = mean(value_mamjja - value_annual, na.rm = T),
                  d_sondjf_annual = mean(value_sondjf - value_annual, na.rm = T),
                  nn = .N),
                elev_fct]


dat_num <- dat[!is.na(value_annual) & !is.na(value_sondjf) & !is.na(value_mamjja),
               .(
                 bias_mamjja_sondjf = mean(d_mamjja_sondjf),
                 bias_mamjja_annual = mean(d_mamjja_annual),
                 bias_sondjf_annual = mean(d_sondjf_annual),
                 mae_mamjja_sondjf = mean(abs(d_mamjja_sondjf)),
                 mae_mamjja_annual = mean(abs(d_mamjja_annual)),
                 mae_sondjf_annual = mean(abs(d_sondjf_annual)),
                 nn = .N
                 )]

dat_num




dat_num2 <- dat[!is.na(value_annual) & !is.na(value_sondjf) & !is.na(value_mamjja),
               .(
                 bias_mamjja_sondjf = mean(d_mamjja_sondjf),
                 bias_mamjja_annual = mean(d_mamjja_annual),
                 bias_sondjf_annual = mean(d_sondjf_annual),
                 mae_mamjja_sondjf = mean(abs(d_mamjja_sondjf)),
                 mae_mamjja_annual = mean(abs(d_mamjja_annual)),
                 mae_sondjf_annual = mean(abs(d_sondjf_annual)),
                 nn = .N
               ),
               keyby = elev_fct3]

dat_num2


# table -------------------------------------------------------------------

dat_header <- data.table(col_keys = names(dat_num2))
dat_header[, c(paste0("V", 1:3)) := tstrsplit(col_keys, "_")]
dat_header[, head1 := stringr::str_to_upper(V1)]
dat_header[, head2 := paste0(toupper(V2),"-", toupper(V3))]
dat_header[c(1, 8), head2 := ""]
dat_header[1, head1 := "Elevation [m]"]
dat_header[8, head1 := "Number"]
dat_header_ft <- dat_header[, .(col_keys, head1, head2)]


dat_num %>% 
  flextable %>% 
  set_header_df(dat_header_ft) %>% 
  colformat_double(digits = 3) %>% 
  merge_h(part = "header") %>% 
  theme_booktabs() %>% 
  fix_border_issues() %>% 
  fontsize(size = 7, part = "all") %>% 
  autofit(0, 0) -> ft1



dat_num2 %>% 
  flextable %>% 
  set_header_df(dat_header_ft) %>% 
  colformat_double(digits = 3) %>% 
  merge_h(part = "header") %>% 
  theme_booktabs() %>% 
  fix_border_issues() %>% 
  fontsize(size = 7, part = "all") %>% 
  autofit(0, 0) -> ft2


read_docx() %>% 
  body_add_flextable(ft1) %>% 
  body_add_par(" new table ") %>% 
  body_add_flextable(ft2) %>% 
  print(target = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/table-cp-seasonal.docx")


