# maps bc
# - winter, spring
# - mean, q10, q90
# - by gcm-rcm
# - past, rcp26, rcp85
# - raw, dc, qm, qdm?



library(fs)
library(data.table)
library(foreach)
library(magrittr)
library(ggplot2)
library(forcats)
library(cowplot)


path_out <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/zenodo/aux-paper_ba-maps-single-rcm/"

load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/clim-02.rda")
rcm_short <- readRDS("data/auxiliary-01-rcm-names.rds")
dat_aux <- readRDS("data/auxiliary-03-modis.rds")

# winter, spring avg ---------------------------------------------------------

clim_modis[, season := mitmatmisc::season_fct(month)]
clim_qm[, season := mitmatmisc::season_fct(month)]
clim_qdm[, season := mitmatmisc::season_fct(month)]
dat_delta2[, season := mitmatmisc::season_fct(month)]


dt_modis <- clim_modis[season %in% c("DJF", "MAM"), 
                       .(snc_mean = mean(snc),
                         snc_q10 = mean(snc_q10),
                         snc_q90 = mean(snc_q90)),
                       .(icell, season)]

dt_qdm <- clim_qdm[season %in% c("DJF", "MAM"), 
                   .(snc_mean = mean(snc_qdm),
                     snc_q10 = mean(snc_qdm_q10),
                     snc_q90 = mean(snc_qdm_q90)), 
                   .(icell, season, rcm, gcm, experiment, fp)]
dt_qm <- clim_qm[season %in% c("DJF", "MAM"), 
                 .(snc_mean = mean(snc_qm),
                   snc_q10 = mean(snc_qm_q10),
                   snc_q90 = mean(snc_qm_q90)), 
                 .(icell, season, rcm, gcm, experiment, fp)]
dt_dc <- dat_delta2[season %in% c("DJF", "MAM"),
                    .(snc_mean = mean(snc_future_delta),
                      snc_q10 = mean(snc_q10_future_delta),
                      snc_q90 = mean(snc_q90_future_delta),
                      fp = "future"),
                    .(icell, season, rcm, gcm, experiment)]
dt_raw <- clim_qdm[season %in% c("DJF", "MAM"), 
                   .(snc_mean = mean(snc),
                     snc_q10 = mean(snc_q10),
                     snc_q90 = mean(snc_q90)), 
                   .(icell, season, rcm, gcm, experiment, fp)]

dat_full <- rbindlist(list(
  # modis = dt_modis,
  # mbc = dt_mbc,
  raw = dt_raw,
  qdm = dt_qdm,
  qm = dt_qm,
  dc = dt_dc
), use.names = T, fill = T, idcol = "ff")

dat_full[, rcm_short := rcm_short[rcm]]

# WRF331F not anymore on ESGF
dat_full <- dat_full[is.na(rcm_short) |
                       !rcm_short %in% c("WRF331F")]



# plot --------------------------------------------------------------------

dat_full[, experiment_f := ifelse(experiment == "rcp26", "RCP2.6", "RCP8.5")]
dat_loop <- dat_full[, .(rcm, gcm, experiment, experiment_f, rcm_short)] %>% unique

foreach(
  i = seq_len(nrow(dat_loop))
) %do% {
  
  dat_i <- dat_loop[i, ] %>% 
    merge(dat_full)
  
  tit <- dat_loop[i, paste0(rcm, ", ", gcm, ", ", experiment_f)]
  
  fn_out <- path(path_out, 
                 dat_loop[i, paste0(rcm, "_", gcm, "_", experiment)],
                 ext = "png")
 
 
  dat_i_past <- rbind(dat_i[fp == "past"],
                      cbind(dt_modis, ff = "obs"),
                      fill = T)
  dat_i_past[, lbl_bc := factor(toupper(ff), levels = toupper(c("obs", "raw", "qm", "qdm")))]
  
  dat_i_future <- dat_i[fp == "future"]
  dat_i_future[, lbl_bc := factor(toupper(ff), levels = toupper(c("dc", "raw", "qm", "qdm")))]
  
  
  gg1 <- dat_i_past %>% 
    merge(dat_aux, by = "icell") %>% 
    
    ggplot(aes(x, y, fill = snc_mean))+
    geom_raster()+
    scale_fill_viridis_c("SNC", option = "E", limits = c(0,1))+
    facet_grid(season ~ lbl_bc)+
    scale_x_continuous(expand = c(0,0))+
    scale_y_continuous(expand = c(0,0))+
    coord_fixed()+
    theme_minimal()+
    theme(panel.grid = element_blank(),
          axis.text = element_blank(),
          axis.ticks = element_blank())+
    xlab(NULL)+ylab(NULL)+
    ggtitle(label = tit, "Past (2001-2020)")
  
  gg2 <- dat_i_future %>% 
    merge(dat_aux, by = "icell") %>% 
    
    ggplot(aes(x, y, fill = snc_mean))+
    geom_raster()+
    scale_fill_viridis_c("SNC", option = "E", limits = c(0,1))+
    facet_grid(season ~ lbl_bc)+
    scale_x_continuous(expand = c(0,0))+
    scale_y_continuous(expand = c(0,0))+
    coord_fixed()+
    theme_minimal()+
    theme(panel.grid = element_blank(),
          axis.text = element_blank(),
          axis.ticks = element_blank())+
    xlab(NULL)+ylab(NULL)+
    ggtitle(label = NULL, subtitle = "Future (2071-2100)")

  gg_out <- plot_grid(gg1, gg2, ncol = 1)
  

  
  ggsave(filename = fn_out,
         gg_out, 
         height = 6,
         width = 8)
  return(fn_out)
}



