# comp DS with BC (only change estimates because of scale issues and DS bias)

# library(stars)
library(raster)
library(data.table)
library(magrittr)
library(ggplot2)
library(cowplot)
library(foreach)
library(fs)
library(forcats)


elev_int <- 100 
# elev_int <- 200
# -> no large difference


rcms_common <- c("CLMcom-CCLM4-8-17", "CLMcom-ETH-COSMO-crCLIM-v1-1",
                 "CNRM-ALADIN63", "IPSL-WRF381P", "SMHI-RCA4")



cols_man <- setNames(scales::brewer_pal(palette = "Dark2")(3),
                     c("AMUNDSEN", "Bias adjustment", "Downscaling"))


# ALPS --------------------------------------------------------------------



## alpine wide: DS ensemble ------------------------------------------------



# common icell 

dat_icell <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/ds/common-icell.rds")
names(dat_icell)
rcms_common <- c("MODIS", "CLMcom-CCLM4-8-17", "CLMcom-ETH-COSMO-crCLIM-v1-1",
                 "CNRM-ALADIN63", "IPSL-WRF381P", "SMHI-RCA4")
lgl_icell_common <- dat_icell[, 
                              apply(.SD, 1, all), 
                              .SDcols = rcms_common]


# glacier 


rr_rgi <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/rgi/modis-hr.tif")
lgl_icell_noglacier <- rr_rgi[] <= 0.1

# modis data 

rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA

rr_modis <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_scd/clim/2000-08-01_2020-07-31.tif")


dat_modis <- data.table(icell = 1:ncell(rr_modis),
                        scd = rr_modis[],
                        elev = rr_elev[])
# dat_modis <- dat_modis[!is.na(scd)]
# dat_modis$elev %>% qplot
# dat_modis[lgl_icell_common & elev > 3000, elev] %>% qplot
# dat_modis[lgl_icell_common & lgl_icell_noglacier & elev > 3000, elev] %>% qplot
# dat_modis[lgl_icell_common & lgl_icell_noglacier, elev] %>% summary

dat_modis[, elev_f := cut(elev, breaks = c(seq(-200, 3600, by = elev_int), 4200), dig.lab = 5)]

dat_modis2 <- dat_modis[lgl_icell_common & lgl_icell_noglacier, 
                        .(scd = mean(scd),
                          nn = .N),
                        .(elev_f)]

# dat_modis2 %>% ggplot(aes(scd, elev_f))+geom_point()


# qdm data 

x <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim/03_qdm/"
dat_loop <- data.table(bc_var = path_file(x),
                       file_rcm = dir_ls(x))

dat_loop[, c("institute_rcm", "gcm", "experiment", "ensemble", "dsr", "fp") := 
           tstrsplit(path_ext_remove(path_file(file_rcm)), "_")]

# dat_loop2 <- dat_loop[institute_rcm %in% rcms_common & fp == "future"]
dat_loop2 <- dat_loop[institute_rcm %in% rcms_common]


dat_rcm2 <- foreach(
  i = seq_len(nrow(dat_loop2)),
  .final = rbindlist
) %do% {
  
  # i_rcm <- dat_loop_rcm[i, institute_rcm]
  
  i_rr <- raster(dat_loop2[i, file_rcm])
  
  dat_i <- data.table(dat_modis[, .(icell, elev, elev_f)],
                      scd = i_rr[])
  
  dat_i2 <- dat_i[lgl_icell_common & lgl_icell_noglacier,  # 
                  .(scd = mean(scd),
                    nn = .N),
                  .(elev_f)]
  
  dat_i3 <- dat_i2 %>% 
    cbind(dat_loop2[i, ])
  dat_i3[, file_rcm := NULL]
  dat_i3
  
  
}




## alpine wide: BC ensemble ------------------------------------------------

load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/clim-02.rda")
dat_aux <- readRDS("data/auxiliary-03-modis.rds")


# only take pixels in common to all RCMs
dat_th <- readRDS("data/auxiliary-02-snow-max-threshold.rds")
dat_th[snow_plausible == "yes" & 
         rcm %in% rcms_common] %>% 
  dcast(icell ~ rcm_short, value.var = "snow_plausible") %>% 
  na.omit %>% 
  .[, icell] -> icell_common_rcm
icell_modis <- sort(unique(clim_modis$icell))
icell_common <- intersect(icell_common_rcm, icell_modis)


# elev means
dat_aux[, alt_f := cut(alt, breaks = c(-10, seq(200, 2600, by = elev_int), 3000), dig.lab = 5)]

clim_qdm2 <- merge(clim_qdm[icell %in% icell_common], 
                   dat_aux, by = "icell")

dat_bc_elev_alps <- clim_qdm2[,
                              .(snc = mean(snc_qdm)),
                              .(alt_f, rcm, gcm, experiment, fp)]

# days in month (-> no sensible difference)
# clim_qdm2[, days_month := lubridate:::N_DAYS_IN_MONTHS[month]]
# 
# dat_bc_elev_alps <- clim_qdm2[, 
#                               .(snc = weighted.mean(snc_qdm, days_month)), 
#                               .(alt_f, rcm, gcm, experiment, fp)]


## combine all -------------------------------------------------------------

dat_plot_alps <- rbind(

  dat_rcm2[, .(gcm, institute_rcm, experiment, fp, 
               elev = tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans, 
               scf = scd/365.25,
               region = "GAR", ff = "Downscaling")]
  ,
  dat_bc_elev_alps[, .(gcm, institute_rcm = rcm, experiment, fp, 
                       elev = tstrsplit(alt_f, ",") %>% sapply(readr::parse_number) %>% rowMeans, 
                       scf = snc,
                       region = "GAR", ff = "Bias adjustment")]
  
)



# subset to common rcms


dat_plot_alps2 <- dat_plot_alps[institute_rcm %in% rcms_common]


dat_plot_alps2[, .(ff, institute_rcm, gcm, experiment)] %>% 
  unique %>% 
  with(table(ff, experiment))








# ROFENTAL ------------------------------------------------------------------



## pixels removed in rcm (glacier & rca4) ----------------------------------


# 5% 2200m
# 10% 2600m
# 20% 2700m
# 50% 3000m

elev_th <- 2700



## flo data -------------------------------------------------------------

dir_ls("data-aux/amundsen-flo-hr2/", regexp = "elev") %>% 
  lapply(function(x){
    
    path_file(x) %>% 
      path_ext_remove() %>% 
      stringr::str_split("_", simplify = T) -> x_split
    
    dat <- fread(x) %>% 
      melt(id.vars = "elev", value.name = "scf", 
           variable.name = "month", variable.factor = F)
    dat[, month := as.integer(month)]
    
    
    dat[, .(month, 
            gcm = x_split[1,6], 
            institute_rcm = x_split[1,7], 
            experiment = x_split[1,8], 
            period = x_split[1,9], 
            elev, 
            scf)]
    
    
  }) %>% rbindlist() -> dat_am_elev


dat_am_elev[, fp := ifelse(period == "2000-2020", "past", "future")]

# make annual
dat_am_elev <- dat_am_elev[, 
                           .(scf = mean(scf)),
                           .(gcm, institute_rcm, experiment, fp, elev)]

## DS data rofental ----------------------------------------------------------------



dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds_comp_amundsen/elev_noglacier_norca4/03_qdm/") %>% 
  lapply(readRDS) %>% 
  rbindlist -> dat_ds_elev



dat_ds_elev[, elev := tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans]

setorder(dat_ds_elev, bc_var, institute_rcm, gcm, experiment, fp, elev)


## modis data --------------------------------------------------------------



# glacier 


rr_rgi <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/rgi/modis-hr.tif")
lgl_icell_noglacier <- rr_rgi[] <= 0.1

# modis data 

rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA

rr_modis <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_scd/clim/2000-08-01_2020-07-31.tif")


dat_modis <- data.table(icell = 1:ncell(rr_modis),
                        scd = rr_modis[],
                        elev = rr_elev[])
# dat_modis <- dat_modis[!is.na(scd)]
# dat_modis$elev %>% qplot
# dat_modis[lgl_icell_common & elev > 3000, elev] %>% qplot
# dat_modis[lgl_icell_common & lgl_icell_noglacier & elev > 3000, elev] %>% qplot
# dat_modis[lgl_icell_common & lgl_icell_noglacier, elev] %>% summary

dat_modis[, elev_f := cut(elev, breaks = c(seq(-200, 3600, by = 100), 4200), dig.lab = 5)]

## rofental modis ----------------------------------------------------------

rr_template_rofental <- raster("data/rofental_modis_laea_flo.tif")

icell_hr_rofental <- cellsFromExtent(rr_modis, rr_template_rofental)

dat_modis2_rofental <- dat_modis[lgl_icell_noglacier & icell %in% icell_hr_rofental & !is.na(scd), 
                                 .(scd = mean(scd),
                                   nn = .N),
                                 .(elev_f)]


dat_plot_obs <- dat_modis2_rofental[, .(elev = tstrsplit(elev_f, ",") %>% 
                                          sapply(readr::parse_number) %>% 
                                          rowMeans,
                                        scf = scd/365.25)]

## rofental: BC ensemble ------------------------------------------------




load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/clim-02.rda")
dat_aux <- readRDS("data/auxiliary-03-modis.rds")



rr_template_rofental <- raster("data/rofental_modis_laea_flo.tif")
rr_template_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")

icell_rofental <- cellsFromExtent(rr_template_lr, rr_template_rofental)

# 
# dat_aux[icell %in% icell_rofental]
# dat_aux[icell %in% icell_rofental, qplot(alt, binwidth = 250)]
dat_bc_rofental <- clim_qdm[icell %in% icell_rofental] %>% 
  merge(dat_aux, by = "icell")

dat_bc_rofental[, alt] %>% summary
dat_bc_rofental[, alt] %>% qplot
# dat_bc_rofental[, alt_f := cut(alt, breaks = seq(1700, 3000, by = 100), dig.lab = 5)]
dat_bc_rofental[, alt_f := cut(alt, breaks = seq(1750, 3000, by = 250), dig.lab = 5)]
dat_bc_rofental[, .N, keyby = alt_f]

dat_bc_rofental_elev <- dat_bc_rofental[, 
                                        .(snc = mean(snc_qdm)), 
                                        .(alt_f, rcm, gcm, experiment, fp)]


## combine all -------------------------------------------------------------

dat_plot_rofental <- rbind(
  
  dat_am_elev[, .(gcm, institute_rcm, experiment, fp, 
                  elev, 
                  scf,
                  region = "Rofental", ff = "AMUNDSEN")]
  ,
  dat_ds_elev[, .(gcm, institute_rcm, experiment, fp, 
                  elev = tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans, 
                  scf,
                  region = "Rofental", ff = "Downscaling")]
  ,
  dat_bc_rofental_elev[, .(gcm, institute_rcm = rcm, experiment, fp, 
                           elev = tstrsplit(alt_f, ",") %>% sapply(readr::parse_number) %>% rowMeans, 
                           scf = snc,
                           region = "Rofental", ff = "Bias adjustment")]
)




## subset to rcms with (almost) full rofental coverage --------------------------------


rcms_to_keep <- c(
  "CLMcom-CCLM4-8-17", 
  "CLMcom-ETH-COSMO-crCLIM-v1-1",
  # "CNRM-ALADIN63", # 7 lr pixels missing
  "IPSL-WRF381P",
  "SMHI-RCA4" # 2 lr pixels missing
)


dat_plot_rofental2 <- dat_plot_rofental[ff == "AMUNDSEN" | 
                                (ff != "AMUNDSEN" & institute_rcm %in% rcms_to_keep)]


dat_plot_rofental2[, .(ff, institute_rcm, gcm, experiment)] %>% 
  unique %>% 
  with(table(ff, experiment))




# comb alps and rofental ---------------------------------------------------------------------

dat_plot <- rbind(dat_plot_alps2, dat_plot_rofental2)


# plot diff ---------------------------------------------------------------

# diff wrt past (not MODIS)

dat_plot_diff <- dat_plot[fp == "future"] %>% 
  merge(dat_plot[fp == "past", .(gcm, institute_rcm, experiment, elev, region, ff, scf_ref = scf)])

dat_plot_diff[, scf_diff := scf - scf_ref]

dat_plot_diff_mmm <- dat_plot_diff[,
                                   .(mod_mean_diff = mean(scf_diff),
                                     mod_min_diff = min(scf_diff),
                                     mod_max_diff = max(scf_diff)),
                                   .(ff, fp, experiment, region, elev)]

setorder(dat_plot_diff_mmm, ff, fp, experiment, region, elev)

dat_plot_diff_mmm[, period := ifelse(fp == "past", "2001-2020", "2071-2100")]
dat_plot_diff_mmm[, experiment_f := ifelse(experiment == "rcp26", "RCP2.6", "RCP8.5")]
dat_plot_diff_mmm[, period_experiment := paste0(period, " (", experiment_f, ")")]
dat_plot_diff_mmm[, region_f := fct_recode(factor(region, levels = c("Rofental", "GAR")),
                                           "Ötztal Alps" = "Rofental")]

# dat_plot_diff_mmm <- dat_plot_diff_mmm[!(elev >= 3500 & region == "Ötztal Alps")]

dat_anno <- data.table(region_f = "Ötztal Alps",
                       xmin = -Inf, xmax = Inf, ymin = elev_th, ymax = 3750)

gg_diff <-
  dat_plot_diff_mmm %>% 
  ggplot(aes(mod_mean_diff*100, elev, 
             xmin = mod_min_diff*100, xmax = mod_max_diff*100,
             colour = ff, fill = ff))+
  geom_vline(xintercept = 0)+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_path()+
  geom_rect(inherit.aes = F, data = dat_anno,
            aes(xmin = xmin, xmax = xmax, ymin = ymin, ymax = ymax),
            fill = "grey70", alpha = 0.3)+
  facet_grid(. ~ region_f + experiment_f)+
  # scale_x_continuous(labels = scales::percent)+
  scale_color_brewer("Model/Method", 
                     palette = "Dark2", aesthetics = c("color", "fill"))+
  theme_bw()+
  theme(panel.grid.minor = element_blank(),
        panel.spacing.x = unit(10, "pt"))+
  xlab("Absolute difference in annual snow cover fraction [percentage points]")+
  ylab("Elevation [m]")




# plot diff rel ---------------------------------------------------------------

# diff wrt past (not MODIS)

dat_plot_diff[, scf_diff_rel := (scf - scf_ref)/scf_ref]

dat_plot_diff_rel_mmm <- dat_plot_diff[,
                                       .(mod_mean_diff_rel = mean(scf_diff_rel),
                                         mod_min_diff_rel = min(scf_diff_rel),
                                         mod_max_diff_rel = max(scf_diff_rel)),
                                       .(ff, fp, experiment, region, elev)]

setorder(dat_plot_diff_rel_mmm, ff, fp, experiment, region, elev)

dat_plot_diff_rel_mmm[, period := ifelse(fp == "past", "2001-2020", "2071-2100")]
dat_plot_diff_rel_mmm[, experiment_f := ifelse(experiment == "rcp26", "RCP2.6", "RCP8.5")]
dat_plot_diff_rel_mmm[, period_experiment := paste0(period, " (", experiment_f, ")")]
dat_plot_diff_rel_mmm[, region_f := fct_recode(factor(region, levels = c("Rofental", "GAR")),
                                               "Ötztal Alps" = "Rofental")]

# dat_plot_diff_rel_mmm <- dat_plot_diff_rel_mmm[!(elev >= 3500 & region == "Ötztal Alps")]

dat_anno <- data.table(region_f = "Ötztal Alps",
                       xmin = -Inf, xmax = Inf, ymin = elev_th, ymax = 3750)

gg_diff_rel <-
  dat_plot_diff_rel_mmm %>% 
  ggplot(aes(mod_mean_diff_rel, elev, 
             xmin = mod_min_diff_rel, xmax = mod_max_diff_rel,
             colour = ff, fill = ff))+
  geom_vline(xintercept = 0)+
  geom_ribbon(alpha = 0.3, linetype = "blank")+
  geom_path()+
  geom_rect(inherit.aes = F, data = dat_anno,
            aes(xmin = xmin, xmax = xmax, ymin = ymin, ymax = ymax),
            fill = "grey70", alpha = 0.3)+
  facet_grid(. ~ region_f + experiment_f)+
  scale_x_continuous(labels = scales::percent)+
  scale_color_brewer("Model/Method", 
                     palette = "Dark2", aesthetics = c("color", "fill"))+
  theme_bw()+
  theme(panel.grid.minor = element_blank(),
        panel.spacing.x = unit(10, "pt"))+
  xlab("Relative change in annual snow cover fraction")+
  ylab("Elevation [m]")


gg_leg <- get_legend(gg_diff)
gg_plots <- plot_grid(gg_diff + theme(legend.position = "none"), 
                      gg_diff_rel + theme(legend.position = "none"), 
                      labels = c("(a)", "(b)"),
                      ncol = 1)

gg_out <- plot_grid(gg_plots, gg_leg, rel_widths = c(1, .2))

ggsave(filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/03-revision-2/comp-bc-ds-with-rofental.png",
       gg_out,
       width = 10,
       height = 5)


ggsave(filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/04-production/Figure 6.pdf",
       gg_out,
       width = 10,
       height = 5)



# numbers -----------------------------------------------------------------


dat_am_elev[!is.na(scf), summary(elev)]

dat_plot_diff_mmm
# needs # per elev to be accurate
dat_plot_diff_mmm[, .(mean(mod_mean_diff)), .(ff, experiment, region)]
dat_plot_diff_rel_mmm[, .(mean(mod_mean_diff_rel)), .(ff, experiment, region)]

# EOF ---------------------------------------------------------------------






