# calc SCD annual (Aug-Jul)

library(fs)
library(raster)
library(data.table)
library(magrittr)
library(foreach)
library(lubridate)


all_files <- dir_ls("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/05_temporal_complete_max10d/")
all_files %>% 
  path_file %>% 
  ymd_hms(truncated = 3) %>% 
  as.Date -> all_dates

dat_fn <- data.table(
  fn = all_files,
  date = all_dates
)

mitmatmisc::add_hydro_year(dat_fn, 8)
dat_fn

all_year <- sort(unique(dat_fn$hydro_year))

# loop --------------------------------------------------------------------

mitmatmisc::init_parallel_ubuntu(12)

foreach(
  i_year = all_year
) %dopar% {
  
  dat_i <- dat_fn[hydro_year == i_year]
  
  rr0 <- raster(dat_i[1, fn])
  rr_count <- raster(rr0)
  rr_count[rr0 != 5] <- 0L
  rr_count[rr0 == 5] <- NA_integer_
  
  
  zz <- foreach(
    # i_row = 1:10
    i_row = 1:nrow(dat_i)
  ) %do% {
    
    rr_i <- raster(dat_i[i_row, fn])
    rr_count[rr_i == 1] <- rr_count[rr_i == 1] + 1L
    return(NULL)
  }
  
  
  fn_out <- path(
    "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_scd/annual/",
    paste0(min(dat_i$date), "_", max(dat_i$date)),
    ext = "tif"
  )
  
  writeRaster(rr_count,
              filename = fn_out,
              overwrite = T)
  return(fn_out)
}


