# calc SCD clim monthly

library(fs)
library(raster)
library(data.table)
library(magrittr)
library(foreach)
library(lubridate)


all_files <- dir_ls("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/05_temporal_complete_max10d/")
all_files %>% 
  path_file %>% 
  ymd_hms(truncated = 3) %>% 
  as.Date -> all_dates

dat_fn <- data.table(
  fn = all_files,
  date = all_dates
)


dat_fn <- dat_fn[date >= "2000-08-01" & date <= "2020-07-31"]

mitmatmisc::add_hydro_year(dat_fn, 8)
dat_fn

dat_fn[, season := mitmatmisc::season_fct(month(date))]
dat_fn[, .N, hydro_year]




# loop --------------------------------------------------------------------

mitmatmisc::init_parallel_ubuntu(4)

foreach(
  i_season = levels(dat_fn$season)
) %dopar% {
  
  dat_i <- dat_fn[season == i_season]
  
  # consistent to ds-clim (remove incomplete JJA seasons start and end)
  if(i_season == "JJA"){
    dat_i[, hydro_year := year(date)]

  }

  dat_n_year <- dat_i[, .N, hydro_year] %>% .[N >= 90] 
  dat_i <- dat_i[hydro_year %in% dat_n_year$hydro_year]

  
  rr0 <- raster(dat_i[1, fn])
  rr_count <- raster(rr0)
  rr_count[rr0 != 5] <- 0L
  rr_count[rr0 == 5] <- NA_integer_
  
  
  zz <- foreach(
    # i_row = 1:10
    i_row = 1:nrow(dat_i)
  ) %do% {
    
    rr_i <- raster(dat_i[i_row, fn])
    rr_count[rr_i == 1] <- rr_count[rr_i == 1] + 1L
    return(NULL)
  }
  
  # normalize to avg
  rr_clim <- rr_count / nrow(dat_n_year)
  
  fn_out <- path(
    "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_scd/clim-seasonal/",
    i_season,
    ext = "tif"
  )
  
  writeRaster(rr_clim,
              filename = fn_out,
              overwrite = T)
  return(fn_out)
}


