# calc SCD clim monthly

library(fs)
library(raster)
library(data.table)
library(magrittr)
library(foreach)
library(lubridate)


all_files <- dir_ls("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/05_temporal_complete_max10d/")
all_files %>% 
  path_file %>% 
  ymd_hms(truncated = 3) %>% 
  as.Date -> all_dates

dat_fn <- data.table(
  fn = all_files,
  date = all_dates
)

mitmatmisc::add_hydro_year(dat_fn, 8)
dat_fn

dat_fn[, nn_avail := .N, .(year(date), month(date))]
dat_fn[, nn_max := days_in_month(date)]

dat_fn[nn_avail == nn_max] %>% with(table(year(date), month(date)))
dat_fn[nn_avail == nn_max] %>% with(table(hydro_year, month(date)))

# only 2001 - 2019 hydro years full
all_year <- 2001:2019

dat_fn_sub <- dat_fn[hydro_year %in% all_year]

# loop --------------------------------------------------------------------

mitmatmisc::init_parallel_ubuntu(4)

foreach(
  i_month = 1:12
) %dopar% {
  
  dat_i <- dat_fn_sub[month(date) == i_month]
  
  rr0 <- raster(dat_i[1, fn])
  rr_count <- raster(rr0)
  rr_count[rr0 != 5] <- 0L
  rr_count[rr0 == 5] <- NA_integer_
  
  
  zz <- foreach(
    # i_row = 1:10
    i_row = 1:nrow(dat_i)
  ) %do% {
    
    rr_i <- raster(dat_i[i_row, fn])
    rr_count[rr_i == 1] <- rr_count[rr_i == 1] + 1L
    return(NULL)
  }
  
  # normalize to avg
  rr_clim <- rr_count / length(all_year)
  
  fn_out <- path(
    "/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD_monthly/hydro_year_2001_2019/",
    sprintf("%02i_%s", i_month, month.abb[i_month]),
    ext = "tif"
  )
  
  writeRaster(rr_clim,
              filename = fn_out,
              overwrite = T)
  return(fn_out)
}


