# test biascorrection

# month by month
# and in space chunks x*x pixels


library(data.table)
library(magrittr)
library(eurocordexr)
library(foreach)
library(lubridate)
library(fs)
library(raster)
library(MBC)

source("R/functions/QDM_snc.R")
source("R/functions/MBC_helpers.R")
source("R/functions/MBCn_snc.R")
source("R/functions/create_mat_neigh.R")
# source("R/functions/yday_window.R")
source("R/functions/raster_chunks.R")

mitmatmisc::init_parallel_ubuntu(13) # with 32gb RAM: 3 maximum


# parameters --------------------------------------------------------------

path_out <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/04_mbc_qdm/"
dir_create(path_out)

# (spatial window)
# k_space <- 2
# (time window)
# k_time <- 45
# (spatial chunk size (x*x))
k_space_chunk <- 19

# qdm options
qdm_opt_ratio = T
qdm_opt_trace = 0.001



# data --------------------------------------------------------------------

dat_th <- readRDS("data/auxiliary-02-snow-max-threshold.rds")
dat_modis <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/modis-10d-02-snc.rds")
# dat_modis[, yday := yday(date)]
dat_modis[, month := month(date)]
dat_modis <- dat_modis[!is.na(snc)]
dat_modis <- dat_modis[date >= "2000-08-01" & date <= "2020-07-31"]

# sample raster (for adjacent)
rr_sample <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/modis_10d/modis_snc_55x133_2000-02-24.tif")



# prep rcm data 

dat_inv_past <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/rcm/2001-2020/", add_files = T)
dat_inv_future <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/rcm/2071-2100/", add_files = T)

# dat_loop_rcm <- dat_inv_future[, .(gcm, institute_rcm, experiment, ensemble, downscale_realisation)]
# dat_loop_rcm <- dat_inv_future[experiment == "rcp26" & institute_rcm != "DMI-HIRHAM5", 
#                                .(gcm, institute_rcm, experiment, ensemble, downscale_realisation)]

# no HIRHAM
dat_loop_rcm <- dat_inv_future[institute_rcm != "DMI-HIRHAM5",
                               .SD, 
                               institute_rcm,
                               .SDcols = c("gcm", "experiment", "ensemble", "downscale_realisation")]

# main loop ---------------------------------------------------------------


# i_rcm <- 2

foreach(
  i_rcm = 1:nrow(dat_loop_rcm),
  .inorder = F
) %dopar% {
    
  dat_loop_rcm[i_rcm] %>% 
    unlist %>% 
    paste0(collapse = "_") -> fn_rcm
  fn_out_past <- path(path_out, paste0(fn_rcm, "_past"), ext = "rds")
  fn_out_fut <- path(path_out, paste0(fn_rcm, "_future"), ext = "rds")
  if(file_exists(fn_out_past) | file_exists(fn_out_fut)) return(paste0("Skipped: ", fn_rcm))
  # dat_loop_rcm[i_rcm] %>% merge(dat_inv_past, by = names(dat_loop_rcm))
  
  
  
  # past data
  dat_inv_past %>% 
    merge(dat_loop_rcm[i_rcm], by = names(dat_loop_rcm)) %>% 
    .$list_files %>% 
    .[[1]] -> files_past1
  dat_inv_past[experiment == "historical"] %>% 
    merge(dat_loop_rcm[i_rcm], by = names(dat_loop_rcm)[names(dat_loop_rcm) != "experiment"]) %>% 
    .$list_files %>% 
    .[[1]] -> files_past2
  files_past <- c(files_past1, files_past2)
  
  dat_past <- foreach(i_fn = files_past, .final = rbindlist) %do% {
    nc_grid_to_dt(i_fn, "snc", interpolate_to_standard_calendar = T)
  }
  
  # future data
  dat_inv_future %>% 
    merge(dat_loop_rcm[i_rcm], by = names(dat_loop_rcm)) %>% 
    .$list_files %>% 
    .[[1]] -> files_future
  
  dat_future <- foreach(i_fn = files_future, .final = rbindlist) %do% {
    nc_grid_to_dt(i_fn, "snc", interpolate_to_standard_calendar = T)
  }
  
  # remove cells with implausible snow (accumulation) & 
  # subset to common cells (modis & rcm)
  icell_rcm <- dat_th[rcm == dat_loop_rcm[i_rcm, institute_rcm] & snow_plausible == "yes", icell]
  icell_modis <- unique(dat_modis$icell)
  icell_yes <- icell_rcm[icell_rcm %in% icell_modis]
  
  dat_past <- dat_past[icell %in% icell_yes]
  dat_future <- dat_future[icell %in% icell_yes]
  
  # other req data
  setkey(dat_past, icell, date)
  setkey(dat_future, icell, date)
  
  # dat_past[, yday := yday(date)]
  # dat_future[, yday := yday(date)]
  dat_past[, month := month(date)]
  dat_future[, month := month(date)]
  
  dat_past[, snc := snc/100]
  dat_future[, snc := snc/100]
  
  dat_past[, snc_qdm := NA_real_]
  dat_future[, snc_qdm := NA_real_]
  
  # split space into chunks
  dat_loop_space <- raster_chunks(rr_sample, k_space_chunk)
  
  # subset to icell available and at one third of the grid cells in chunks (or not)
  dat_loop_space <- dat_loop_space[icell %in% icell_yes]
  dat_loop_space[, nn := .N, chunk]
  # dat_loop_space <- dat_loop_space[nn > (k_space_chunk*k_space_chunk)/3]
  
  # for testing
  # dat_aux <- readRDS("data/auxilary-03-modis.rds")
  # dat_loop_space %>% merge(dat_aux, by = "icell") %>% ggplot(aes(x,y,fill = chunk))+geom_raster()
  # i_chunk <- "x4y1"
  
  foreach(
    # i_icell = dat_icell$icell[1:10]
    i_chunk = unique(dat_loop_space$chunk)
  ) %do% {
    
    foreach(
      i_month = 1:12
    ) %do% {
        
      # i_icell_neigh <- dat_loop_space[icell == i_icell, icell_neigh]
      i_icell_neigh <- dat_loop_space[chunk == i_chunk, icell]
      
      obs_hist <- dat_modis[icell %in% i_icell_neigh & month == i_month]
      mod_hist <- dat_past[icell %in% i_icell_neigh & month == i_month]
      mod_fut <- dat_future[icell %in% i_icell_neigh & month == i_month]
      
      o.c.orig <- na.omit(dt2wide(obs_hist))
      m.c.orig <- dt2wide(mod_hist)
      m.p.orig <- dt2wide(mod_fut)
      
      # mod_hist_eval <- dat_past[icell == i_icell & month == i_month]
      # mod_fut_eval <- dat_future[icell == i_icell & month == i_month]
      
      # try({
      
        l_mbc <- MBCn_snc(o.c = o.c.orig, 
                          m.c = m.c.orig, 
                          m.p = m.p.orig,
                          # ratio = qdm_opt_ratio, 
                          trace = qdm_opt_trace,
                          # n.escore = 1000,
                          # iter = 50,
                          # return.all = T,
                          silent = T)
        
        dat_past[icell %in% i_icell_neigh & month == i_month,
                 snc_qdm := as.numeric(l_mbc$mhat.c)]
        
        dat_future[icell %in% i_icell_neigh & month == i_month,
                   snc_qdm := as.numeric(l_mbc$mhat.p)]
        
      # })
      
      
      return(NULL)
    }
    
  }
  
  
  # save results
  saveRDS(dat_past, fn_out_past)
  saveRDS(dat_future, fn_out_fut)
  
  return(paste0("Done: ", fn_rcm))
  
  
  
}

