# implement filling of scf_th based on cp curves from similar lr-pixels

# search up to 50 similar pixels 
#   (optional : sub with wasserstein dist < 200-300, 
#    but for highest elev that does not work good...)
# add buffer around alt_f (+- 100-200m) 

library(raster)
library(data.table)
library(magrittr)
library(foreach)
library(fs)

source("R/functions/cp_helpers.R")
source("R/functions/cp_mae.R")


# settings ----------------------------------------------------------------

n_sim_pixels <- 50 # number of lr pixels to look for
alt_buffer <- 150 # meters to look for similar cp

path_out_single_icell <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_impute_similar_icell/"
dir_create(path_out_single_icell)

# prep data ---------------------------------------------------------------

rr_alt <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_alt[rr_alt < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_alt[rr_corine == 0] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
# rr_alt[rr_water == 1] <- NA #problems with aggregate later

rr_scf <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v02_monotonic.tif")
rr_scf
# plot(rr_scf)
# plot(is.na(rr_scf))

dat_scf <- data.table(icell = 1:ncell(rr_scf),
                      water = rr_water[],
                      scf_th = rr_scf[],
                      alt = rr_alt[])
dat_scf[, alt_f := cut(alt, c(-150, 0, 1:47*100, 4800), dig.lab = 5)]




# add alt_diff
rr_alt_lr <- aggregate(rr_alt,
                       fact = c(37, 53),
                       fun = mean,
                       na.rm = F)

mat_alt_lr <- raster::as.matrix(rr_alt_lr)
mat_alt_lr2hr <- mat_alt_lr[rep(1:55, each = 53),
                            rep(1:133, each = 37)]
mat_alt_lr2hr <- mat_alt_lr2hr[1:2867, 1:4901]
rr_alt_lr2hr <- raster(rr_alt)
rr_alt_lr2hr[] <- mat_alt_lr2hr

dat_scf[, alt_lr := rr_alt_lr2hr[]]
dat_scf[, alt_diff := alt_lr - alt]

# add lr2hr icell
rr_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")
rr_lr[] <- 1:ncell(rr_lr)
mat_icell_lr <- raster::as.matrix(rr_lr)
mat_icell_lr2hr <- mat_icell_lr[rep(1:55, each = 53),
                                rep(1:133, each = 37)]
mat_icell_lr2hr <- mat_icell_lr2hr[1:2867, 1:4901]
rr_icell_lr2hr <- raster(rr_alt)
rr_icell_lr2hr[] <- mat_icell_lr2hr

dat_scf[, icell_lr := rr_icell_lr2hr[]]


# remove water and not available pixels
dat_scf_eval <- dat_scf[!is.na(alt) & !is.na(alt_lr) & water == 0]


# prep for loop 
dat_scf_eval_missing <- dat_scf_eval[, 
                                     .(n_na = sum(is.na(scf_th)),
                                       n_avail = sum(!is.na(scf_th))),
                                     .(icell_lr)]

lr_icell <- dat_scf_eval_missing[n_na > 0, icell_lr]

# dist lr icells
mat_dist <- as.matrix(readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/dist-01-wasserstein.rds"))


# cp data 

rr_cp <- brick("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp/cp_20bins_monotonic.tif")
band_lbl <- levels(cut(0:100/100, 
                       breaks = c(0, 0.001, 1:19/20, 0.999, 1), 
                       include.lowest = T))

mat_cp_full <- raster::as.matrix(rr_cp) # icell*l

mat_cp_full[mat_cp_full < 0] <- NA
colnames(mat_cp_full) <- band_lbl
rownames(mat_cp_full) <- 1:ncell(rr_cp)


# main loop ---------------------------------------------------------------

mitmatmisc::init_parallel_ubuntu(10)

foreach(
  i_lr_icell = lr_icell,
  # i_lr_icell = lr_icell[c(1:10, 13:26)], # test n_na>=10
  # i_lr_icell = lr_icell[c(1:24)],
  .inorder = F
) %dopar% {
  
  fn_out <- path(path_out_single_icell,
                 i_lr_icell,
                 ext = "rds")
  if(file_exists(fn_out)) return(paste0("Skipped: ", i_lr_icell))
  
  i_lr_icell_similar <- icell_min_dist(i_lr_icell, n_sim_pixels)$icell_with_min_dist
  
  i_hr_missing <- dat_scf_eval[icell_lr == i_lr_icell & is.na(scf_th), icell]
  
  dat_scf_eval_i <- dat_scf_eval[!is.na(scf_th) & 
                         icell_lr %in% c(i_lr_icell, i_lr_icell_similar)]
  
  lapply(as.character(i_hr_missing), function(x){
    
    x_cp <- mat_cp_full[x, ]
    x_alt <- dat_scf_eval[icell == as.integer(x), alt]
    
    icell_to_check <- dat_scf_eval_i[between(alt, 
                                             x_alt - alt_buffer, 
                                             x_alt + alt_buffer), 
                                  icell]
    mat_cp_to_check <- mat_cp_full[icell_to_check, , drop = F]
    
    cp_mae(x, x_cp, mat_cp_to_check)
    
  }) %>% rbindlist -> dat_next_icell
  
  
  saveRDS(dat_next_icell, fn_out)
  return(paste0(format(Sys.time()),
                " | ", i_lr_icell))
  
}






# EOF ---------------------------------------------------------------------


