# downscale everything (copy of python code)

library(raster)
library(data.table)
library(magrittr)
library(fs)
library(foreach)


path_out_main <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full/"
path_out_main_rofental <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental/"

# path_out_main <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test_r/"
# path_out_main_rofental <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test_r_rofental/"


# dirs_bc_variant <- c(
#   "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/02_qm/",
#   "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/03_qdm/",
#   "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/04_mbc_qdm/"
# )
# 
# names_bc_column <- c(
#   "snc_qm",
#   "snc_qdm",
#   "snc_qdm"
# )

dirs_bc_variant <- c(
  # "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/02_qm/",
  "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/03_qdm/"
)
names_bc_column <- c(
  # "snc_qm",
  "snc_qdm"
)

lapply(seq_along(dirs_bc_variant), function(i){
  
  x <- dirs_bc_variant[i]
  data.table(bc_var = path_file(x),
             name_bc_column = names_bc_column[i],
             file_rcm = dir_ls(x))
  
}) %>% rbindlist() -> dat_loop


# scf and rasters ---------------------------------------------------------

rr_lr <- raster("data/sample_modis_snc_55x133_2000-03-18.tif")

rr_scf_th <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v04_imputed_alt.tif")
mat_scf_th <- raster::as.matrix(rr_scf_th)

rr_rofental <- raster("data/rofental_modis_laea_flo.tif")
cells_rofental <- cellsFromExtent(rr_scf_th, rr_rofental)
xy_rofental <- rowColFromCell(rr_scf_th, cells_rofental)
xy_rofental <- apply(xy_rofental, 2, function(x) sort(unique(x)))


# looping -----------------------------------------------------------------


mitmatmisc::init_parallel_ubuntu(13)


# for(i in seq_along(dirs_bc_variant)){
#   
#   i_dir <- dirs_bc_variant[i]
#   i_files <- dir_ls(i_dir)

foreach(
  i = 1:nrow(dat_loop),
  .inorder = F
) %dopar% {
  
  i_fn <- dat_loop[i, file_rcm]
  i_bc_var <- dat_loop[i, bc_var]
  
  # paths
  i_rcm <- path_ext_remove(path_file(i_fn))
  
  path_out_main_i <- path(path_out_main,
                          i_bc_var,
                          i_rcm)
  path_out_main_rofental_i <- path(path_out_main_rofental,
                                   i_bc_var,
                                   i_rcm)
  if(dir_exists(path_out_main_i)){
    return(paste0("Skipped: ", i_rcm))
  } else {
    dir_create(path_out_main_i)
    dir_create(path_out_main_rofental_i)
  }
  
  
  
  # bc data
  dat_bc <- readRDS(i_fn)
  dat_bc[, year := year(date)]
  dat_bc[, month := month(date)]
  
  dat_bc_loop <- unique(dat_bc[, .(year, month)])
  
 
  for(i_ym in 1:nrow(dat_bc_loop)){
    
    i_year <- dat_bc_loop[i_ym, year]
    i_month <- dat_bc_loop[i_ym, month]
    
    
    
    dat_bc_i <- dat_bc[year == i_year & month == i_month]
    
    fn_out <- path(path_out_main_i,
                   sprintf("%4i-%02i.tif", i_year, i_month))
    
    
    i_dates <- sort(unique(dat_bc_i$date))
    mat_scd <- matrix(0, ncol = ncol(mat_scf_th), nrow = nrow(mat_scf_th))
    
    for(d in seq_along(i_dates)){
      
      i_date <- i_dates[d]
      dat_bc_i2 <- dat_bc_i[date == i_date]
      
      # upscale
      rr_lr_bc <- raster(rr_lr)
      rr_lr_bc[dat_bc_i2$icell] <- dat_bc_i2[[dat_loop[i, name_bc_column]]]
      
      mat_lr_bc <- raster::as.matrix(rr_lr_bc)
      mat_lr_bc_2hr <- mat_lr_bc[rep(1:55, each = 53),
                                 rep(1:133, each = 37)]
      mat_lr_bc_2hr <- mat_lr_bc_2hr[1:2867, 1:4901]
      
      # binary snow
      mat_binary_snow <- (mat_lr_bc_2hr > mat_scf_th)
      
      # crop and save rofental
      fn_out_rofental <- path(path_out_main_rofental_i,
                              i_date, ext = "tif")
      mat_rofental <- mat_binary_snow[xy_rofental$row, xy_rofental$col]
      rr_out_rofental <- raster(rr_rofental)
      rr_out_rofental[] <- mat_rofental
      writeRaster(rr_out_rofental, fn_out_rofental, overwrite = T)
      
      
      # update monthly scd
      mat_scd <- mat_scd + mat_binary_snow
      
      
    }
    
    # mat_scd[is.na(mat_scf_th)] <- NA # not needed
    rr_out <- raster(rr_scf_th)
    rr_out[] <- mat_scd
    
    
    writeRaster(rr_out, fn_out, overwrite = T)
    
    
    
  }
  
  
  return(paste0(format(Sys.time()),
                " | ", 
                path_ext_remove(path_file(i_fn))))
  
  
  
  
}





