# make ensemble mean of annual SCD
# - only for five RCMs in paper
# - common icell
# - by rcp

library(raster)
# library(stars)
# library(ggplot2)
# library(scico)
library(data.table)
library(magrittr)
library(fs)



rr_rgi <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/rgi/modis-hr.tif")




# common icell ------------------------------------------------------------
# 
# dat_icell <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/ds/common-icell.rds")
# names(dat_icell)
rcms_common <- c("MODIS", "CLMcom-CCLM4-8-17", "CLMcom-ETH-COSMO-crCLIM-v1-1",
                 "CNRM-ALADIN63", "IPSL-WRF381P", "SMHI-RCA4")
# lgl_icell_common <- dat_icell[, 
#                               apply(.SD, 1, all), 
#                               .SDcols = rcms_common]




# rcm ensemble mean -------------------------------------------------------

x <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim_2041_2070/03_qdm/"
dat_loop <- data.table(bc_var = path_file(x),
                       file_rcm = dir_ls(x))

dat_loop[, c("institute_rcm", "gcm", "experiment", "ensemble", "dsr", "fp") := 
           tstrsplit(path_ext_remove(path_file(file_rcm)), "_")]

dat_loop2 <- dat_loop[institute_rcm %in% rcms_common & fp == "future"]


rr_rcp26 <- stack(dat_loop2[experiment == "rcp26", file_rcm])
rr_rcp26_mean <- calc(rr_rcp26, 
                      mean, 
                      "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_2041_2070/rcp26.tif",
                      overwrite = T)


rr_rcp85 <- stack(dat_loop2[experiment == "rcp85", file_rcm])
rr_rcp85_mean <- calc(rr_rcp85, 
                      mean, 
                      "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_2041_2070/rcp85.tif",
                      overwrite = T)



# elev anomaly ------------------------------------------------------------


rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_elev[rr_corine == 0] <- NA


dat_scd <- data.table(icell = 1:ncell(rr_elev),
                      # water = rr_water[],
                      rcp26 = rr_rcp26_mean[],
                      rcp85 = rr_rcp85_mean[],
                      elev = rr_elev[])

dat_scd[!is.na(rcp26), summary(elev)]
dat_scd[, elev_f := cut(elev, c(-150, seq(-50, 3950, by = 100), 4300), dig.lab = 5)]
dat_scd_nn <- dat_scd[!is.na(rcp26), .N, keyby = elev_f]
dat_scd_nn

dat_scd[, rcp26_avg := mean(rcp26, na.rm = T), elev_f]
dat_scd[, rcp26_anom := rcp26 - rcp26_avg]

dat_scd[, rcp85_avg := mean(rcp85, na.rm = T), elev_f]
dat_scd[, rcp85_anom := rcp85 - rcp85_avg]

rr_rcp26_anom <- raster(rr_rcp26_mean)
rr_rcp26_anom[] <- dat_scd$rcp26_anom

plot(rr_rcp26_anom)

writeRaster(rr_rcp26_anom, 
            filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_2041_2070/rcp26_elev_anom.tif",
            overwrite = T)



rr_rcp85_anom <- raster(rr_rcp85_mean)
rr_rcp85_anom[] <- dat_scd$rcp85_anom

plot(rr_rcp85_anom)

writeRaster(rr_rcp85_anom, 
            filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_2041_2070/rcp85_elev_anom.tif",
            overwrite = T)





# rcm ensemble mean (glacier removed) -------------------------------------------------------



rr_rcp26_mean_rgi <- rr_rcp26_mean
rr_rcp26_mean_rgi[rr_rgi > 0.1] <- NA

writeRaster(rr_rcp26_mean_rgi, 
            "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_2041_2070/rcp26_noglacier.tif",
            overwrite = T)


rr_rcp85_mean_rgi <- rr_rcp85_mean
rr_rcp85_mean_rgi[rr_rgi > 0.1] <- NA

writeRaster(rr_rcp85_mean_rgi, 
            "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_2041_2070/rcp85_noglacier.tif",
            overwrite = T)



# elev anomaly (glacier removed) ------------------------------------------------------------


rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_corine <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/aux_laea.tif", 3)
rr_elev[rr_corine == 0] <- NA


dat_scd <- data.table(icell = 1:ncell(rr_elev),
                      # water = rr_water[],
                      rcp26 = rr_rcp26_mean_rgi[],
                      rcp85 = rr_rcp85_mean_rgi[],
                      elev = rr_elev[])

dat_scd[!is.na(rcp26), summary(elev)]
dat_scd[, elev_f := cut(elev, c(-150, seq(-50, 3750, by = 100), 4150), dig.lab = 5)]
dat_scd_nn <- dat_scd[!is.na(rcp26), .N, keyby = elev_f]
dat_scd_nn

dat_scd[, rcp26_avg := mean(rcp26, na.rm = T), elev_f]
dat_scd[, rcp26_anom := rcp26 - rcp26_avg]

dat_scd[, rcp85_avg := mean(rcp85, na.rm = T), elev_f]
dat_scd[, rcp85_anom := rcp85 - rcp85_avg]

rr_rcp26_anom_rgi <- raster(rr_rcp26_mean_rgi)
rr_rcp26_anom_rgi[] <- dat_scd$rcp26_anom

# plot(rr_rcp26_anom_rgi)

writeRaster(rr_rcp26_anom_rgi, 
            filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_2041_2070/rcp26_elev_anom_noglacier.tif",
            overwrite = T)



rr_rcp85_anom_rgi <- raster(rr_rcp85_mean_rgi)
rr_rcp85_anom_rgi[] <- dat_scd$rcp85_anom

# plot(rr_rcp85_anom_rgi)

writeRaster(rr_rcp85_anom_rgi, 
            filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_2041_2070/rcp85_elev_anom_noglacier.tif",
            overwrite = T)



# elev anomaly of change (glacier removed) --------------------------------------------------

rr_modis <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_scd/clim/2000-08-01_2020-07-31.tif")


dat_scd <- data.table(icell = 1:ncell(rr_elev),
                      # water = rr_water[],
                      rcp26 = rr_rcp26_mean_rgi[] - rr_modis[],
                      rcp85 = rr_rcp85_mean_rgi[] - rr_modis[],
                      elev = rr_elev[])

dat_scd[!is.na(rcp26), summary(elev)]
dat_scd[, elev_f := cut(elev, c(-150, seq(-50, 3750, by = 100), 4150), dig.lab = 5)]
dat_scd_nn <- dat_scd[!is.na(rcp26), .N, keyby = elev_f]
dat_scd_nn

dat_scd[, rcp26_avg := mean(rcp26, na.rm = T), elev_f]
dat_scd[, rcp26_anom := rcp26 - rcp26_avg]

dat_scd[, rcp85_avg := mean(rcp85, na.rm = T), elev_f]
dat_scd[, rcp85_anom := rcp85 - rcp85_avg]

rr_rcp26_anom_rgi <- raster(rr_rcp26_mean_rgi)
rr_rcp26_anom_rgi[] <- dat_scd$rcp26_anom

# plot(rr_rcp26_anom_rgi)

writeRaster(rr_rcp26_anom_rgi, 
            filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_2041_2070/rcp26_dscd_elev_anom_noglacier.tif",
            overwrite = T)



rr_rcp85_anom_rgi <- raster(rr_rcp85_mean_rgi)
rr_rcp85_anom_rgi[] <- dat_scd$rcp85_anom

# plot(rr_rcp85_anom_rgi)

writeRaster(rr_rcp85_anom_rgi, 
            filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_2041_2070/rcp85_dscd_elev_anom_noglacier.tif",
            overwrite = T)

