# create summary of downscaled upscaled modis

library(raster)
library(data.table)
library(magrittr)
library(fs)
library(foreach)




# paths, settings ---------------------------------------------------------


rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA

rr_th <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v05_impute_level.tif")



dat_ds <- data.table(file_ds = dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/alps-daily/"))
dat_ds[, date := lubridate::ymd(path_file(file_ds))]

dat_orig <- data.table(file_orig = dir_ls("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/05_temporal_complete_max10d/"))
dat_orig[, date := lubridate::ymd_hms(path_file(file_orig)) %>% as.Date]


dat_loop <- merge(dat_ds, dat_orig)



# loop and summarise ------------------------------------------------------


dat_summ <- foreach(
  i = 1:nrow(dat_loop),
  # i = 1:100, # test
  .final = rbindlist
) %do% {
  
  
  rr_ds <- raster(dat_loop[i, file_ds])
  rr_orig <- raster(dat_loop[i, file_orig])
  
  dat_i <- data.table(icell = 1:ncell(rr_ds),
                      value_ds = rr_ds[],
                      value_orig = rr_orig[],
                      elev = rr_elev[],
                      th_level = rr_th[])
  dat_i[, elev_fct := cut(elev, breaks = seq(-150, 4850, by = 100), dig.lab = 5)]
  
  dat_i_summ <- dat_i[!is.na(value_ds)] %>% 
    dcast(elev_fct + th_level + value_orig ~ value_ds, fun.aggregate = length) %>% 
    melt(id.vars = c("elev_fct", "th_level", "value_orig"), 
         variable.name = "value_ds",
         value.name = "N",
         variable.factor = F)
  dat_i_summ[, value_ds := as.integer(value_ds)]
  dat_i_summ[, date := dat_loop[i, date]]
  
  
  dat_i_summ
  
}


saveRDS(dat_summ, 
        "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/02-summary-daily-elev-th.rds")
