
# check mosaic ------------------------------------------------------------


dat[, .N, tile]
with(dat, table(tile, year))

fn1 <- dat[tile == "h18v04" & date == "2008-01-18", file_full]
fn2 <- dat[tile == "h19v04" & date == "2008-01-18", file_full]

rr1 <- raster(fn1)
rr2 <- raster(fn2)
rr1
rr2
plot(rr1)
plot(rr2)

rr12 <- merge(rr1, rr2)
rr21 <- merge(rr2, rr1)

plot(rr12)
plot(rr21)

rr12m <- mosaic(rr1, rr2, fun = min)
plot(rr12m)
rr21m <- mosaic(rr2, rr1, fun = min)
plot(rr21m)


writeRaster(rr12m, "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_gapfill/test.tif")

ff <- freq(rr12m)
ff
dim(rr12m)
ncell(rr12m)



# check 2020 h19
rr1 <- raster(dat[tile == "h19v04" & date == "2019-09-18", file_full])
rr1
plot(rr1)

# check aux ---------------------------------------------------------------

rr1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_gapfill/00_aux/eurac_modis_altitude_sinu.tif")
rr1[rr1 < 0] <- NA
plot(rr1)

rr2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_gapfill/00_aux/eurac_modis_aspect_sinu.tif")
plot(rr2)


rr3 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_gapfill/00_aux/eurac_modis_water_mask_sinu.tif")
plot(rr3)


# check gapfill -----------------------------------------------------------


library(raster)
dd <- "20000323"
rr1 <- raster(paste0("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_gapfill/01_original/", dd, "_120000.tif"))
rr2 <- raster(paste0("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_gapfill/02_preprocessing/", dd, "_120000.tif"))
rr3 <- raster(paste0("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_gapfill/03_temporal_conservative/", dd, "_120000.tif"))
rr4 <- raster(paste0("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_gapfill/04_temporal_complete_max10d/", dd, "_120000.tif"))
plot(stack(rr1, rr2, rr3, rr4))



# check number of raster images -------------------------------------------


path_in <- "/mnt/CEPH_PROJECTS/SNOW_3rd_gen/MOD/"

files_all <- dir_ls(path_in, recurse = T, glob = "*EURAC*.tif")

dat_mod <- data.table(file_full = files_all,
                  file_base = path_file(files_all))
dat_mod[, paste0("col", 1:10) := tstrsplit(file_base, "[.]")]
setnames(dat_mod, c("col3", "col4", "col6"), c("tile", "year", "doy"))

dat_mod[, date := make_date(year)]
yday(dat_mod$date) <- as.numeric(dat_mod$doy)

dat_mod

dat_mod[, .N, tile]
with(dat_mod, table(tile, year))

dat_mod[tile == "h18v04", .(date, in_mod = "available")] %>% 
  merge(.[, .(date = seq(min(date), max(date), by = "day"))], all = T) %>% 
  ggplot(aes(date, in_mod))+
  geom_point()+
  facet_wrap(~year(date), scales = "free_x")



# check number of dates ---------------------------------------------------

all_dates <- sort(unique(dt_modis$date))
all_dates %>% diff %>% as.numeric %>% summary
dtn <- dt_modis[, .N, icell]
dtn %>% summary

dtn[N < 7162]

dt_na <- dt_modis[is.na(snc)]
dt_na[, .N, date] %>% summary
dt_na[, .N, date] %>% .[N > 1000]


# check missing obs in modis after gapfill --------------------------------

dat_modis
# check N non-na obs 
zz <- dat_modis[!is.na(snc) & icell %in% icell_yes, .N, icell]
zz <- dat_modis[!is.na(snc), .N, icell]
zz %>% summary
zz[N < 7162]
zz[N < 6000]
# -> ok for icell_yes?

dat_modis[icell == 3913] %>% summary
dat_modis[icell == 3913] %>% 
  ggplot(aes(date, snc))+
  geom_point()+
  facet_wrap(~year(date), scales = "free_x")


# testing bc --------------------------------------------------------------

i_yday <- 15
i_icell <- 2574
i_icell <- 3099 #much snow
i_icell <- 3997


obs_hist <- dat_modis[icell %in% i_icell & month == i_month]
mod_hist <- dat_past[icell %in% i_icell & month == i_month]
mod_fut <- dat_future[icell %in% i_icell & month == i_month]

dat_icell[icell == i_icell]

rr0 <- rr_sample
rr0[i_icell] <- 100
plot(rr0)


# supplied pars
o.c = obs_hist$snc
m.c = mod_hist$snc 
m.p = mod_fut$snc
ratio = qdm_opt_ratio
trace = qdm_opt_trace
m.c.eval = mod_hist_eval$snc
m.p.eval = mod_fut_eval$snc
m.c.eval = NULL
m.p.eval = NULL

# other pars
trace.calc = 0.5 * trace
jitter.factor = 0
n.tau = NULL
ratio.max = 2
ratio.max.trace = 10 * trace
subsample = NULL
pp.type = 7


# other

plot(ecdf(m.c), verticals = T, xlim = c(0,1))
plot(ecdf(m.p), verticals = T, add = T, col = "red")
plot(ecdf(o.c), verticals = T, add = T, col = "blue")

plot(ecdf(mhat.c), verticals = T)
plot(ecdf(mhat.p), verticals = T, add = T, col = "red")
plot(ecdf(o.c), verticals = T, add = T, col = "blue")


# temp moving window ------------------------------------------------------

# req loop space

dat_icell <- data.table(icell = unique(dat_past$icell))

if(k_space == 0){
  
  dat_loop_space <- copy(dat_icell)
  dat_loop_space[, icell_neigh := icell]
  
} else {
  
  mat_neigh <- create_mat_neigh(k_space)
  dat_loop_space <- dat_icell[, 
                              .(icell_neigh = c(icell,
                                                adjacent(rr_sample,
                                                         icell,
                                                         mat_neigh)[, 2])),
                              icell]
  
}


# req loop time

# dat_yday <- data.table(yday = 1:366)
# dat_loop_time <- dat_yday[, .(yday_wind = yday_window(yday, k_time)), .(yday)]

# l_out_hist <- list()
# l_out_fut <- list()


# split raster into chunks ------------------------------------------------

rr0 <- rr_sample
rr0[] <- 1:ncell(rr0)
plot(rr0)

rr_chunk <- raster(rr_sample)

nc <- ncol(rr_chunk)
nr <- nrow(rr_chunk)
#mat_chunk <- 
vals <- as.numeric(paste0(
  rep((1:nc - 1) %/% 5, nr),
  rep((1:nr - 1) %/% 5, each = nc)
))
(1:nc - 1) %/% 5
(1:nr - 1) %/% 5
  
rr_chunk[] <- vals
plot(log(rr_chunk))  

dat_zz <- data.table(icell = 1:ncell(rr_sample),
                     vals = (paste0(
                       "x", rep((1:nc - 1) %/% 5, nr),
                       "y", rep((1:nr - 1) %/% 5, each = nc)
                     )))

as.data.frame(rr_chunk, xy = T) %>% 
  ggplot(aes(x,y,fill=factor(layer)))+geom_raster()

rr0 %>% plot
rr0[dat_zz[vals == "x0y0", icell]] <- 20000
plot(rr0)


# check mbc ---------------------------------------------------------------


rr0 <- setValues(rr_sample, as.factor(dat_loop_space$chunk))
plot(rr0)
plot(rr_sample)

i_chunk <- "x1y2"



plot(ecdf(mod_hist$snc))
plot(ecdf(mod_fut$snc), add = T, col = "red")
plot(ecdf(obs_hist$snc), add = T, col = "blue")

plot(ecdf(l_qdm$mhat.c), verticals = T)
plot(ecdf(l_qdm$mhat.p), add = T, col = "red", verticals = T)
plot(ecdf(obs_hist$snc), add = T, col = "blue")


# mbc param ---------------------------------------------------------------

iter = 30
ratio.seq = rep(TRUE, ncol(o.c))
trace = 0.001 # was 0.05
trace.calc = 0.5 * trace
jitter.factor = 0
n.tau = NULL
ratio.max = 2
ratio.max.trace = 10 * trace
ties = "first" # was first
qmap.precalc = FALSE
rot.seq = NULL
silent = TRUE # was FALSE
n.escore = 0
return.all = FALSE
subsample = NULL
pp.type = 7



plot(ecdf(mod_hist$snc))
plot(ecdf(mod_fut$snc), verticals = T, add = T, col = "red")
plot(ecdf(obs_hist$snc), verticals = T, add = T, col = "blue")

plot(ecdf(m.c.qmap), verticals = T)
plot(ecdf(m.p.qmap), verticals = T, add = T, col = "red")
plot(ecdf(obs_hist$snc), verticals = T, add = T, col = "blue")

plot(ecdf(m.c), verticals = T)
plot(ecdf(m.p), verticals = T, add = T, col = "red")
plot(ecdf(obs_hist$snc), verticals = T, add = T, col = "blue")



plot(cor(o.c.orig), cor(m.c.orig))
plot(cor(o.c.orig), cor(m.c.qmap))

plot(cor(o.c.orig), cor(m.c))
plot(cor(m.p))



# check test mbc ----------------------------------------------------------


dat_mbc <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/03_bc_test_mbc/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future.rds")
dat_mbc <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/03_bc_test_mbc/CNRM-ALADIN63_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v2_past.rds")

dat_mbc[is.na(snc_qdm)]
dat_mbc[is.na(snc_qdm), .(icell, month)] %>% 
  unique -> dat_mbc_na
dat_mbc_na
dat_mbc_na[month == 1]

dat_mbc_na[month == 1 & icell %in% i_icell_neigh]



# test QM -----------------------------------------------------------------


qm_obj <- fitQmapQUANT(obs_hist$snc, mod_hist$snc, wet.day = F, qstep = 0.001)

snc_qm_hist <- doQmapQUANT(mod_hist$snc, qm_obj, type = "tricub")
snc_qm_fut <- doQmapQUANT(mod_fut$snc, qm_obj, type = "tricub")

snc_qm_hist <- doQmapQUANT(mod_hist$snc, qm_obj)
snc_qm_fut <- doQmapQUANT(mod_fut$snc, qm_obj)


plot(ecdf(mod_hist$snc), verticals = T)
plot(ecdf(mod_fut$snc), verticals = T, add = T, col = "red")
plot(ecdf(obs_hist$snc), verticals = T, add = T, col = "blue")

plot(ecdf(l_qm$mod.hat.c), verticals = T)
plot(ecdf(l_qm$mod.hat.p), verticals = T, add = T, col = "red")
plot(ecdf(obs_hist$snc), verticals = T, add = T, col = "blue")



plot(ecdf(mod_hist$snc), verticals = T)
plot(ecdf(l_qm$mod.hat.c), verticals = T, add = T, col = "red")
plot(ecdf(obs_hist$snc), verticals = T, add = T, col = "blue")
plot(ecdf(l_qdm$mhat.c), verticals = T, add = T, col = "orange")


l_qdm <- QDM_snc(o.c = obs_hist$snc, 
                 m.c = mod_hist$snc, 
                 m.p = mod_fut$snc,
                 ratio = qdm_opt_ratio, 
                 trace = qdm_opt_trace)

l_qm <- QM_snc(obs = obs_hist$snc, 
               mod.c = mod_hist$snc, 
               mod.p = mod_fut$snc,
               trace = qm_opt_trace,
               qstep = 0.001)



# compare QDM MBC ---------------------------------------------------------

plot(cor(l_mbc$mhat.c), cor(m.c.orig))

plot(cor(l_mbc$mhat.c), cor(o.c.orig))
plot(cor(m.c.orig), cor(o.c.orig))

plot(l_mbc$mhat.c, l_mbc$m.iter[[1]][["m.c"]])
plot(l_mbc$mhat.c, l_mbc$m.iter[[15]][["m.c"]])
plot(l_mbc$mhat.c, l_mbc$m.iter[[30]][["m.c"]])


plot(colMeans(o.c.orig), colMeans(m.c.orig))
plot(colMeans(o.c.orig), colMeans(l_mbc$mhat.c))
plot(colMeans(o.c.orig), colMeans(l_mbc$m.iter[[1]][["m.c"]]))
plot(colMeans(l_mbc$m.iter[[1]][["m.c"]]), colMeans(l_mbc$m.iter[[15]][["m.c"]]))
plot(colMeans(l_mbc$m.iter[[1]][["m.c"]]), colMeans(l_mbc$m.iter[[50]][["m.c"]]))
plot(colMeans(l_mbc$mhat.c), colMeans(l_mbc$m.iter[[50]][["m.c"]]))




# compare backup ŕun ------------------------------------------------------

dat_test <- merge(
  clim_mbc[month == 1 & rcm == "CLMcom-CCLM4-8-17",
           .(icell, snc, fp,
             snc_mbc = snc_qdm, snc_mbc_q10 = snc_qdm_q10, snc_mbc_q90 = snc_qdm_q90)],
  clim_qdm[month == 1 & rcm == "CLMcom-CCLM4-8-17",
           .(icell, snc, fp,
             snc_qdm, snc_qdm_q10, snc_qdm_q90)]
)

dat_test[, snc_mbc - snc_qdm] %>% summary

dat_test %>% 
  ggplot(aes(snc_mbc - snc_qdm, colour = fp))+
  geom_freqpoly()

# -> differences low: abs < 0.05


dat_test2 <- merge(
  clim_mbc[,
           .(icell, snc, fp, month, rcm, gcm,
             snc_mbc = snc_qdm, snc_mbc_q10 = snc_qdm_q10, snc_mbc_q90 = snc_qdm_q90)],
  clim_qdm[,
           .(icell, snc, fp, month, rcm, gcm,
             snc_qdm, snc_qdm_q10, snc_qdm_q90)]
)

dat_test2[, snc_mbc - snc_qdm] %>% summary
dat_test2 %>% 
  ggplot(aes(snc_mbc - snc_qdm, colour = fp))+
  geom_freqpoly(bins = 100)

dat_test2[, abs(snc_mbc - snc_qdm)] %>% 
  quantile(probs = c(0, 0.25, 0.5, 0.75, 0.9, 0.99, 0.999, 1)) %>% 
  sprintf("q=%s: %0.4f", names(.), .)

# low differences, average 0,
# absolute 99% < 0.02




# amundsen modis ----------------------------------------------------------

rr_modis <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/modis_10d/modis_snc_55x133_2000-03-07.tif")
plot(rr_modis)
rr_modis[c(2590, 2591)] <- 100




# check if Flo's data is fine ---------------------------------------------



library(raster)
library(eurocordexr)

rr_alt <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/amundsen-flo/02-test2/altitude.tif")
rr_alt
plot(rr_alt)

rr_cov <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/amundsen-flo/02-test2/coverage.tif")
rr_cov
plot(rr_cov)

icell_sub <- which(!is.na(rr_alt[]))

nc_open("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/amundsen-flo/02-test2/CNRM-CERFACS-CNRM-CM5_CLMcom-CCLM4-8-17_rcp85_threshold5.nc")

dat_swe <- nc_grid_to_dt("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/amundsen-flo/02-test2/CNRM-CERFACS-CNRM-CM5_CLMcom-CCLM4-8-17_rcp85_threshold5.nc",
                         "swe")
dat_swe_sub <- dat_swe[icell %in% icell_sub]
dat_swe2 <- dat_swe[!is.na(swe)]
summary(dat_swe2)

dat_scf <- nc_grid_to_dt("data-aux/amundsen-flo/CNRM-CERFACS-CNRM-CM5_SMHI-RCA4_rcp85_threshold5.nc",
                         "scf")
dat_scf2 <- dat_scf[!is.na(scf)]
dat_scf2 %>% summary




# update euro-cordex data -------------------------------------------------


library(eurocordexr)

dat_inv <- get_inventory("/mnt/CEPH_BASEDATA/METEO/SCENARIO", add_files = F)

dat_inv_tp <- dat_inv[variable %in% c("tas", "tasmin", "tasmax", "pr")]

dat_inv_tp[, 
           .N,
           .(institute_rcm)]

dat_inv_tp[institute_rcm == "MPI-CSC-REMO2009", 
           .N,
           .(experiment)]

dat_inv_tp[institute_rcm == "GERICS-REMO2015", 
           .N,
           .(gcm)]

dat_inv_tp[institute_rcm == "MPI-CSC-REMO2009", 
           .N,
           .(ensemble)]

dat_inv_tp[institute_rcm == "SMHI-RCA4", 
           .N,
           .(downscale_realisation)]

dat_inv_tp[institute_rcm == "KNMI-RACMO22E" & experiment %in% c("historical", "rcp26", "rcp85"), 
           .N,
           .(ensemble)]


dat_inv_tp[institute_rcm == "KNMI-RACMO22E" & experiment %in% c("historical", "rcp26", "rcp85") &
             ensemble %in% c("r1i1p1","r3i1p1"), 
           .N,
           .(downscale_realisation)]


dat_inv_tp[institute_rcm == "MPI-CSC-REMO2009"]


dat_inv_snow <- dat_inv[variable %in% c("snc", "snd", "snw")]

dat_inv_snow[,
             .N,
             institute_rcm]


dat_inv_snow[institute_rcm == "DMI-HIRHAM5", 
           .N,
           .(experiment)]

dat_inv_snow[institute_rcm == "DMI-HIRHAM5", 
           .N,
           .(gcm)]



# check cropping of snow --------------------------------------------------

dt1_base <- get_inventory("/mnt/CEPH_BASEDATA/METEO/SCENARIO/")
dt1_base <- dt1_base[variable %in% c("snc", "snd", "snw")]

dt2_wget <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/wget-snow/")

dt3_crop <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/EUROCORDEX-ALPS/")

dt_old <- unique(rbind(dt1_base, dt2_wget))

dt_test <-dt_old[, .(.N), 
       .(variable, gcm, institute_rcm, experiment, ensemble, downscale_realisation)]

dt_test[N > 1] %>% 
  merge(dt_old)


dt3_crop

check_inventory(dt3_crop)


dt1_base <- get_inventory("/mnt/CEPH_BASEDATA/METEO/SCENARIO/")




# clean pr tas rcm data ---------------------------------------------------


dt1 <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/wget-tp/")

dt1 <- get_inventory("/mnt/CEPH_PROJECTS/SCENARIO/CLIMATEDATA/")
dt1_orog <- dt1[variable == "orog"]
dt1_tp <- dt1[variable != "orog"]
check_inventory(dt1_tp)
check_inventory(dt1_tp, check_vars = T)
# compare_variables_in_inventory(dt1_tp)


fwrite(dt1_tp, "/mnt/CEPH_PROJECTS/SCENARIO/zz_clean_rcms/20210528T084700_inventory.csv")

capture.output({check_inventory(dt1_tp, check_vars = T)},
               file = "/mnt/CEPH_PROJECTS/SCENARIO/zz_clean_rcms/20210528T084700_inventory_check.txt")


dt1_tp[, .N, experiment]
dt1_tp[, .N, institute_rcm]
dt1_tp[, .N, gcm]

dt1_tp[, .N, .(institute_rcm, gcm)]
dt1_tp[experiment != "evaluation", .N, .(institute_rcm, gcm)]


with(dt1_tp, table(experiment, total_simulation_years))

# check ICHEC and MPI multiple ensemble
dt1_tp[gcm == "ICHEC-EC-EARTH", .N, ensemble]
dt1_tp[gcm == "MPI-M-MPI-ESM-LR", .N, ensemble]


# remove some
dt1_files <- get_inventory("/mnt/CEPH_PROJECTS/SCENARIO/CLIMATEDATA/", add_files = T)
dt1_files <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/wget-tp/", add_files = T)

dt1_tp[institute_rcm == "DMI-HIRHAM5" & downscale_realisation == "v2" & gcm == "NCC-NorESM1-M"] %>% 
  merge(dt1_files) -> dt1_del
dt1_del$list_files %>% 
  unlist %>% 
  fs::file_delete()



dt1[institute_rcm == "DMI-HIRHAM5" & gcm == "NCC-NorESM1-M"] %>% 
  merge(dt1_files) -> dt1_mv
dt1_mv$list_files %>% 
  unlist %>% 
  fs::file_move("/mnt/CEPH_PROJECTS/CLIRSNOW/wget-tp/temp/")


# check rcm coverage
dt1_tp[, .N, institute_rcm]

dt1_tp[institute_rcm == "IPSL-WRF381P", .N, gcm]
dt1_tp[institute_rcm == "KNMI-RACMO22E", .N, experiment]
dt1_tp[institute_rcm == "SMHI-RCA4" & downscale_realisation == "v2", .N, gcm]
dt1_tp[institute_rcm == "SMHI-RCA4" & gcm == "NCC-NorESM1-M", .N, ensemble]
dt1_tp[institute_rcm == "ICTP-RegCM4-6" & gcm == "MPI-M-MPI-ESM-LR"]
dt1_tp[institute_rcm == "GERICS-REMO2015" & gcm == "NCC-NorESM1-M" & ensemble == "r3i1p1"]
dt2 <- dt1_tp[institute_rcm == "SMHI-RCA4"]
dt1_tp[institute_rcm == "DMI-HIRHAM5", .N, downscale_realisation]




# check bc runs -----------------------------------------------------------

dt1 <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/02_qm/CLMcom-CCLM4-8-17_ICHEC-EC-EARTH_rcp26_r12i1p1_v1_future.rds")



# check python cp ---------------------------------------------------------


rr1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/modis_10d/modis_snc_55x133_2000-03-10.tif")
rr2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/temp/modis_snc_55x133_2000-03-10.tif")
rr1
rr2
rr_hr
plot(rr1)
plot(rr2)

rr2[rr2 == 0] <- NA


r_cp <- brick("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp_10bins.tif")
r_cp
plot(r_cp)

r_cp <- brick("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp_20bins.tif")
r_cp
plot(r_cp)

# common icell in BC ------------------------------------------------------

library(raster)
rr0 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/modis_10d/modis_snc_55x133_2000-02-24.tif")

plot(rr0)

rr1 <- rr0
rr1[icell_modis] <- 99
plot(rr1)


dat_aux$alt %>% qplot
dat_aux[icell %in% icell_common_rcm, alt] %>% qplot
dat_test <- rbind(
  dat_aux[, .(icell, alt, ff = "all")],
  dat_aux[icell %in% icell_common_rcm, .(icell, alt, ff = "common rcm")]
)

dat_test %>% 
  ggplot(aes(alt, colour = ff))+
  geom_freqpoly(pad = F)

dat_test %>% 
  ggplot(aes(alt))+
  geom_histogram()+
  facet_wrap(~ff)


dat_th %>% 
  merge(dat_aux) %>% 
  ggplot(aes(x,y,fill = snow_plausible))+
  geom_raster()+
  facet_wrap(~rcm_short)


clim_mbc[rcm == "KNMI-RACMO22E" & gcm == "NCC-NorESM1-M"]
clim_mbc[rcm == "KNMI-RACMO22E" & gcm == "NCC-NorESM1-M", sort(unique(icell))] -> zz_icell

dat_th[rcm == "KNMI-RACMO22E", .N, snow_plausible]
dat_th[rcm == "KNMI-RACMO22E", icell] %>% 
  intersect(icell_modis) %>% str

# check differences avg SNC mbc -------------------------------------------

dat_full[rcm == "SMHI-RCA4" & gcm == "MPI-M-MPI-ESM-LR"]
clim_mbc

dat1 <- merge(
  clim_mbc[icell %in% icell_common & 
             month == 1 & fp == "past" &
             rcm == "SMHI-RCA4" & gcm == "MPI-M-MPI-ESM-LR",
           .(icell, month, snc_mbc = snc_qdm)],
  clim_qdm[icell %in% icell_common & 
             month == 1 & fp == "past" &
             rcm == "SMHI-RCA4" & gcm == "MPI-M-MPI-ESM-LR",
           .(icell, month, snc_qdm)]
)

dat1[, qplot(snc_mbc - snc_qdm)]

dat1 %>% 
  merge(dat_aux) %>% 
  ggplot(aes(x,y,fill = snc_mbc - snc_qdm))+
  geom_raster()+
  scale_fill_gradient2()




# runtime ds --------------------------------------------------------------

# space needed GB / qm,qdm,bmc
27*50*12*38/1024

# runtime days / qm,qdm,bmc
40*50*12*38/3600/24
60*50*12*38/3600/24

# if 14h per 80y then:
50*38/80*14/24


# 20 cp
90*50*12*38/3600/24

# 20cp: 22.5h per 80y
50*38/80*22.5/24

# 20cp per 50y group
22.5/80*50

# total runtime all qm/qdm/mbc per day
(22.5/80*50) * 38*3 / 24

# with 6-7 cores
67/6
67/7


# check ds ----------------------------------------------------------------

rr3 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future/2071-01.tif")
plot(rr3)

rr <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds/test1/2071-04.tif")
rr2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds/test1/2072-04.tif")

rr <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future/2071-04.tif")
rr2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future/2072-04.tif")

rr
plot(rr)
plot(rr2)

plot(is.na(rr3))
plot(rr3 < 0)



rr <- brick("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-rofental/test-cp.tif")
rr
plot(rr)




rr_rof1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-rofental/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future/2071-02-21.tif")
rr_rof1
plot(rr_rof1)

l_fn_rof <- paste0("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-rofental/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future/2071-0",
                   1:9,
                   "-21.tif")
rr_rof2 <- stack(l_fn_rof)
rr_rof2
plot(rr_rof2)

# check ds missing pixels ----------------------------------------------------

rr_tmp <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test-ds-cp20/KNMI-RACMO22E_ICHEC-EC-EARTH_rcp85_r3i1p1_v1_past/2000-01.tif")

plot(rr_tmp)
plot(rr_tmp < 0)




# temp fill scf th --------------------------------------------------------


rr_lr <- raster(rr_alt_lr)
rr_lr[dat_scf_missing_lr$icell_lr] <- dat_scf_missing_lr$n_na
plot(rr_lr)

rr_hr <- raster(rr_alt)
rr_hr[dat_scf_eval$icell] <- dat_scf_eval$alt
plot(rr_hr)


# runtime scf th imputation

# first 12 lr cells: 300s, ~*420 #icell_lr, ~1240 #icell_hr
# first 24 lr cells: 882s, ~*210 #icell_lr, ~340 #icell_hr

# hr cell number 12: 2800 / 24: 10300

300*420/3600
300*1240/3600

882*210/3600
882*340/3600



# last scf_th imputation
190*210/3600/6
190*340/3600/6



# check cropping rofental -------------------------------------------------

m0 <- matrix(1:12, 3, 4)
m0
m1 <- matrix(0, 3, 4)
m1
m1 + (m0 > 5)


xy_rofental <- rowColFromCell(rr_scf_th, cells_rofental)
xy_rofental %>% sapply %>% range
xy_rofental_range <- apply(xy_rofental, 2, function(x) sort(unique(x)))
# vec_scd <- as.vector(mat_scd)[cells_rofental]
mat_rofental <- mat_scd[xy_rofental_range$row, xy_rofental_range$col]

rr_out_rofental <- raster(rr_rofental)
rr_out_rofental[] <- mat_rofental
plot(rr_out_rofental, zlim = c(15,31))
plot(is.na(rr_out_rofental))

plot(is.na(rr_scf_th))
plot(is.na(rr_out))

mapview(rr_out)+mapview(rr_out_rofental)



# test r downscale --------------------------------------------------------


rr1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test_r/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future/2071-01.tif")
plot(rr1)

rr2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/test_r/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future/2071-07.tif")
plot(rr2)

rr_modis <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD_monthly/hydro_year_2001_2019/07_Jul.tif")
plot(rr_modis)

rr2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future/2071-01-01.tif")
plot(rr2)


# runtime r downscaler ----------------------------------------------------


# space needed GB / for all qm,qdm,bmc
50*50*38*3/1024


# runtime days / for all qm,qdm,bmc
30*50*12*38*3/3600/24
# multi-core
30*50*12*38*3/3600/24 / 12

# RAM needs (~5 GB per core)


# check calc impute -------------------------------------------------------

dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_impute_similar_icell/") %>% 
  lapply(readRDS) %>% 
  rbindlist -> dat_th

dat_th




# rofental ----------------------------------------------------------------

zz <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future.rds")
zz2 <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.rds")


zz <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds_comp_amundsen/elev/03_qdm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future.rds")
zz


zz2 <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_rofental_rds_comp_amundsen/month/03_qdm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future.rds")
zz2


rs_elev <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rs_elev[rs_elev < -30000] <- NA

rs_water <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rs_elev[rs_water == 1] <- NA

rs_modis <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")
rs_modis_rof <- st_crop(rs_modis, rs_template_ds)


rs_rof_past2

rs_modis_repro <- st_warp(rs_modis_rof, rs_rof_past2)
rs_elev_repro <- st_warp(rs_elev, rs_rof_past2)

dat_check <- data.table(
  scd_modis = rs_modis_repro[[1]] %>% as.vector,
  scd_amundsen = rs_rof_past2[[1]] %>% as.vector,
  elev = rs_elev_repro[[1]] %>% as.vector
)

dat_check %>% 
  ggplot(aes(scd_modis, scd_amundsen))+
  geom_bin2d()+
  geom_abline()+
  scale_fill_viridis_c(trans = "log10")



dat_check %>% 
  ggplot(aes(elev, scd_amundsen))+
  geom_bin2d()+
  geom_abline()+
  scale_fill_viridis_c(trans = "log10")



dat_check %>% 
  ggplot(aes(elev, scd_amundsen - scd_modis))+
  geom_bin2d()+
  geom_hline(yintercept = 0)+
  scale_fill_viridis_c(trans = "log10")

dat_check[, elev_f := cut(elev, breaks = seq(500, 4000, by = 500), dig.lab = 5)]

dat_check[!is.na(elev_f), 
          .(bias = mean(scd_amundsen - scd_modis, na.rm = T),
            mae = mean(abs(scd_amundsen - scd_modis), na.rm = T),
            nn = .N)]

dat_check[!is.na(elev_f), 
          .(bias = mean(scd_amundsen - scd_modis, na.rm = T),
            mae = mean(abs(scd_amundsen - scd_modis), na.rm = T),
            nn = .N),
          keyby = .(elev_f)]

ggplot()+
  geom_stars(data = rs_rof_past2 - rs_modis_repro)+
  scale_fill_gradient2("")



# lr pixel removed and glacier extent --------------------------------------


rr_rgi_lr <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/rgi/modis-lr.tif")

plot(rr_rgi_lr > 0.1)



# test downscale change anomalies -----------------------------------------

rs1 <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp85_dscd_elev_anom_noglacier.tif")


ggplot()+
  geom_stars(data = rs1, downsample = 10)+
  scale_fill_gradient2()



# rel change
rs_modis <- read_stars("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")
rs_rcp26 <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp26.tif")
rs_rcp85 <- read_stars("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp85.tif")

rs_rcp26_rel <- (rs_rcp26 - rs_modis)/rs_modis
rs_rcp85_rel <- (rs_rcp85 - rs_modis)/rs_modis

# remove low scd
rs_rcp85_rel[rs_modis <= 5] <- NA

plot(rs_rcp85_rel > 1)
plot(rs_modis <= 5)

range(rs_rcp85_rel[[1]], na.rm = T)

ggplot()+
  geom_stars(data = rs_rcp26_rel, downsample = 10)+
  scale_fill_gradient2(limits = c(-1,1))


ggplot()+
  geom_stars(data = rs_rcp85_rel, downsample = 10)+
  scale_fill_gradient2(limits = c(-1,1))


rr_modis <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")
rr_ens_scd <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp85_noglacier.tif")
rr_ens_scd %>% plot
plot(rr_ens_scd - rr_modis)

summary(rr_ens_scd[] - rr_modis[])



# compare bc results 2041 2071 --------------------------------------------


dat1 <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/03_qdm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.rds")
dat2 <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full_2041_2070/03_qdm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.rds")


snc_d <- dat1$snc_qdm - dat2$snc_qdm
summary(snc_d)
hist(snc_d)


dat2[, snc_qdm_d := snc_qdm - dat1$snc_qdm]

dat2 %>% 
  ggplot(aes(snc, snc_qdm_d))+
  geom_bin2d()+
  scale_fill_viridis_c(trans = "log10")



# -> differences for random components

# compare clim of bc!

# clim scd

rr1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_2041_2070/rcp85_noglacier.tif")
rr2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp85_noglacier.tif")


rr1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean_2041_2070/rcp26_noglacier.tif")
rr2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scd_ensemble_mean/rcp26_noglacier.tif")

plot(stack(rr1, rr2), zlim = c(0,366))
plot(rr2 - rr1)



# upscaled modis eval ds -----------------------------------------------------------


files <- dir_ls("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/alps-daily/")
xx12 <- 1:12 - 1

rr1 <- stack(files[40 + xx12])
plot(rr1)



dat_month <- dat_summ[, .(n_true = sum(N[(value_orig == 1 & value_ds == 1) | 
                                           (value_orig == 2 & value_ds == 0)]),
                          n_false = sum(N[!{(value_orig == 1 & value_ds == 1) | 
                              (value_orig == 2 & value_ds == 0)}])), month(date)]

dat_month[, frac_true := n_true / (n_true + n_false)]
dat_month



# seasonal CP -------------------------------------------------------------

rr_cp <- brick("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/cp_seasonal/cp_20bins_SONDJF.tif")
plot(rr_cp)



# agg modis upscaled downscaled -------------------------------------------


dat_modis_n <- dat_modis[, .N, icell]
dat_modis_n2 <- dat_modis[!is.na(snc), .N, icell]
dat_modis_n3 <- dat_modis[!is.na(snc), .N, date]

sum(dat_modis_n2$N) / (nrow(dat_modis_n2) * max(dat_modis_n2$N))
sum(dat_modis_n3$N) / (nrow(dat_modis_n3) * max(dat_modis_n3$N))


dat1 <- dat_modis[date == "2001-09-15"]


rr_clim <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/alps-clim/2000-10-01_2019-09-30.tif")
rr_clim %>% plot

rr1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/alps-monthly/2001-09.tif")
# rr1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/alps-monthly/2020-08.tif")
plot(rr1)

rr2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/alps-daily/2001-09-04.tif")
plot(rr2)

rr3 <- stack(paste0("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/alps-daily/2001-09-0", 1:9, ".tif"))
plot(rr3)


rr3 <- stack(paste0("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/alps-daily/2001-09-1", 0:9, ".tif"))
plot(rr3)


rr_up <- brick("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/modis_10d/modis_snc_55x133_2001-09-15.tif")
plot(rr_up)


rr_up2 <- raster(rr_up)
rr_up2[dat1$icell] <- dat1$snc
plot(rr_up2)


# seasonal DS -------------------------------------------------------------

rr_past <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim_seasonal/DJF/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.tif")
rr_future <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/bc_full_clim_seasonal/DJF/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future.tif")
plot(stack(rr_past, rr_future))




# scale issues ------------------------------------------------------------

rr_modis
rr_modis_lr %>% plot

mat_lr <- raster::as.matrix(rr_modis_lr)
mat_lr2hr <- mat_lr[rep(1:55, each = 53),
                    rep(1:133, each = 37)]
mat_lr2hr <- mat_lr2hr[1:2867, 1:4901]
rr_modis_lr2hr <- raster(rr_modis)
rr_modis_lr2hr[] <- mat_lr2hr

plot(rr_modis_lr2hr - rr_modis)
plot((rr_modis_lr2hr - rr_modis)/rr_modis)
mapview::mapview(rr_modis_lr2hr - rr_modis)


rr_diff_rel <- (rr_modis_lr2hr - rr_modis)/rr_modis
plot(rr_diff_rel < 0)



# past bias ---------------------------------------------------------------

load("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/clim-02.rda")

clim_qdm_icell_month <- clim_qdm[icell %in% icell_common & fp == "past" & rcm %in% rcms_common,
                                 .(snc = snc_qdm, icell, month, rcm, gcm, experiment, fp)] %>% 
  merge(clim_modis[, .(icell, month, snc_modis = snc)])


clim_qdm_icell_month[, snc_diff := snc - snc_modis]
clim_qdm_icell_month[, snc_diff_rel := (snc - snc_modis)/snc_modis]

clim_qdm_icell_month[is.na(snc_diff_rel)]

clim_qdm_icell_month[experiment == "rcp26", qplot(snc - snc_modis)]
clim_qdm_icell_month[experiment == "rcp26", qplot((snc - snc_modis)/snc_modis)]
clim_qdm_icell_month[experiment == "rcp26"] %>% 
  ggplot(aes(snc-snc_modis, color = factor(month)))+
  geom_freqpoly()+
  facet_wrap(~month)

clim_qdm_icell_month[experiment == "rcp26"] %>% 
  merge(dat_aux) %>% 
  ggplot(aes(x,y,fill = (snc - snc_modis)/snc_modis))+
  geom_raster()+
  scale_fill_gradient2()+
  # theme_bw()+
  facet_grid(rcm + gcm ~ month)


dat_bias_summary <- clim_qdm_icell_month[,
                                        .(bias = mean(snc_diff),
                                          mae = mean(abs(snc_diff))),
                                        .(month, rcm, gcm, experiment)]
dat_bias_summary %>% summary
dat_bias_summary %>% 
  dcast(month+rcm+ gcm ~ experiment, value.var = "bias") %>% 
  na.omit




# check modis clim lr -----------------------------------------------------


dat_modis <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/modis-10d-02-snc.rds")
clim_modis2 <- dat_modis[date >= "2000-10-01" & date <= "2019-09-30", 
                        .(snc = mean(snc, na.rm = T)), 
                        .(icell, month(date))]
clim_modis3 <- dat_modis[date >= "2000-08-01" & date <= "2020-07-31", 
                         .(snc = mean(snc, na.rm = T)), 
                         .(icell, month(date))]

# clim_modis2[, .(icell, month, snc_hydro = snc)] %>% 
clim_modis3[, .(icell, month, snc_hydro = snc)] %>% 
  merge(clim_modis[, .(icell, month, snc)]) -> dat_comp_modis_clim

dat_comp_modis_clim[, snc_diff := snc - snc_hydro]
dat_comp_modis_clim[, snc_diff_rel := (snc - snc_hydro)/snc_hydro]
summary(dat_comp_modis_clim)

dat_comp_modis_clim %>% 
  ggplot(aes(snc_diff))+
  geom_histogram()+
  facet_wrap(~month)




# any bias from not using hydro years?! -----------------------------------


dat_rcm1 <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/03_qdm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.rds")
dat_rcm1_avg <- dat_rcm1[, .(snc = mean(snc_qdm)), .(icell, month)]
dat_comp1 <- dat_rcm1_avg %>% 
  merge(clim_modis[, .(icell, month, snc_modis = snc)])


dat_comp1[, snc_diff := snc - snc_modis]
dat_comp1[, snc_diff_rel := (snc - snc_modis)/snc_modis]
summary(dat_comp1)

dat_comp1 %>% 
  merge(dat_aux) %>% 
  ggplot(aes(x,y,fill = snc_diff))+
  geom_raster()+
  scale_fill_gradient2()+
  facet_wrap(~month)

dat_comp1[snc_diff_rel < 1] %>% 
  merge(dat_aux) %>% 
  ggplot(aes(x,y,fill = snc_diff_rel))+
  geom_raster()+
  scale_fill_gradient2()+
  facet_wrap(~month)


# re-run hydro ------------------------------------------------------------

# dc
dat1 <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/_bc_full/01_delta_change/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1.rds")
dat2 <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/01_delta_change/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1.rds")


plot(dat1$delta_snc_mult, dat2$delta_snc_mult)
qplot(dat1$delta_snc_mult - dat2$delta_snc_mult)
quantile(dat1$delta_snc_mult - dat2$delta_snc_mult, 0:10/10, na.rm = T)

# future qm
dat1 <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/_bc_full/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future.rds")
dat2 <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_future.rds")

quantile(dat1$snc_qm - dat2$snc_qm, 0:10/10)

# past qm
dat1 <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/_bc_full/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.rds")
dat2 <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/02_qm/CLMcom-CCLM4-8-17_CNRM-CERFACS-CNRM-CM5_rcp85_r1i1p1_v1_past.rds")

cbind(quantile(dat1$snc_qm, 0:20/20),
      quantile(dat2$snc_qm, 0:20/20))



rr1 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/zz_backup/scf_threshold/scf_cp50_v02_monotonic.tif")
rr2 <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/scf_threshold/scf_cp50_v02_monotonic.tif")
plot(rr1)
plot(rr2)
plot(is.na(rr2) & is.na(rr1))


# amundsen v1 v2 elev month  ----------------------------------------------------


dir_ls("data-aux/amundsen-flo-hr/", regexp = "elev") %>% 
  lapply(function(x){
    
    path_file(x) %>% 
      path_ext_remove() %>% 
      stringr::str_split("_", simplify = T) -> x_split
    
    dat <- fread(x) %>% 
      melt(id.vars = "elev", value.name = "scf")
    
    dat[, c("gcm", "institute_rcm") := tstrsplit(variable, "_")]
    
    dat[, .(gcm, institute_rcm, 
            experiment = x_split[1,4], period = x_split[1,5], 
            elev, scf)]
    
    
  }) %>% rbindlist() -> dat_am1
dat_am1[, fp := ifelse(period == "2000-2020", "past", "future")]


dir_ls("data-aux/amundsen-flo-hr2/", regexp = "elev") %>% 
  lapply(function(x){
    
    path_file(x) %>% 
      path_ext_remove() %>% 
      stringr::str_split("_", simplify = T) -> x_split
    
    dat <- fread(x) %>% 
      melt(id.vars = "elev", value.name = "scf", 
           variable.name = "month", variable.factor = F)
    dat[, month := as.integer(month)]
    

    dat[, .(month, 
            gcm = x_split[1,6], 
            institute_rcm = x_split[1,7], 
            experiment = x_split[1,8], 
            period = x_split[1,9], 
            elev, 
            scf)]
    
    
  }) %>% rbindlist() -> dat_am2
dat_am2[, fp := ifelse(period == "2000-2020", "past", "future")]


dat_am2[, .(scf_fm = mean(scf)),
        .(gcm, institute_rcm, experiment, fp, elev)] %>% 
  merge(dat_am1) -> dat_zz

dat_zz %>% with(plot(scf, scf_fm))
dat_zz[, qplot(scf - scf_fm)]


# geometric mean ----------------------------------------------------------




geom_mean <- function(x){
  exp(mean(log(x)))
}

geom_wmean <- function(x, w){
  exp(weighted.mean(log(x), w))
}

dat_plot_diff[1:2, exp(sum(log(1+snc_diff_rel))/2)]
dat_plot_diff[1:2, 1+snc_diff_rel] %>% prod %>% sqrt
dat_plot_diff[1:2, 1+snc_diff_rel] %>% mean
dat_plot_diff[1:2, geom_mean(1+snc_diff_rel)]
dat_plot_diff[, geom_mean(1+snc_diff_rel)]
dat_plot_diff[, mean(snc_diff_rel)]



# EOF ---------------------------------------------------------------------


